/* recA128C.c - record support routines for the ALICE 128 C chip (STAR/SSD) */
/*
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author:		David Bonnet
 *	Date:		03/04/00
 */

#include	<vxWorks.h>
#include 	<taskLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include        <stdlib.h>
#include	<lstLib.h>
#include	<string.h>

#include	<alarm.h>
#include 	<special.h>
#include	<dbDefs.h>
#include	<dbAccess.h>
#include	<dbScan.h>
#include	<dbFldTypes.h>
#include	<devSup.h>
#include	<errMdef.h>
#include	<recSup.h>
#include 	<jtagCommon.h>
#include 	<jtagA128C.h>
#include 	<jtagCostar.h>
#include 	<jtagConnexion.h>
#define GEN_SIZE_OFFSET
#include 	<A128CRecord.h>
#undef GEN_SIZE_OFFSET

/* Create RSET - Record Support Entry Table*/
#define report NULL
static long initialize();
static long init_record();
static long process();
/*static long special();*/
#define special NULL
static long get_value(); 
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
#define get_precision NULL
static long get_enum_str();
static long get_enum_strs();
static long put_enum_str();
#define get_graphic_double NULL
#define get_control_double NULL
#define get_alarm_double NULL


struct rset A128CRSET={
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double};

int    async_A128C(A128CRecord *);
static long A128C_io();
static void monitor();
static void alarm();

static char INIT_JTAG[LADDER_NB];
SEM_ID sem_jtag;
chip   map_list[LADDER_NB][MAX_CHIPS];
static unsigned short vh=0;
extern costar costar_list[LADDER_NB][MODULE_NB];		 /* Coming from recCostar.c */
extern unsigned char costar_list_old_vref[LADDER_NB][MODULE_NB]; /* Coming from recCostar.c */
extern unsigned short module_mux[LADDER_NB][MODULE_NB];          /* Coming from recCostar.c */
static long initialize()
{
	int i;
		
	sem_jtag=semBCreate(SEM_Q_PRIORITY, SEM_FULL);
	for (i=0;i<LADDER_NB;i++) INIT_JTAG[i]=0;
	writea16(PIO2_OUT,1);   
	if (!reset_jtag_var(1,1,5))
      	   printf("reset JTAG : OK\n");
   	else 
      	   printf(" ERROR : JTAG Board not present.\n");
	writea16(PIO2_OUT,1);   
	return 0;
}

static long init_record(A128CRecord *pA128C,int pass)
{
/*     long status; */
	
/* 	if (pass!=0)
	{
	status = recGblInitFastInLink(&(pA128C->outi), (void *) pA128C, DBR_SHORT, "JINI");
        if (status)
           return(status);
	}*/
	return 0;
}

static long process(A128CRecord *pA128C)
{
	long status;
	unsigned char pact=pA128C->pact;
	
	status=A128C_io(pA128C); /* Do A128C I/O operation */
	if(!pact && pA128C->pact) return 0;
	
	pA128C->pact=TRUE;
	
	alarm(pA128C);
	monitor(pA128C);
	recGblGetTimeStamp(pA128C);
	recGblFwdLink(pA128C);

	pA128C->pact=FALSE;
	return(status);
}


static long get_value(A128CRecord *pA128C,struct valueDes *pvdes)
{
    pvdes->field_type = DBF_ENUM;
    pvdes->no_elements=1;
    pvdes->pvalue = &pA128C->val;
    return(0);
}

static long get_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char	  *pstring;
{
    struct A128CRecord	*pA128C=(struct A128CRecord *)paddr->precord;

    if(paddr->pfield==(void *)&pA128C->val)
    {
       if(pA128C->val==0) {
	  strcpy(pstring,"NOP");
       } else if(pA128C->val==1) {
	  strcpy(pstring,"INIT");
       } else if(pA128C->val==2) {
	  strcpy(pstring,"BIAS");
       } else if(pA128C->val==3) {
	  strcpy(pstring,"RESET");
       } else if(pA128C->val==4) {
	  strcpy(pstring,"TEST");
       } else if(pA128C->val==5) {
	  strcpy(pstring,"TRANSP");
       } else if(pA128C->val==6) {
	  strcpy(pstring,"LATCHUP");
       } else if(pA128C->val==7) {
	  strcpy(pstring,"COSTAR");
       } else if(pA128C->val==8) {
	  strcpy(pstring,"RECOVER");
       } else if(pA128C->val==9) {
	  strcpy(pstring,"POWON");
       } else if(pA128C->val==10) {
	  strcpy(pstring,"POWOFF");
       } else {
	  strcpy(pstring,"Illegal_Value");
       }
    }
    else if(paddr->pfield==(void *)&pA128C->spac)
    {
       if(pA128C->spac==0) {
	  strcpy(pstring,"No Pulse");
       } else if(pA128C->spac==1) {
	  strcpy(pstring,"1 per ladder");
       } else if(pA128C->spac==2) {
	  strcpy(pstring,"1 per chip");
       } else if(pA128C->spac==3) {
	  strcpy(pstring,"2 per chip");
       } else if(pA128C->spac==4) {
	  strcpy(pstring,"4 per chip");
       } else if(pA128C->spac==5) {
	  strcpy(pstring,"8 per chip");
       } else if(pA128C->spac==6) {
	  strcpy(pstring,"Patern");
       } else {
	  strcpy(pstring,"Illegal_Value");
       }
    }
    return(0);
}

static long get_enum_strs(paddr,pes)
    struct dbAddr *paddr;
    struct dbr_enumStrs *pes;
{
    struct A128CRecord	*pA128C=(struct A128CRecord *)paddr->precord;

    if(paddr->pfield==(void *)&pA128C->val)
    {
       pes->no_str = 11;
       memset(pes->strs,'\0',sizeof(pes->strs));
       strcpy(pes->strs[0],"NOP");
       strcpy(pes->strs[1],"INIT");   
       strcpy(pes->strs[2],"BIAS");   
       strcpy(pes->strs[3],"RESET");   
       strcpy(pes->strs[4],"TEST");   
       strcpy(pes->strs[5],"TRANSP");   
       strcpy(pes->strs[6],"LATCHUP");   
       strcpy(pes->strs[7],"COSTAR");   
       strcpy(pes->strs[8],"RECOVER");   
       strcpy(pes->strs[9],"POWON");   
       strcpy(pes->strs[10],"POWOFF");   
    }
    else if(paddr->pfield==(void *)&pA128C->spac)
    {
       pes->no_str = 7;
       memset(pes->strs,'\0',sizeof(pes->strs));
       strcpy(pes->strs[0],"No Pulse");
       strcpy(pes->strs[1],"1 per ladder");   
       strcpy(pes->strs[2],"1 per chip");   
       strcpy(pes->strs[3],"2 per chip");   
       strcpy(pes->strs[4],"4 per chip");   
       strcpy(pes->strs[5],"8 per chip");   
       strcpy(pes->strs[6],"Patern");   
    
    }
    return(0);   
}

static long put_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char          *pstring;
{
    struct A128CRecord     *pA128C=(struct A128CRecord *)paddr->precord;

    if(paddr->pfield==(void *)&pA128C->val)
    {
       if(strcmp(pstring,"NOP")==0) pA128C->val = 0;
       else  if(strcmp(pstring,"INIT")==0) pA128C->val = 1;
       else  if(strcmp(pstring,"BIAS")==0) pA128C->val = 2;
       else  if(strcmp(pstring,"RESET")==0) pA128C->val = 3;
       else  if(strcmp(pstring,"TEST")==0) pA128C->val = 4;
       else  if(strcmp(pstring,"TRANSP")==0) pA128C->val = 5;
       else  if(strcmp(pstring,"LATCHUP")==0) pA128C->val = 6;
       else  if(strcmp(pstring,"COSTAR")==0) pA128C->val = 7;
       else  if(strcmp(pstring,"RECOVER")==0) pA128C->val = 8;
       else  if(strcmp(pstring,"POWON")==0) pA128C->val = 9;
       else  if(strcmp(pstring,"POWOFF")==0) pA128C->val = 10;
       else return(S_db_badChoice);
    }    
    else if(paddr->pfield==(void *)&pA128C->spac)
    {
       if(strcmp(pstring,"No Pulse")==0) pA128C->val = 0;
       else  if(strcmp(pstring,"1 per ladder")==0) pA128C->val = 1;
       else  if(strcmp(pstring,"1 per chip")==0) pA128C->val = 2;
       else  if(strcmp(pstring,"2 per chip")==0) pA128C->val = 3;
       else  if(strcmp(pstring,"4 per chip")==0) pA128C->val = 4;
       else  if(strcmp(pstring,"8 per chip")==0) pA128C->val = 5;
       else  if(strcmp(pstring,"Patern")==0) pA128C->val = 6;
       else return(S_db_badChoice);
    }    
    
    return(0);
}
static void alarm(A128CRecord *pA128C)
{	
	
  /*if((pA128C->udf == TRUE)||(pA128C->jini!=2)||(!pA128C->hyby))
   {
      recGblSetSevr(pA128C,UDF_ALARM,INVALID_ALARM);
      return;
   }*/
	
   if(pA128C->jals&&pA128C->jfhy)
   {
      recGblSetSevr(pA128C,DISABLE_ALARM,pA128C->jals);
      return;
   }	
   
   if(pA128C->bals&&pA128C->bsta)
   {
      recGblSetSevr(pA128C,COMM_ALARM,pA128C->bals);
      return;
   }	

   if(pA128C->lals&&(pA128C->lahy!=0xffff))
   {
      recGblSetSevr(pA128C,HIGH_ALARM,pA128C->lals);
      return;
   }	
   
   return;	
}	

static void monitor(A128CRecord *pA128C)
{
	unsigned short	monitor_mask,mm;
/* 	float		delta; */

        monitor_mask = mm = recGblResetAlarms(pA128C);
	/* check for value change */
	if (INIT_JTAG[pA128C->ladr]!=pA128C->jini) {
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
		/* update last value monitored */
		INIT_JTAG[pA128C->ladr]=pA128C->jini;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pA128C,&pA128C->jini,monitor_mask);
	   db_post_events(pA128C,&pA128C->jfhy,monitor_mask);
	   db_post_events(pA128C,&pA128C->lahy,monitor_mask);
	   db_post_events(pA128C,&pA128C->lsta,monitor_mask);
	   db_post_events(pA128C,&pA128C->cxid,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm0,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm1,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm2,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm3,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm4,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm5,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm6,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm7,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm8,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm9,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm10,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm11,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm12,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm13,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm14,monitor_mask);
	   db_post_events(pA128C,&pA128C->hm15,monitor_mask);
	}
        
	/*monitor_mask = mm;
	if (pA128C->val==3) *//*RESET*//*{*/
		/* post events for value change */
	/*	monitor_mask |= DBE_VALUE;
	}*/

	/* send out monitors connected to the value field */
	/*if (monitor_mask)
	{
	   db_post_events(pA128C,&pA128C->jclk,monitor_mask);
	}*/

	monitor_mask = mm;	
	if ((pA128C->val==6)||vh) /* Latchup*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
		vh=0;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pA128C,&pA128C->lahy,monitor_mask);
	}
	
	monitor_mask = mm;
	if (pA128C->val==2) /*BIAS*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pA128C,&pA128C->bsta,monitor_mask);
	}
	monitor_mask = mm;
	if (pA128C->val==7) /*COSTAR*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pA128C,&pA128C->tmp1,monitor_mask);
	   db_post_events(pA128C,&pA128C->tmp2,monitor_mask);
	   db_post_events(pA128C,&pA128C->tmp3,monitor_mask);
	}
	monitor_mask = mm;
	if ((pA128C->val==9)||(pA128C->val==10)) /*POWER ON or OFF*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pA128C,&pA128C->lahy,monitor_mask);
	}
}

static long A128C_io(A128CRecord *pA128C)
{
   long status;
   
   if(pA128C->pact) 
   {

      status = dbPutLink(&(pA128C->outi),DBR_SHORT , &(pA128C->jini),1);
      if(status) return status; 

       return OK;
   }
   else
   {
      pA128C->pact=TRUE;
      taskSpawn("tA128",ASYN_A128C_TASK_PRIORITY,0,ASYN_A128C_TASK_STACK,&async_A128C,(int)pA128C,0,0,0,0,0,0,0,0,0);
   }  
   return OK;
}

char * get_hybrid_map_ptr(A128CRecord *pA128C,unsigned char hyb)
{
   hyb&=0xf;
   switch(15-hyb)
   {
     case 0: return pA128C->hm0;
     case 1: return pA128C->hm1;
     case 2: return pA128C->hm2;
     case 3: return pA128C->hm3;
     case 4: return pA128C->hm4;
     case 5: return pA128C->hm5;
     case 6: return pA128C->hm6;
     case 7: return pA128C->hm7;
     case 8: return pA128C->hm8;
     case 9: return pA128C->hm9;
     case 10: return pA128C->hm10;
     case 11: return pA128C->hm11;
     case 12: return pA128C->hm12;
     case 13: return pA128C->hm13;
     case 14: return pA128C->hm14;
     case 15: return pA128C->hm15;
   }
   return 0; /* we should never get here */
}

int get_hybrid_map(A128CRecord *pA128C,unsigned char hyb,chip *map)
{
   int i;
   char * hmap;
   
   hmap=get_hybrid_map_ptr(pA128C,hyb);
   for(i=0;(i<7)&&(hmap[i]!=0);i++)
      switch(hmap[i]){
         case 'A' : map[i]=ALICE128C;
	            break;
         case 'C' : map[i]=COSTAR;
	            break;
         default : return 0;
      }
   map[i]=CONNEXION;
   map[i+1]=READOUT;
   map[i+2]=END;
   return 1;
}

void get_ascii_map(chip *map,char *ascii_map)
{
  int i,end=0;
  for(i=0;i<10;i++){
    switch(map[i]){
    case ALICE128C : 
      ascii_map[i]='A';
      break;
    case COSTAR :
      ascii_map[i]='C';
      break;
    default :
      end=1;
      break;
    }
    if(end) break;
  }
  ascii_map[i]=0;
  return;
}

void get_bypass_array(char *command,unsigned char *bypass)
{
  int i,j,n;
  char tmp[5];
  for(i=0;i<NB_MAX_A128C;i++) bypass[i]=1;
  i=0;
  while(command[i]!=0){
    j=0;
    while((command[i]>='0')&&(command[i]<='9')&&(j<4)){
      tmp[j]=command[i];
      i++;j++;
    }
    tmp[j]=0;
    if((j>0)&&((n=atoi(tmp))>0)&&(n<NB_MAX_A128C))
      bypass[n]=0;
    while((command[i]!=0)&&((command[i]<='0')||(command[i]>='9'))) i++;
  }
}
int async_A128C(A128CRecord *pA128C)
{
   struct rset *prset=(struct rset *)(pA128C->rset);
   unsigned char bias[NB_MAX_A128C][7],old_bias[NB_MAX_A128C][7],
                 bypass[NB_MAX_A128C];
   unsigned short test[8],old_test[8],mask,cnx_temp[3];
   int i,j,k,val/* ,nb */,l,valt[7];
   unsigned char dac[4],old_dac[4];
   char name[100],bias_name[16],bias_name_tab[NB_MAX_A128C][32];
   FILE *bias_file,*chip_bias_file,*old_bias_file;
   chip cnx_chip[3]={CONNEXION,READOUT,END};
   chip hybrid_chip[10]={COSTAR,ALICE128C,ALICE128C,ALICE128C,ALICE128C,ALICE128C,ALICE128C,CONNEXION,READOUT,END};
/*    chip hybrid_force[10]={ALICE128C,ALICE128C,ALICE128C,ALICE128C,ALICE128C,ALICE128C,CONNEXION,READOUT,END}; */
   chip hybrid_map[10];
   
   semTake(sem_jtag,-1);	   	/* Protect this process from the others, wait forever */
   connect_tap(pA128C->adrs);
   switch(pA128C->val){
     /* NOP */
   case 0 : break;
     
     /* INIT */
   case 1 : 
     pA128C->jfhy=0;
     if(!pA128C->hyby){
        pA128C->jini=253;
        break; /* if all the hybrids are bypassed do nothing */
     }
     set_bypass_connexion(0,cnx_chip);/* connect the connexion board only */
     if(detect_length()!=2){
       printf("INIT : connexion board not detected\n");
       pA128C->jfhy=0xffff;
       pA128C->jini=0xff;
       break;
     }
     set_power_connexion(pA128C->hyby|pA128C->fbia,cnx_chip);
     pA128C->cxid=get_id_connexion(cnx_chip);
     set_vref_connexion(pA128C->vref,cnx_chip);
     pA128C->lahy=get_power_status_connexion(cnx_chip)|(~pA128C->hyby);
     pA128C->lsta=(pA128C->lahy!=0xffff);
     for(i=0,k=0;i<MODULE_NB;i++){
       mask=1<<(MODULE_NB-1-i);
       if((mask&pA128C->hyby)==0){   /* if this hybrid is bypassed */
	 if(mask&pA128C->fbia){       /* if we want to force bias on this hybrid */
	   costar_list[pA128C->ladr][i].hard_bypass=1;	/* do not hard bypass costar */
	   strcpy(get_hybrid_map_ptr(pA128C,i),"Force");
	 }
	 else{
	   costar_list[pA128C->ladr][i].hard_bypass=0;	/* hard bypass costar */
	   strcpy(get_hybrid_map_ptr(pA128C,i),"Bypass"); 
	 }
       }
       else{
	 set_bypass_connexion(mask,cnx_chip);
	 clear_bypass();
	 l=detect_length();
	 if ((l>2)&&(l<10)){
	   if(l==9){ /* 1 rb + 1 cnx + 6 a128c + 1 costar */
	     costar_list[pA128C->ladr][i].hard_bypass=1;
	     strcpy(get_hybrid_map_ptr(pA128C,i),"CAAAAAA");
	     get_hybrid_map(pA128C,i,hybrid_map);
	     for(j=0;j<l-2;j++) map_list[pA128C->ladr][k++]=hybrid_map[j];
	     start_conversion_one_costar(0,hybrid_chip);
	   }
	   else {
/* 	     printf("lad%d:hyb%d:length=%d\n",pA128C->ladr,i,l); */
/* 	     costar_list[pA128C->ladr][i].hard_bypass=0;				 */
/* 	     strcpy(get_hybrid_map_ptr(pA128C,i),"AAAAAA"); */
/* 	     get_hybrid_map(pA128C,i,hybrid_map); */
	     auto_init_map(hybrid_map);
	     get_ascii_map(hybrid_map,name);
	     strcpy(get_hybrid_map_ptr(pA128C,i),name);
	     if(name[0]='C')
	       costar_list[pA128C->ladr][i].hard_bypass=1;
	     else 
	       costar_list[pA128C->ladr][i].hard_bypass=0;
	     
	     for(j=0;j<l-2;j++) map_list[pA128C->ladr][k++]=hybrid_map[j];
	   }
	 }
	 else{
	   pA128C->jfhy|=mask;		  /* if the hybrid does not respond to jtag */
	   costar_list[pA128C->ladr][i].hard_bypass=0;
	   strcpy(get_hybrid_map_ptr(pA128C,i),"Failed"); 
	 }
       }
     }
     map_list[pA128C->ladr][k++]=CONNEXION;
     map_list[pA128C->ladr][k++]=READOUT;
     map_list[pA128C->ladr][k++]=END;
     pA128C->jini=1;
     set_bypass_connexion((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia,cnx_chip);
     clear_bypass();
     
     break;   
     /* BIAS */	    
   case 2 : 
     if ((INIT_JTAG[pA128C->ladr]==1)||(INIT_JTAG[pA128C->ladr]==2)){
       if(pA128C->fbia) set_bypass_connexion(pA128C->hyby&~pA128C->jfhy,cnx_chip);
       
/*        for(i=0;(i<NB_MAX_A128C)&&(pA128C->byps[i]!=0);i++) /\* convert ascii bypass string into binary array *\/ */
/* 	 bypass[i]=pA128C->byps[i]-'0'; */
       get_bypass_array(pA128C->byps,bypass);
       if(strcmp(pA128C->bfil,"std")){
          strcpy(name,pA128C->bdir);
          strcat(name,pA128C->bfil);
          strcat(name,".bias");
          bias_file=fopen(name,"r"); /* open bias file (this file contains the name of the chip bias file for each chip*/
          if (bias_file==NULL){
	     printf("\nERROR : can't open %s.\n",name);
	     pA128C->bsta=255;
	     pA128C->val=0;
	     break;
          }
          for(i=0;i<NB_MAX_A128C;i++) strcpy(bias_name_tab[i],fgets(bias_name,32,bias_file)); /* store the datas of each chip */
          fclose(bias_file);	   /* in a string array */
       }
       else{
          for(i=0;i<NB_MAX_A128C;i++) 
	     strcpy(bias_name_tab[i],"20 100 50 91 60 160 100"); 
	     /* store standard parameters */
       }
       for(i=0;i<NB_MAX_A128C;i++){
	 if(bypass[i]){					/* if the chip is not soft bypassed */
	   if((bias_name_tab[i][0]>='0')&&(bias_name_tab[i][0]<='9')){ /* if the 1st char of the name of the file is a digit */ 
	     sscanf(bias_name_tab[i],"%d %d %d %d %d %d %d",		/* then it is the list of the list of parameters */
		    valt,valt+1,valt+2,valt+3,valt+4,valt+5,valt+6);
	     for(j=0;j<7;j++) bias[i][j]=valt[j];
	   }
	   else{					/* else open the file and get the parameters */
	     strcpy(name,pA128C->bdir);
	     strcat(name,bias_name_tab[i]);
	     for(j=0;(name[j]!='\n')&&(j<100);j++);
	     if (j<100) name[j]=0;
	     strcat(name,".chip");
	     chip_bias_file=fopen(name,"r");
	     if (chip_bias_file==NULL){
	       printf("\nERROR : can't open %s.\n",name);
	       for(j=0;j<7;j++) bias[i][j]=0;
	     }
	     else 
	       for(j=0;j<7;j++){
		 fscanf(chip_bias_file,"%d",&val);
		 bias[i][j]=val;
	       }
	     fclose(chip_bias_file);
	   }
	 }
	 else for(j=0;j<7;j++) bias[i][j]=0; /* if the chip is soft bypassed then put zeros in the bias register */ 
       }

       /*for(i=0;i<NB_MAX_A128C;i++)for(j=0;j<7;j++)printf("%d ",bias[i][j]); */
       for(i=0;i<MODULE_NB;i++){
	 mask=1<<(MODULE_NB-1-i);
	 if((mask&pA128C->hyby&~pA128C->jfhy)&&get_hybrid_map(pA128C,i,hybrid_map)){
	   set_bypass_connexion(mask,cnx_chip);
	   set_bias_all_a128c(bias+i*6,old_bias+i*6,hybrid_map);
	   set_bias_all_a128c(bias+i*6,old_bias+i*6,hybrid_map);
	   set_onoff_all_a128c(bypass+i*6,hybrid_map);
	   set_token_all_a128c(bypass+i*6,hybrid_map);
	 }
	 else if(mask&(pA128C->fbia|pA128C->jfhy)){
	   set_bypass_connexion(mask,cnx_chip);
	   for(j=0;j<6;j++){
	     set_bias_one_a128c(bias[i*6+j],old_bias[i*6+j],j,hybrid_chip);
	     set_bias_one_a128c(bias[i*6+j],old_bias[i*6+j],j,hybrid_chip);
	     set_onoff_one_a128c(bypass[i*6+j],j,hybrid_chip);
	     set_token_one_a128c(bypass[i*6+j],j,hybrid_chip);
	   }
	 }
       }
       set_bypass_connexion((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia,cnx_chip);
       clear_bypass();

       val=0;

       sprintf(name,"%sbias%d",pA128C->bdir,1+pA128C->ladr/2);	/* put the bias paremeters coming out */
       if(pA128C->ladr%2)
	 strcat(name,"P.ret");
       else
	 strcat(name,"N.ret");
       old_bias_file=fopen(name,"w");				/* in a file 			*/
       if (old_bias_file==NULL)
	 printf("\nERROR : can't open %s.\n",name);
       else{
	 for(i=0;i<NB_MAX_A128C;i++){
	   if(i%6==0)
	     fprintf(old_bias_file,"Hybrid %d\n",MODULE_NB-1-(i/6));
	   for(j=0;j<7;j++){
	     if(bias[i][j]==old_bias[i][j])
	       fprintf(old_bias_file,"%d\t",old_bias[i][j]);
	     else{
	       fprintf(old_bias_file,"X%dX\t",old_bias[i][j]);
	       if(pA128C->hyby&(~pA128C->jfhy)&(1<<(MODULE_NB-1-(i/6))))
		 val++;
	     }
	   }
	   fprintf(old_bias_file,"\n");
	 }
	 fclose(old_bias_file);
       }
       pA128C->bsta=val;	    	    
       pA128C->jini=2;
     }
     else pA128C->jini=255;
     break;   
     
     /* RESET */	     	    
   case 3  : 
     set_bypass_connexion(0,cnx_chip);
     clear_bypass();
     pA128C->jini=0;
     pA128C->bsta=0;
     pA128C->lsta=0;
     if(detect_length()!=2) {
       /*printf("RESET : connexion board not detected\n");*/
       /*pA128C->jini=0xff;*/
       break;
     }
     set_power_connexion(0,cnx_chip); /* power off all hybrids */
     set_power_connexion(pA128C->hyby|pA128C->fbia,cnx_chip); /* power on working hybrids */
     
     for(i=0;i<6;i++)
       for(j=0;j<7;j++)
	 bias[i][j]=0;  /* set biases to zero */
     
     for(i=0;i<6;i++) bypass[i]=0;
     
     for(i=0;i<MODULE_NB;i++) {
       set_bypass_connexion(1<<i,cnx_chip);   		/* select this hybrid */
       set_bias_all_a128c(bias,old_bias,hybrid_chip);	/* upload bias */ 
       set_onoff_all_a128c(bypass,hybrid_chip);		
       set_token_all_a128c(bypass,hybrid_chip);
     }	   
     
     break;
     
   /* TEST */		
   case 4  : 
     val=0;
     sprintf(name,"%spulser%d",pA128C->bdir,1+pA128C->ladr/2);	
     if(pA128C->ladr%2)
       strcat(name,"P.ret");
     else
       strcat(name,"N.ret");
     old_bias_file=fopen(name,"w");				/* in a file 			*/
     if (old_bias_file==NULL)
       printf("\nERROR : can't open %s.\n",name);
     else
       val=1;


     if (INIT_JTAG[pA128C->ladr]&&pA128C->spac){
       for(k=0;k<MODULE_NB;k++){
	 mask=1<<(MODULE_NB-1-k);
	 if (val) fprintf(old_bias_file,"Hybrid %d\n",k);
	 if((mask&pA128C->hyby&~pA128C->jfhy)&&get_hybrid_map(pA128C,k,hybrid_map)){
	   set_bypass_connexion(mask,cnx_chip);
	   /* set the pulse level */
	   set_pulse_all_a128c(pA128C->puls,hybrid_map);

	   for(i=0;i<6;i++){
	     switch(pA128C->spac){
	       /* No Pulse */
	     case 0 : 
	       for(j=0;j<8;test[j++]=0);
	       break;
	       /* 1 per ladder */
	     case 1 : 
	       for(j=0;j<8;test[j++]=0);
	       if ((k*6+i)==(pA128C->chan>>7))
		 test[(pA128C->chan&127)>>4]=(1<<(pA128C->chan&15));
	       break;
	       /* 1 per chip */
	     case 2 : 
	       for(j=0;j<8;test[j++]=0);
	       test[(pA128C->chan&127)>>4]=(1<<(pA128C->chan&15));
	       break;
	       
	       /* 2 per chip */
	     case 3 : 
	       for(j=0;j<8;test[j++]=0);
	       test[(pA128C->chan&63)>>4]=(1<<(pA128C->chan&15));
	       test[((pA128C->chan&63)>>4)+4]=(1<<(pA128C->chan&15));
	       break;
	       /* 4 per chip */
	     case 4 : 
	       for(j=0;j<8;test[j++]=0);
	       test[(pA128C->chan&31)>>4]=(1<<(pA128C->chan&15));
	       test[((pA128C->chan&31)>>4)+2]=(1<<(pA128C->chan&15));
	       test[((pA128C->chan&31)>>4)+4]=(1<<(pA128C->chan&15));
	       test[((pA128C->chan&31)>>4)+6]=(1<<(pA128C->chan&15));
	       break;
	       
	       /* 8 per chip */
	     case 5 : 
	       for(j=0;j<8;test[j++]=(1<<(pA128C->chan&15)));
	       break;
	       
	       /* patern */    
	     case 6 : 
	       for(j=0;j<8;test[j++]=0);
	       test[((k*6+i)&127)>>4]=(1<<((k*6+i)&15));
	       break;
	     default : break;
	     }
	     /* get in test channel mode */
	     set_test_one_a128c(test,old_test,i,hybrid_map);
	     set_test_one_a128c(test,old_test,i,hybrid_map);
             if(val){
               for(l=0;l<8;l++) fprintf(old_bias_file,"%04X",old_test[l]);
               fprintf(old_bias_file," pulse=%d\n",set_pulse_one_a128c(pA128C->puls,i,hybrid_map));
             }
	   }
	   set_bypass_connexion(((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia),cnx_chip);
	   clear_bypass();
	 }
       }
     }
     if(val) fclose(old_bias_file);
     break;
     
     /* Transparent Channel */	     
   case 5  : 
     if (INIT_JTAG[pA128C->ladr]){
       i=(pA128C->tran>>7)/6; /* get the module to put in transparent */
       set_bypass_connexion(1<<(MODULE_NB-1-i),cnx_chip);
       get_hybrid_map(pA128C,i,hybrid_map);
       transp_chan_a128c(pA128C->tran&127,(pA128C->tran>>7)%6,hybrid_map);
       /* disable all outputs but the one which will be read */
       for(i=0;(i<NB_MAX_A128C);i++) 
	 bypass[i]=0;
       bypass[pA128C->tran>>7]=1; 
       for(i=0;(i<MODULE_NB);i++){
	 set_bypass_connexion(1<<(MODULE_NB-1-i),cnx_chip);
	 get_hybrid_map(pA128C,i,hybrid_map);
 	 set_onoff_all_a128c(bypass+i*6,hybrid_map);
	 set_pulse_all_a128c(pA128C->puls,hybrid_map);
       }
       set_bypass_connexion((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia,cnx_chip);
       clear_bypass();
       
     }
     break;	     
     /* Latchup */			 
   case 6  : 
     if (INIT_JTAG[pA128C->ladr]){
       set_bypass_connexion(0,cnx_chip);			/* select only the connexion  board */
       pA128C->lahy=get_power_status_connexion(cnx_chip)|(~pA128C->hyby);	/* get the hybrid latchup status */
       pA128C->lsta=(pA128C->lahy!=0xffff);
       for(i=0;i<MODULE_NB;i++){
	 if((pA128C->lahy&(1<<(MODULE_NB-1-i)))==0){	/* if the module ni is bypassed because of latchup */
	   costar_list[pA128C->ladr][i].hard_bypass=0;
	   strcpy(get_hybrid_map_ptr(pA128C,i),"Latchup");
	 }
       }
       set_bypass_connexion((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia,cnx_chip);
       clear_bypass();
       if(pA128C->lsta||pA128C->jfhy||pA128C->bsta)
	 pA128C->jini=254;
     }
     break;	     
     
     /* read COSTAR */	
   case 7 : 
     set_bypass_connexion(0,cnx_chip);
     get_temperature_connexion(cnx_temp,cnx_chip);
     if(cnx_temp[0]==0)
       get_temperature_connexion(cnx_temp,cnx_chip); /* the first temperature read is always bad */
     pA128C->tmp1=cnx_temp[0]/10-273;
     pA128C->tmp2=cnx_temp[1]/10-273;
     pA128C->tmp3=cnx_temp[2]/10-273;
     if (INIT_JTAG[pA128C->ladr]){
       for(i=0;i<MODULE_NB;i++){
	 mask=1<<(MODULE_NB-1-i);
	 if((mask&pA128C->hyby&~pA128C->jfhy)&&get_hybrid_map(pA128C,i,hybrid_map)
	    &&costar_list[pA128C->ladr][i].hard_bypass
	    &&costar_list[pA128C->ladr][i].soft_bypass){
	   set_bypass_connexion(mask,cnx_chip);
	   stop_conversion_one_costar(0,hybrid_map);
	   get_adc0_one_costar(costar_list[pA128C->ladr][i].adc0,0,hybrid_map);
	   get_adc1_one_costar(costar_list[pA128C->ladr][i].adc1,0,hybrid_map);
	   start_conversion_one_costar(0,hybrid_map);
	   if (costar_list[pA128C->ladr][i].vref!=costar_list_old_vref[pA128C->ladr][i]){  /* if vref value is different from the last one */
	     for(j=0;j<4;j++) dac[j]=costar_list[pA128C->ladr][i].vref;	
	     set_dac_one_costar(dac,old_dac,0,hybrid_map);	/* set the new value */
	     costar_list_old_vref[pA128C->ladr][i]=costar_list[pA128C->ladr][MODULE_NB-1-i].vref;
	   }
	 }
       }
       set_input_mux_connexion(module_mux[pA128C->ladr],cnx_chip);
       set_bypass_connexion((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia,cnx_chip);
       clear_bypass();
     }
     break;	     
     
   case 8 :  /* not used anymore */ 
     break;	     


   case 9 : /* Power on */
     
     set_bypass_connexion(0,cnx_chip);
     if(detect_length()!=2){
       printf("POWER ON : connexion board not detected\n");
       pA128C->jini=0xff;
       break;
     }
     set_power_connexion(pA128C->hyby|pA128C->fbia,cnx_chip);
     pA128C->lahy=get_power_status_connexion(cnx_chip)|(~pA128C->hyby);
     set_bypass_connexion((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia,cnx_chip);
     clear_bypass();
     break;		
     
   case 10 : /* Power off */
     
     set_bypass_connexion(0,cnx_chip);
     if(detect_length()!=2) {
       printf("POWER OFF : connexion board not detected\n");
       pA128C->jini=0xff;
       break;
     }
     set_power_connexion(0,cnx_chip);
     pA128C->lahy=get_power_status_connexion(cnx_chip)|(~pA128C->hyby);
     set_bypass_connexion((pA128C->hyby&~pA128C->jfhy)|pA128C->fbia,cnx_chip);
     clear_bypass();
     break;		
     
   default : break;
   }
   semGive(sem_jtag);			   		/* release the protection */
   dbScanLock((struct dbCommon *)pA128C);
   (*prset->process)(pA128C);
   dbScanUnlock((struct dbCommon *)pA128C);
   if(pA128C->jini==255) /* if initialisation failed */ 
     pA128C->val=3; /*RESET*/
   else if(pA128C->jini==254) /* if latchup */
     pA128C->val=1; /* INIT */
   else if(pA128C->jini==253) /* if all bypassed */
     pA128C->val=10; /* POWEROFF */
   else
     pA128C->val=0; /*NOP*/
   db_post_events(pA128C,&pA128C->val,DBE_VALUE);
   return 0;
}
