/* caen.c		Caen Sy527 functions 
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	Annan Fedoul, Jerome Baudot & David Bonnet
 *	Date :		20/10/00
 *
 */
  
#include "vxWorks.h"
#include "sysLib.h"
#include "stdio.h"
#include "caen.h"

char INIT;
v288_reg *v288;
static unsigned short buffer_fifo[256];
const int CRATE_NB[NUMBER_OF_CRATE]={74,25};

void reset_caen(void)
{
   v288->reset=1;
}

void init_caen(crate *cr)
{
   int i;
	
   INIT=0;
		
   /* Calculate the V288 registers address */
   sysBusToLocalAdrs(AM_V288,(char *) V288_ADR,(char **) &v288);
	
   /* Read the parameters of all crates */
   for (i=0;i<NUMBER_OF_CRATE;i++) 
   {
      cr[i].nb=CRATE_NB[i];
      if(read_crate(&cr[i])!=0)
      printf("Caen Initialize : crate %d not found.\n",i);
   }
   INIT=1;
   return;
}


/* storing data in the tx buffer */

int stockage (unsigned short data)
{ 
  int i=0;
  
  do
  {
     v288->fifo=data;
     i++;
  }
  while((v288->status!=BUFFER_VALID)&&(i<TIMEOUT));
  if (i==TIMEOUT) return 1;
  return 0;            
}

/* sending data to buffer */

int envoi (unsigned short crate_nb,unsigned short donnee[], unsigned short nb_data)
{
int i;
if(stockage (CPU_v288)) return 1;
if(stockage (crate_nb)) return 1;
for(i=0;i<nb_data;i++)
  if(stockage (donnee[i])) return 1;
return 0;  
}

/* validating the envoi function */

int ordre_envoi()
{
  int i=0;
  do
  {
     v288->control=ORDRE;
     i++;
  }
  while((v288->status!=BUFFER_VALID)&&(i<TIMEOUT));
  if (i==TIMEOUT) return 1;
  return 0;            
}


/* receiving data */

unsigned short recept()
{    
 unsigned short i=0;
 int j=0;
 
  do {
    buffer_fifo[i]=v288->fifo;
    j++;
  } while((v288->status!=BUFFER_VALID)&&(j<TIMEOUT));
  if (j==TIMEOUT){
    buffer_fifo[i]=1;
/*     printf("Caen : Timeout\n"); */
  }
  i++;

  do {
    buffer_fifo[i]=v288->fifo;
    i++;
    if (i>200) break;
  } while(v288->status==BUFFER_VALID);
 return i;
}

/* check the presence of the crate */

int check_crate(crate *cr)   
{
  unsigned short n,b[1];
   
  /* read the name of the crate */ 
  b[0]=0x00;   /* identification code */
  envoi(cr->nb,b,1);        
  ordre_envoi();
  n=recept();
  if (buffer_fifo[0]!=0) return 0; /* error */
  return 1;
}

/* read the parameters of the crate */

int read_crate(crate *cr)   
{
  unsigned short n,b[1],occup;
  int i,err=0;
 
   /* read the name of the crate */ 
  b[0]=0x00;   /* identification code */
  if (envoi(cr->nb,b,1)) return 1;        
  if (ordre_envoi()) return 1;
  n=recept();
  if (buffer_fifo[0]!=0) return 1; /* error */
  for(i=0;i<n;i++) 
  { 
    cr->name[i]=lo16byte(buffer_fifo[i+1]);
  }
  cr->name[n]=0;
  
  /* read the occupation of the crate */
  b[0]=0x04;   /* occupation code */
  if (envoi(cr->nb,b,1)) return 1;        
  if (ordre_envoi()) return 1;
  recept();
  if (buffer_fifo[0]!=0) return 2; /* error */
  occup=buffer_fifo[1];
  
  /* read board caracteristics */
  
  for (i=0;i<NB_BRD_MAX;i++)
  {
      if (occup&(1<<i)) err=read_board(cr,i);
      else cr->brd[i].name[0]=0;
      if (err!=0) return 3;
  }
  
  return 0;
}

/* read the board parameters */

int read_board(crate *cr,unsigned short nb)
{
 int i,err=0;
 unsigned short b[2];

 b[0]=0x03;
 b[1]=nb;                                         
 if (envoi(cr->nb,b,2)) return 1;
 if (ordre_envoi()) return 1;
 recept();
 if (buffer_fifo[0]!=0) return 1; /* error */

 for(i=0;i<(NAME_BRD_SIZE/2);i++) 
 {
   if( i<=1 ) {
     cr->brd[nb].name[i*2]=(char) hi16byte(buffer_fifo[i+1]);
     cr->brd[nb].name[i*2+1]=(char) lo16byte(buffer_fifo[i+1]);
   }
   else {
     cr->brd[nb].name[i*2]=(char) hi16byte(buffer_fifo[i+1]);
     cr->brd[nb].name[i*2+1]='\0';
     cr->brd[nb].CurrentUnits=lo16byte(buffer_fifo[i+1]);
     
   }  
 }
 
 cr->brd[nb].Ser_Num=buffer_fifo[4];
 cr->brd[nb].Ver1=hi16byte(buffer_fifo[5]);
 cr->brd[nb].Ver2=lo16byte(buffer_fifo[5]);
 cr->brd[nb].Num_of_channels=hi16byte(buffer_fifo[16]);
 cr->brd[nb].Vmax=lo16byte(buffer_fifo[18])*(1<<24)+buffer_fifo[19]*(1<<8)+hi16byte(buffer_fifo[20]);
 cr->brd[nb].Imax=hilo16byte(lo16byte(buffer_fifo[20]),hi16byte(buffer_fifo[21]));
 cr->brd[nb].Ramp_min=hilo16byte(lo16byte(buffer_fifo[21]),hi16byte(buffer_fifo[22]));
 cr->brd[nb].Ramp_max=hilo16byte(lo16byte(buffer_fifo[22]),hi16byte(buffer_fifo[23]));
 cr->brd[nb].Vres=hilo16byte(lo16byte(buffer_fifo[23]),hi16byte(buffer_fifo[24]));
 cr->brd[nb].Ires=hilo16byte(lo16byte(buffer_fifo[24]),hi16byte(buffer_fifo[25]));
 cr->brd[nb].Vdec=hilo16byte(lo16byte(buffer_fifo[25]),hi16byte(buffer_fifo[26]));
 cr->brd[nb].Idec=hilo16byte(lo16byte(buffer_fifo[26]),hi16byte(buffer_fifo[27]));
  
  for (i=0;i<cr->brd[nb].Num_of_channels;i++) 
  {
    err=read_channel(cr,nb,i);
    if (err!=0) return 1;
    if (INIT=0) cr->brd[nb].chan[i].used=0;
  }
 return 0;
}

/* read the channel parameters */

int read_channel(crate *cr,unsigned short nb,unsigned short nc)
{
 int i;
 unsigned short b[2];
 
 b[0]=0x02;  /* reading channel parameters code */
 b[1]=hilo16byte(nb&15,nc);                                        
 if (envoi(cr->nb,b,2)) return 1;
 if (ordre_envoi()) return 2;
 recept();
 if (buffer_fifo[0]!=0) return 3; /* error */
 
 for(i=0;i<NAME_CHAN_SIZE/2;i++) {
   cr->brd[nb].chan[nc].name[i*2]=hi16byte(buffer_fifo[i+1]);
   cr->brd[nb].chan[nc].name[i*2+1]=lo16byte(buffer_fifo[i+1]);
 } 
 
 cr->brd[nb].chan[nc].V0set=hilo32byte(buffer_fifo[7],buffer_fifo[8]);
 cr->brd[nb].chan[nc].V1set=hilo32byte(buffer_fifo[9],buffer_fifo[10]);
 cr->brd[nb].chan[nc].I0set=buffer_fifo[11];
 cr->brd[nb].chan[nc].I1set=buffer_fifo[12];
 cr->brd[nb].chan[nc].Vmax=buffer_fifo[13];
 cr->brd[nb].chan[nc].Rup=buffer_fifo[14];
 cr->brd[nb].chan[nc].Rdwn=buffer_fifo[15];
 cr->brd[nb].chan[nc].Trip=buffer_fifo[16];
 cr->brd[nb].chan[nc].Flag= buffer_fifo[17];
 
 return 0;
}


/* read the channel status */

int read_channel_status(crate *cr,unsigned short nb,unsigned short nc)
{
unsigned short b[2];


    b[0]=0x01;           /* reading the channel status code */
    b[1]=hilo16byte(nb&15,nc);     
    if (envoi(cr->nb,b,2)) return 1;    
    if (ordre_envoi()) return 2;
    recept();
    if (buffer_fifo[0]!=0) return 3; /* error */
    
    cr->brd[nb].chan[nc].Vmon=hilo32byte(buffer_fifo[1],buffer_fifo[2]);
    cr->brd[nb].chan[nc].HVmax=buffer_fifo[3];
    cr->brd[nb].chan[nc].Imon=buffer_fifo[4];
    cr->brd[nb].chan[nc].Status=buffer_fifo[5];
/*    if ((cr->brd[nb].chan[nc].Status&0x8000)==0) cr->brd[nb].chan[nc].Power=0;
    else cr->brd[nb].chan[nc].Power=1;*/
    
    return 0;
}

/*********************************************************************************
*	 MODIFICATION DES PARAMETRES                                            *
*********************************************************************************/

int set_name(crate *cr,unsigned short nb,unsigned short nc,char *name)
{
  int i;
  unsigned short b[8];
  
  b[0]=0x19;   /* seting channel name */
  b[1]=hilo16byte(nb&15,nc);      /* channel number */
  b[2]=hilo16byte(name[0],name[1]);
  b[3]=hilo16byte(name[2],name[3]);
  b[4]=hilo16byte(name[4],name[5]);
  b[5]=hilo16byte(name[6],name[7]);
  b[6]=hilo16byte(name[8],name[9]);
  b[7]=hilo16byte(name[10],0);
  i=0;
  do
  {
     envoi(cr->nb,b,8);
     ordre_envoi();
     recept();
  } while ((buffer_fifo[0]!=0)&&(i++<TIMEOUT));
  if (i==TIMEOUT) return 1; /* error */

   return 0;
}

int set_V0set(crate *cr,unsigned short nb,unsigned short nc,int val)
{
  int i;
  unsigned short b[3];
  
  b[0]=0x10; /* seting V0set value */
  b[1]=hilo16byte(nb&15,nc); /* channel number */
  b[2]=val;
  i=0;
  do
  {
     if(envoi(cr->nb,b,3)) return 1; 
     if(ordre_envoi())return 2;
     recept();
     i++;
  } while ((buffer_fifo[0]!=0)&&(i<TIMEOUT));
  if (i==TIMEOUT) return 3; /* error */

   return 0;
}

int set_I0set(crate *cr,unsigned short nb,unsigned short nc,unsigned short val)
{
  int i;
  unsigned short b[3];
  
  
  b[0]=0x12; 
  b[1]=hilo16byte(nb&15,nc); /* channel number */
  b[2]=val;
  i=0;
  do
  {
     if(envoi(cr->nb,b,3)) return 1; 
     if(ordre_envoi()) return 2;
     recept();
     i++;
  } while ((buffer_fifo[0]!=0)&&(i<TIMEOUT));
  if (i==TIMEOUT) return 3; /* error */

   return 0;
}


int set_Rup(crate *cr,unsigned short nb,unsigned short nc,unsigned short v)
{
  int i;
  unsigned short b[3];
  
  b[0]=0x15; 
  b[1]=hilo16byte(nb&15,nc); /* channel number */
  b[2]=v;
  i=0;
  do
  {
     if(envoi(cr->nb,b,3)) return 1; 
     if(ordre_envoi()) return 2;
     recept();
     i++;
  } while ((buffer_fifo[0]!=0)&&(i<TIMEOUT));
  if (i==TIMEOUT) return 3; /* error */

   return 0;
}

int set_Rdwn(crate *cr,unsigned short nb,unsigned short nc,unsigned short v)
{
  int i;
  unsigned short b[3];
  
  b[0]=0x16; 
  b[1]=hilo16byte(nb&15,nc); /* channel number */
  b[2]=v;
  i=0;
  do
  {
     if(envoi(cr->nb,b,3)) return 1; 
     if(ordre_envoi()) return 2;
     recept();
     i++;
  } while ((buffer_fifo[0]!=0)&&(i<TIMEOUT));
  if (i==TIMEOUT) return 3; /* error */

   return 0;
}
 

int set_flag(crate *cr,unsigned short nb,unsigned short nc,unsigned short f)
{
  int i;
  unsigned short b[3];
  
  b[0]=0x18; 
  b[1]=hilo16byte(nb&15,nc); /* channel number */
  b[2]=f;
  i=0;
  do
  {
     if(envoi(cr->nb,b,3)) return 1; 
     if(ordre_envoi()) return 2;
     recept();
     i++;
  } while ((buffer_fifo[0]!=0)&&(i<TIMEOUT));
  if (i==TIMEOUT) return 3; /* error */

   return 0;
}

