/* caen.h	Caen SY527+V288 record support header 
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	David Bonnet
 *	Date :		16/02/00
 *
 */
 
#define NUMBER_OF_CRATE 2
#define NB_CHANNEL_MAX  16
#define NB_BRD_MAX      10
#define NAME_CHAN_SIZE  12
#define NAME_BRD_SIZE   6
#define NAME_CRATE_SIZE 20
#define V288_ADR        0x800000	/* V288 board address*/
#define AM_V288         0x39		/* address modifier */
#define CPU_v288        0x01
#define ORDRE           0x01
#define BUFFER_VALID    0xfffe
#define TIMEOUT		100000
#define ASYN_CAEN_TASK_DELAY 120	/* 2 second */
#define ASYN_CAEN_TASK_PRIORITY 200
#define ASYN_CAEN_TASK_STACK 10000


/* channel structure */

typedef struct
{
char name[NAME_CHAN_SIZE];	/* symbolic name of the channel*/
int Vmon;			/* voltage monitored value*/              
unsigned short Imon;		/* current monitored value*/     
unsigned short HVmax;		/* hardware volrage limit*/
int Vmax;			/* maximum voltage value programmable for a channel*/ 
unsigned short Status;		/* status of the channel*/
int V0set;			/* 1st voltage programmable values*/
unsigned short I0set;		/* 1st current limit*/
int V1set;			/* 2nd voltage programmable values*/
unsigned short I1set;		/* 2nd allowed current limit*/
unsigned short Rdwn;		/* maximum voltage programmable decrease rate*/
unsigned short Rup;		/* maximum voltage programmable increase rate*/
unsigned short Trip;		/* maximum time an overcurrent is allowd to last*/
unsigned short Flag; 
short 	Power ; 
int comm;			/* Number of communication failure */
int used;			/* is this channel used ? */    
}channel;                                                                             


/* Board structure */

typedef struct
{
char name[NAME_BRD_SIZE];	/* name of the board */
unsigned short CurrentUnits;	/* Ioset,I1set and Imon unit */ 
unsigned short Ser_Num;		/* serial number of the board */
unsigned char Ver1;		/* 1st byte which represents the software version */
unsigned char Ver2;		/* 2nd byte which represents the software version */
unsigned char Num_of_channels;	/* number of channels housed in the board */
int Vmax;			/* maximum output voltage of the board channels   */
unsigned short Imax;		/* maximum output current of the board channels  */
unsigned short Ramp_min;	/* minimum of the ramp up/ramp down value programmablefor the channel of the board */
unsigned short Ramp_max;	/* maximum of the ramp up/ramp down value programmablefor the channel of the board */
unsigned short Vres;		/* Vset/monitor resolution for the board */
unsigned short Ires;		/* Iset/monitor resolution for the board */
unsigned short Vdec;		/* number of significant figures after the decimal point for Vset */
unsigned short Idec;		/* number of significant figures after the decimal point for Iset */
channel  chan[NB_CHANNEL_MAX];  /* channels */
}board;


/* crate structure */

typedef struct
{
unsigned short nb;        	/* id number of the crate */
int bnb;        	/* v288 board number of the crate */
char   name[NAME_CRATE_SIZE];   /* name of the crate */
board brd[NB_BRD_MAX];     	/* boards */
}crate;


/* V288 CAENet VME module registers */

typedef struct
{
unsigned short fifo;       /* I/O fifo*/
unsigned short status;     /* status of the operation */
unsigned short control;  
unsigned short reset;
}v288_reg;



/* usefull functions */

/* short to char */

static unsigned char lo16byte(unsigned short a)
{
return (a&0xff);
}

static unsigned char hi16byte(unsigned short a)
{
return ((a&0xff00)>>8);
}

/* char to short */

static unsigned short hilo16byte(unsigned char hi, unsigned char lo)
{
return ( (lo & 0x00ff) | ((hi <<8) & 0xff00) );
}

/* int to short */

static unsigned short lo32byte(unsigned int a)
{
return (a&0xffff);
}

static unsigned short hi32byte(unsigned int a)
{
return (a&0xffff0000)>>16;
}

/* short to int */

static unsigned int hilo32byte(unsigned short hi, unsigned short lo)
{
return ( (lo & 0x0000ffff) | ((hi <<16) & 0xffff0000) );
}

void reset_caen(void);
int  read_crate(crate *cr);
int  read_board(crate *cr,unsigned short nb);
int  read_channel(crate *cr,unsigned short nb,unsigned short nc);
int  read_channel_status(crate *cr,unsigned short nb,unsigned short nc);
int  stockage(unsigned short data);
int  envoi(unsigned short crate_nb,unsigned short donnee[], unsigned short nb_data);
int  ordre_envoi();
unsigned short recept();
int  set_name(crate *cr,unsigned short nb,unsigned short nc,char *name);
int  set_V0set(crate *cr,unsigned short nb,unsigned short nc,int val);
int  set_I0set(crate *cr,unsigned short nb,unsigned short nc,unsigned short val);
int  set_Rup(crate *cr,unsigned short nb,unsigned short nc,unsigned short v);
int  set_Rdwn(crate *cr,unsigned short nb,unsigned short nc,unsigned short v);
void init_caen(crate *cr);
