/* recCaen.c - record support routines for CAEN SY527 crate and V288 VME module*/
/*
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author:		David Bonnet
 *	Date:		16/02/00
 */

#include	<vxWorks.h>
#include 	<taskLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<string.h>

#include	<alarm.h>
#include	<dbDefs.h>
#include	<dbAccess.h>
#include	<dbScan.h>
#include	<dbFldTypes.h>
#include	<devSup.h>
#include	<errMdef.h>
#include	<recSup.h>
#include 	<caen.h>
#include 	<jtagCommon.h>
#define GEN_SIZE_OFFSET
#include 	<caenRecord.h>
#undef GEN_SIZE_OFFSET

extern const int CRATE_NB[NUMBER_OF_CRATE]; /* comes from caen.c */
extern unsigned short relay;

/* Create RSET - Record Support Entry Table*/
#define report NULL
static long initialize();
static long init_record();
static long process();
#define special NULL
static long get_value(); 
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
static long get_units();
static long get_precision();
static long get_enum_str();
static long get_enum_strs();
static long put_enum_str();
static long get_graphic_double();
static long get_control_double();
#define get_alarm_double NULL


struct rset caenRSET={
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double};

int async_caen(void);
static long caen_io();
static void monitor();
static void alarm();

extern char INIT;
crate crate_list[NUMBER_OF_CRATE],crate_list_copy[NUMBER_OF_CRATE];
static SEM_ID sem_caen;


static long initialize(void)
{
	init_caen(crate_list);
	sem_caen=semBCreate(SEM_Q_PRIORITY, SEM_FULL);
	return 0;
}

static long init_record(caenRecord *pcaen,int pass)
{
	int i,vm=1,im=1,err=0;
	
	if ((pcaen->crat>NUMBER_OF_CRATE)||(pcaen->brd>NB_BRD_MAX)||(pcaen->chan>NB_CHANNEL_MAX))
		printf("Init Error\n");
	
	if (pass==0)
	{
		if (/*(crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].name[0]==0)
		  ||*/(pcaen->chan>crate_list[pcaen->crat].brd[pcaen->brd].Num_of_channels))
		{   
			recGblRecordError(S_dev_badCard,(void *)pcaen,"caen: init_record");
	printf("error : crate : %d brd: %d chan %d\n",pcaen->crat,pcaen->brd,pcaen->chan);
                        crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].used=0;
			return S_dev_badCard;
		}
	
  		for (i=0;i<crate_list[pcaen->crat].brd[pcaen->brd].Vdec;i++) vm*=10;
  		for (i=0;i<crate_list[pcaen->crat].brd[pcaen->brd].Idec;i++) im*=10;
		read_channel_status(&crate_list[pcaen->crat],pcaen->brd,pcaen->chan);
		crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].used=1;
		if((crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Status&0x8000)!=0)
		{
		   pcaen->val=1;
		   pcaen->chst=0x8000;
		   pcaen->pwst=1;
		}
		/*err=set_name(&crate_list[pcaen->crat],pcaen->brd,pcaen->chan,pcaen->name);*/
		if (err!=0)
		{   
			recGblRecordError(S_dev_badCard,(void *)pcaen,"caen: init_record");
			errPrintf(0,__FILE__,__LINE__,"error : crate : %d brd: %d chan %d\n",pcaen->crat,pcaen->brd,pcaen->chan);
	return S_dev_badCard;
		}
	}
	
	if ((pass==1)&&(INIT==1))
	{
		/*for (i=0;i<NUMBER_OF_CRATE;i++) 
		{
			crate_list[i].nb=CRATE_NB[i];
			if((err=read_crate(&crate_list[i]))!=0)
				printf("Caen init_record (%d): crate %d (%d) not found.\n",err,i,crate_list[i].nb);
		}*/
		for (i=0;i<NUMBER_OF_CRATE;i++) crate_list_copy[i]=crate_list[i]; 
		
		taskSpawn("tCaenAsync",ASYN_CAEN_TASK_PRIORITY,0,ASYN_CAEN_TASK_STACK,&async_caen,0,0,0,0,0,0,0,0,0,0);
		INIT=2;
	}
	
	return 0;
}

static long process(caenRecord *pcaen)
{
	long status;
	
	status=caen_io(pcaen); /* Do caen I/O operation */
	
	pcaen->pact=TRUE;
	alarm(pcaen);
	monitor(pcaen);
	recGblGetTimeStamp(pcaen);
	recGblFwdLink(pcaen);

	pcaen->pact=FALSE;
	return(status);
}	

static long get_value(caenRecord *pcaen,struct valueDes *pvdes)
{
    pvdes->field_type = DBF_ENUM;
    pvdes->no_elements=1;
    (unsigned short *)(pvdes->pvalue) = &pcaen->val;
    return(0);
}

static long get_units(struct dbAddr *paddr,char *units)
{
    	caenRecord	*pcaen=(caenRecord *)paddr->precord;
     	
	if(paddr->pfield==(void *)&pcaen->vmon || 
           paddr->pfield==(void *)&pcaen->vset /* ||
	   paddr->pfield==(void *)&pcaen->vhh || 
           paddr->pfield==(void *)&pcaen->vll || 
           paddr->pfield==(void *)&pcaen->vhi ||
           paddr->pfield==(void *)&pcaen->vlo */)
    	
		strncpy(units,"V",2); 
	
	else if (paddr->pfield==(void *)&pcaen->imon || 
                 paddr->pfield==(void *)&pcaen->iset /* ||
	         paddr->pfield==(void *)&pcaen->ihh || 
                 paddr->pfield==(void *)&pcaen->ill || 
                 paddr->pfield==(void *)&pcaen->ihi ||
                 paddr->pfield==(void *)&pcaen->ilo */)
	
		switch(crate_list[pcaen->crat].brd[pcaen->brd].CurrentUnits)
		{
		case 0 :
			strncpy(units,"A",2);
			break;
		case 1 :
			strncpy(units,"mA",3);
			break;
		case 2 :
			strncpy(units,"uA",3);
			break;
		case 3 :
			strncpy(units,"nA",3);
			break;
		default :
			strncpy(units,"Unknown",8);
			break;
		}
			
	else if (paddr->pfield==(void *)&pcaen->rup || 
	          paddr->pfield==(void *)&pcaen->rdwn)
	
		strncpy(units,"V/s",4); 
	
        else units[0]=0;
	 
    return(0);
}

static long get_precision(paddr,precision)
    struct dbAddr *paddr;
    long	  *precision;
{
    caenRecord	*pcaen=(caenRecord *)paddr->precord;
    
    if(paddr->pfield==(void *)&pcaen->vmon || 
       paddr->pfield==(void *)&pcaen->vset)
       
       *precision=crate_list[pcaen->crat].brd[pcaen->brd].Vdec;

    else if(paddr->pfield==(void *)&pcaen->imon || 
       paddr->pfield==(void *)&pcaen->iset)
       
       *precision=crate_list[pcaen->crat].brd[pcaen->brd].Idec;
       
    else recGblGetPrec(paddr,precision);
    return(0);
}

static long get_graphic_double(struct dbAddr *paddr,struct dbr_grDouble	*pgd)
{
    	caenRecord	*pcaen=(caenRecord *)paddr->precord;
  	int im=1,i;
	
	
	for (i=0;i<crate_list[pcaen->crat].brd[pcaen->brd].Idec;i++) im*=10;

     	if(paddr->pfield==(void *)&pcaen->vmon || 
           paddr->pfield==(void *)&pcaen->vset /* ||
	   paddr->pfield==(void *)&pcaen->vhh || 
           paddr->pfield==(void *)&pcaen->vll || 
           paddr->pfield==(void *)&pcaen->vhi ||
           paddr->pfield==(void *)&pcaen->vlo  */)
	{ 
         	pgd->upper_disp_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Vmax; 
         	pgd->lower_disp_limit = 0.0;
	} 
	else if (paddr->pfield==(void *)&pcaen->imon || 
                 paddr->pfield==(void *)&pcaen->iset /* ||
	         paddr->pfield==(void *)&pcaen->ihh || 
                 paddr->pfield==(void *)&pcaen->ill || 
                 paddr->pfield==(void *)&pcaen->ihi ||
                 paddr->pfield==(void *)&pcaen->ilo  */)
	{
         	pgd->upper_disp_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Imax/im; 
         	pgd->lower_disp_limit = 0.0;
	} 
	else if (paddr->pfield==(void *)&pcaen->rup || 
	          paddr->pfield==(void *)&pcaen->rdwn)
	{
          	pgd->upper_disp_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Ramp_max; 
          	pgd->lower_disp_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Ramp_min;
	}
        else recGblGetGraphicDouble(paddr,pgd);
	 
    	return(0);
}

static long get_control_double(struct dbAddr *paddr,struct dbr_ctrlDouble *pcd)
{
    	caenRecord	*pcaen=(caenRecord *)paddr->precord;
  	int i,im=1;
	
	
	for (i=0;i<crate_list[pcaen->crat].brd[pcaen->brd].Idec;i++) im*=10;

     	if(paddr->pfield==(void *)&pcaen->vmon || 
           paddr->pfield==(void *)&pcaen->vset/*  ||
	   paddr->pfield==(void *)&pcaen->vhh || 
           paddr->pfield==(void *)&pcaen->vll || 
           paddr->pfield==(void *)&pcaen->vhi ||
           paddr->pfield==(void *)&pcaen->vlo  */)
	{ 
         	pcd->upper_ctrl_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Vmax; 
         	pcd->lower_ctrl_limit = 0.0;
	} 
	else if (paddr->pfield==(void *)&pcaen->imon || 
                 paddr->pfield==(void *)&pcaen->iset /* ||
	         paddr->pfield==(void *)&pcaen->ihh || 
                 paddr->pfield==(void *)&pcaen->ill || 
                 paddr->pfield==(void *)&pcaen->ihi ||
                 paddr->pfield==(void *)&pcaen->ilo  */)
	{
         	pcd->upper_ctrl_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Imax/im; 
         	pcd->lower_ctrl_limit = 0.0;
	} 
	else if (paddr->pfield==(void *)&pcaen->rup || 
	          paddr->pfield==(void *)&pcaen->rdwn)
	{
          	pcd->upper_ctrl_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Ramp_max; 
          	pcd->lower_ctrl_limit = (float) crate_list[pcaen->crat].brd[pcaen->brd].Ramp_min;
	}
        else recGblGetControlDouble(paddr,pcd);
	 
    	return(0);
}

static long get_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char	  *pstring;
{
    struct caenRecord	*pcaen=(struct caenRecord *)paddr->precord;

    if(paddr->pfield==(void *)&pcaen->val)
    {
       if(pcaen->val==0) {
	  strcpy(pstring,"OFF");
       } else if(pcaen->val==1) {
	  strcpy(pstring,"ON");
       } else {
	  strcpy(pstring,"Illegal_Value");
       }
    } else if(paddr->pfield==(void *)&pcaen->pwst)
    {
       if(pcaen->pwst==0) {
	  strcpy(pstring,"OFF");
       } else if(pcaen->pwst==1) {
	  strcpy(pstring,"ON");
       } else {
	  strcpy(pstring,"Illegal_Value");
       }
    } 
    return(0);
}

static long get_enum_strs(paddr,pes)
    struct dbAddr *paddr;
    struct dbr_enumStrs *pes;
{
    struct caenRecord	*pcaen=(struct caenRecord *)paddr->precord;

      pes->no_str = 2;
      memset(pes->strs,'\0',sizeof(pes->strs));
      strcpy(pes->strs[0],"OFF");
      strcpy(pes->strs[1],"ON");   
    return(0);   
}

static long put_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char          *pstring;
{
    struct caenRecord     *pcaen=(struct caenRecord *)paddr->precord;

    if(paddr->pfield==(void *)&pcaen->val)
    {
       if(strcmp(pstring,"OFF")==0) pcaen->val = 0;
       else  if(strcmp(pstring,"ON")==0) pcaen->val = 1;
       else return(S_db_badChoice);
    } else if(paddr->pfield==(void *)&pcaen->pwst)
    {
       if(strcmp(pstring,"OFF")==0) pcaen->pwst = 0;
       else  if(strcmp(pstring,"ON")==0) pcaen->pwst = 1;
       else return(S_db_badChoice);
    }
    
    return(0);
}

static void alarm(caenRecord *pcaen)
{
	/* float		vmon,vhh,vhi,vll,vlo,vhys,vlal;
	float		imon,ihh,ihi,ill,ilo,ihys,ilal;
	unsigned short	vhhs,vhs,vlls,vls;
	unsigned short	ihhs,ihs,ills,ils; */
	short chst;

	chst=pcaen->chst;
	
/*	if (((chst&0xff60)==0)&&(!pcaen->dcha))
	{
	   recGblSetSevr(pcaen,STATE_ALARM,INVALID_ALARM);
	   return;
	}*/
	/* vhh = pcaen->vhh; vll = pcaen->vll; vhi = pcaen->vhi; vlo = pcaen->vlo;
	ihh = pcaen->ihh; ill = pcaen->ill; ihi = pcaen->ihi; ilo = pcaen->ilo;
	vhhs = pcaen->vhhs; vlls = pcaen->vlls; vhs = pcaen->vhs; vls = pcaen->vls;
	ihhs = pcaen->ihhs; ills = pcaen->ills; ihs = pcaen->ihs; ils = pcaen->ils;
	vmon = pcaen->vmon; vhys = pcaen->vhys; vlal = pcaen->vlal;
	imon = pcaen->imon; ihys = pcaen->ihys; ilal = pcaen->ilal; */

	/* alarm condition hihi */
	/* if (vhhs && (vmon >= vhh || ((vlal==vhh) && (vmon >= vhh-vhys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->vhhs)) pcaen->vlal = vhh;
		return;
	}
	if (ihhs && (imon >= ihh || ((ilal==ihh) && (imon >= ihh-ihys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->ihhs)) pcaen->ilal = ihh;
		return;
	} */

	/* alarm condition lolo */
	/* if (vlls && (vmon <= vll || ((vlal==vll) && (vmon <= vll+vhys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->vlls)) pcaen->vlal = vll;
		return;
	}
	if (ills && (imon <= ill || ((ilal==ill) && (imon <= ill+ihys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->ills)) pcaen->ilal = ill;
		return;
	} */

	/* alarm condition high */
	/* if (vhs && (vmon >= vhi || ((vlal==vhi) && (vmon >= vhi-vhys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->vhs)) pcaen->vlal = vhi;
		return;
	}
	if (ihs && (imon >= ihi || ((ilal==ihi) && (imon >= ihi-ihys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->ihs)) pcaen->ilal = ihi;
		return;
	} */

	/* alarm condition low */
	/* if (vls && (vmon <= vlo || ((vlal==vlo) && (vmon <= vlo+vhys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->vls)) pcaen->vlal = vlo;
		return;
	}
	if (ils && (imon <= ilo || ((ilal==ilo) && (imon <= ilo+ihys)))){
	        if (recGblSetSevr(pcaen,HIHI_ALARM,pcaen->ils)) pcaen->ilal = ilo;
		return;
	}  */
	
	
	/* alarm condition internal trip or kill */
	if (pcaen->stsv && ((chst&0x60)!=0)){
	   recGblSetSevr(pcaen,STATE_ALARM,pcaen->stsv);
	   return;
	}
	/* alarm condition over current */
	if (pcaen->socs && ((chst&0x1000)!=0)){
	   recGblSetSevr(pcaen,HIHI_ALARM,pcaen->socs);
	   return;
	}
	/* alarm condition over voltage */
	if (pcaen->sovs && ((chst&0x400)!=0)){
	   recGblSetSevr(pcaen,HIGH_ALARM,pcaen->sovs);
	   return;
	}
	/* alarm condition under voltage */
	if (pcaen->suvs && ((chst&0x800)!=0)){
	   recGblSetSevr(pcaen,LOW_ALARM,pcaen->suvs);
	   return;
	}
	/* alarm condition other */
	if (pcaen->sosv && ((chst&0x6300)!=0)){
	   recGblSetSevr(pcaen,STATE_ALARM,pcaen->sosv);
	   return;
	}
	
	/* alarm condition communication failure */
	if (crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].comm>pcaen->nbcf){
	   recGblSetSevr(pcaen,COMM_ALARM,pcaen->cfsv);
	   return;
	}
	
	/* we get here only if val is out of alarm by at least hyst */
	/* pcaen->vlal = vmon;
	pcaen->ilal = imon; */
	return;
}

static void monitor(caenRecord *pcaen)
{
	unsigned short	monitor_mask,mm;
	float		delta;

        monitor_mask = mm = recGblResetAlarms(pcaen);
	/* check for voltage value change */
	delta = pcaen->lvm - pcaen->vmon;
	if(delta<0.0) delta = -delta;
	if (delta > pcaen->vmde) {
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
		/* update last value monitored */
		pcaen->lvm = pcaen->vmon;
	}
	/* check for voltage archive value change */
	delta = pcaen->vlar - pcaen->vmon;
	if(delta<0.0) delta = -delta;
	if (delta > pcaen->vade) {
		/* post events for value change */
		monitor_mask |= DBE_LOG;
		/* update last value monitored */
		pcaen->vlar = pcaen->vmon;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
		db_post_events(pcaen,&pcaen->vmon,monitor_mask);
	
        monitor_mask = mm;
	/* check for value change */
	delta = pcaen->lim - pcaen->imon;
	if(delta<0.0) delta = -delta;
	if (delta > pcaen->imde) {
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
		/* update last value monitored */
		pcaen->lim = pcaen->imon;
	}
	/* check for archive value change */
	delta = pcaen->ilar - pcaen->imon;
	if(delta<0.0) delta = -delta;
	if (delta > pcaen->iade) {
		/* post events for value change */
		monitor_mask |= DBE_LOG;
		/* update last value monitored */
		pcaen->ilar = pcaen->imon;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
		db_post_events(pcaen,&pcaen->imon,monitor_mask);
		
        monitor_mask = mm;
	if (pcaen->chst != pcaen->lst)
	{
	   monitor_mask |= DBE_VALUE;
	   pcaen->lst = pcaen->chst;
	}
	if (monitor_mask)
	{
	   db_post_events(pcaen,&pcaen->chst,monitor_mask|DBE_LOG);
	   db_post_events(pcaen,&pcaen->pwst,monitor_mask);
	   db_post_events(pcaen,&pcaen->val,monitor_mask);
	}
	return;
}

static long caen_io(caenRecord *pcaen)
{
	int vm=1,im=1,i;
	
  	for (i=0;i<crate_list[pcaen->crat].brd[pcaen->brd].Vdec;i++) vm*=10;
  	for (i=0;i<crate_list[pcaen->crat].brd[pcaen->brd].Idec;i++) im*=10;
	
	if (semTake(sem_caen,6)==OK);	/* wait during 100 ms maximum */
	{
		pcaen->vmon=(float)crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Vmon/vm;
		pcaen->imon=(float)crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Imon/im;
		pcaen->chst=(crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Status)&0xff60;
		pcaen->pwst=(pcaen->chst&0x8000)!=0;
		crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].V0set=(int)((pcaen->vset)*vm);
		crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].I0set=(unsigned short)((pcaen->iset)*im);
		crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Rup=pcaen->rup;
		crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Rdwn=pcaen->rdwn;
		if(pcaen->dcha)
		  crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Power=0;
		else
		  crate_list[pcaen->crat].brd[pcaen->brd].chan[pcaen->chan].Power=pcaen->val;
		semGive(sem_caen);
		return 0;
	}
	return 1;
}


int async_caen(void)
{
	int i,j,k,err,crate_err=1;
	unsigned short flag,crate_status;
	
	while (1)				/* INFINITE LOOP */
	{
	  reada16(PIO2_IN,&crate_status);
	  for (i=0;i<NUMBER_OF_CRATE;i++){
	    if ((crate_list[i].name[0]!=0)&&(!(relay&(0x40+0x40*i)))&&(crate_status&(8+i*4))&&(crate_err=check_crate(&crate_list[i]))){	      
	      for (j=0;j<NB_BRD_MAX;j++)
		if (crate_list[i].brd[j].name[0]!=0)
		  for (k=0;k<NB_CHANNEL_MAX;k++)				/* For every channel */
		    if (crate_list[i].brd[j].chan[k].used==1)	/* if this channel is used */
		      {
			if(err=read_channel_status(&crate_list_copy[i],j,k)){	/* read voltage current and status */
			  crate_list_copy[i].brd[j].chan[k].comm++;
			  printf("Caen error (status) C=%d B=%d Ch=%d err=%d\n",i,j,k,err);
			}
			else
			  {
			    crate_list_copy[i].brd[j].chan[k].comm=0;
			    crate_list_copy[i].brd[j].chan[k].Status|=2; /* crate is on and ready */
			    
			    if (crate_list[i].brd[j].chan[k].V0set					/* if different */
				!=crate_list_copy[i].brd[j].chan[k].V0set)				/* from last value */
			      {   
				if(err=set_V0set(&crate_list[i],j,k,crate_list[i].brd[j].chan[k].V0set))	/* change voltage */
				  printf("Caen error (V0set) C=%d B=%d Ch=%d err=%d\n",i,j,k,err);
				else
				  crate_list_copy[i].brd[j].chan[k].V0set=crate_list[i].brd[j].chan[k].V0set;
			      }
			    if (crate_list[i].brd[j].chan[k].I0set
				!=crate_list_copy[i].brd[j].chan[k].I0set)
			      {  
				if(err=set_I0set(&crate_list[i],j,k,crate_list[i].brd[j].chan[k].I0set))	/* Idem for current */
				  printf("Caen error (I0set) C=%d B=%d Ch=%d err=%d\n",i,j,k,err);
				else
				  crate_list_copy[i].brd[j].chan[k].I0set=crate_list[i].brd[j].chan[k].I0set;
			      }
			    if (crate_list[i].brd[j].chan[k].Rup
				!=crate_list_copy[i].brd[j].chan[k].Rup)
			      {   
				if(err=set_Rup(&crate_list[i],j,k,crate_list[i].brd[j].chan[k].Rup))	/* Idem for Ramp Up */
				  printf("Caen error (Rup) C=%d B=%d Ch=%d err=%d\n",i,j,k,err);
				else
				  crate_list_copy[i].brd[j].chan[k].Rup=crate_list[i].brd[j].chan[k].Rup;
			      }
			    if (crate_list[i].brd[j].chan[k].Rdwn
				!=crate_list_copy[i].brd[j].chan[k].Rdwn)
			      {   
				if(err=set_Rdwn(&crate_list[i],j,k,crate_list[i].brd[j].chan[k].Rdwn))	/* Idem for Ramp down */
				  printf("Caen error (Rdwn) C=%d B=%d Ch=%d err=%d\n",i,j,k,err);
				else
				  crate_list_copy[i].brd[j].chan[k].Rdwn=crate_list[i].brd[j].chan[k].Rdwn;
			      }
			    if (crate_list[i].brd[j].chan[k].Power
				!=crate_list_copy[i].brd[j].chan[k].Power)
			      {   
				if (crate_list[i].brd[j].chan[k].Power!=0)
				  flag=0x0808;
				else
				  flag=0x0800;
				
				if(err=set_flag(&crate_list[i],j,k,flag))	/* Idem for Power */
				  printf("Caen error (flag) C=%d B=%d Ch=%d err=%d\n",i,j,k,err);
				else
				  crate_list_copy[i].brd[j].chan[k].Power=crate_list[i].brd[j].chan[k].Power;
			      }
			  }
		      }   
	    }
	    else{
	      for (j=0;j<NB_BRD_MAX;j++)
		if (crate_list[i].brd[j].name[0]!=0)
		  for (k=0;k<NB_CHANNEL_MAX;k++){				/* For every channel */
		    if(crate_status&(8*i*4))
		      crate_list_copy[i].brd[j].chan[k].Status=2;
		    else
		      crate_list_copy[i].brd[j].chan[k].Status=0;
		    crate_list_copy[i].brd[j].chan[k].Vmon=0;
		    crate_list_copy[i].brd[j].chan[k].Imon=0;
		  }
	    }
	    /*         reset_caen();	 */
	    taskDelay(ASYN_CAEN_TASK_DELAY/8);
	    
	  }			    
	  semTake(sem_caen,WAIT_FOREVER);		   /* Protect this process from the others */
	  for (i=0;i<NUMBER_OF_CRATE;i++)
	    crate_list[i]=crate_list_copy[i];  /* copy the intermadiate parameter list into the real one */
	  semGive(sem_caen);			   /* release the protection */
	  reset_caen();
	  taskDelay(ASYN_CAEN_TASK_DELAY);
	}
	return 0;					/* END INFINITE LOOP */
}

