/* caenSim.c		Simulation of Caen Sy527 functions 
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	Annan Fedoul, Jerome Baudot & David Bonnet
 *	Date :		20/10/00
 *
 */
 
#include "vxWorks.h"
#include "sysLib.h"
#include "stdio.h"
#include "caen.h"
#include <string.h>
char INIT;
v288_reg *v288;
static unsigned short buffer_fifo[256];
crate simCrateList[NUMBER_OF_CRATE];

void init_caen(crate *cr)
{
   int err,i,n,nb;
	
   INIT=0;
   v288=(v288_reg *)malloc(sizeof(v288_reg));	
   for(n=0;n<2;n++)
     {
     strcpy(simCrateList[n].name,"SY527");
     simCrateList[n].nb=n+1;
     for (nb=0;nb<5;nb++)
        {
	strcpy(simCrateList[n].brd[nb].name,"A525");
	simCrateList[n].brd[nb].CurrentUnits=0;
	simCrateList[n].brd[nb].Num_of_channels=8;
	simCrateList[n].brd[nb].Imax=300;
	simCrateList[n].brd[nb].Vmax=4;
	simCrateList[n].brd[nb].Ramp_max=4;
	simCrateList[n].brd[nb].Ramp_min=1;
	simCrateList[n].brd[nb].Vres=1;
	simCrateList[n].brd[nb].Ires=1;
	simCrateList[n].brd[nb].Vdec=2;
	simCrateList[n].brd[nb].Idec=2;
	for(i=0;i<8;i++)
	   strcpy(simCrateList[n].brd[nb].chan[i].name,"aaa");
	}
     for (nb=5;nb<8;nb++)
        {
	strcpy(simCrateList[n].brd[nb].name,"A516");
	simCrateList[n].brd[nb].CurrentUnits=0;
	simCrateList[n].brd[nb].Num_of_channels=8;
	simCrateList[n].brd[nb].Imax=150;
	simCrateList[n].brd[nb].Vmax=12;
	simCrateList[n].brd[nb].Ramp_max=10;
	simCrateList[n].brd[nb].Ramp_min=1;
	simCrateList[n].brd[nb].Vres=1;
	simCrateList[n].brd[nb].Ires=1;
	simCrateList[n].brd[nb].Vdec=2;
	simCrateList[n].brd[nb].Idec=2;
	for(i=0;i<8;i++)
	   strcpy(simCrateList[n].brd[nb].chan[i].name,"aaa");
	}
     for (nb=8;nb<10;nb++)
        {
	strcpy(simCrateList[n].brd[nb].name,"A515");
	simCrateList[n].brd[nb].CurrentUnits=2;
	simCrateList[n].brd[nb].Num_of_channels=16;
	simCrateList[n].brd[nb].Imax=20000;
	simCrateList[n].brd[nb].Vmax=100;
	simCrateList[n].brd[nb].Ramp_max=50;
	simCrateList[n].brd[nb].Ramp_min=1;
	simCrateList[n].brd[nb].Vres=4;
	simCrateList[n].brd[nb].Ires=5;
	simCrateList[n].brd[nb].Vdec=2;
	simCrateList[n].brd[nb].Idec=2;
	for(i=0;i<16;i++)
	   strcpy(simCrateList[n].brd[nb].chan[i].name,"aaa");
	}
     }
   /* Read the parameters of all crates */
   for (i=0;i<NUMBER_OF_CRATE;i++) 
   {
      cr[i].nb=CRATE_NB[i];
      if((err=read_crate(&cr[i]))!=0)
      printf("Caen Initialize : crate %d not found.(error=%d)\n",i,err);
   }
   INIT=1;
   return;
}


/* check the presence of the crate */

int check_crate(crate *cr)   
{
  unsigned short i;
   
  for(i=0;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) return 1;
  return 0;
}

/* read the parameters of the crate */

int read_crate(crate *cr)   
{
  unsigned short occup;
  int i,err=0,n;
 
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    {
    if(cr->nb==simCrateList[i].nb) n=i;
    /*printf("%d->%d : %d\n",i,cr->nb,simCrateList[i].nb);*/
    }
  if(n==-1) return 1; 
  
  for(i=0;i<NAME_CRATE_SIZE;i++) 
    cr->name[i]=simCrateList[n].name[i];
  
  /* read the occupation of the crate */
  occup=0;
  for(i=0;i<NB_BRD_MAX;i++)
     if(simCrateList[n].brd[i].name[0]!=0) occup|=(1<<i);
       
  /* read board caracteristics */
  
  for (i=0;i<NB_BRD_MAX;i++)
  {
      if (occup&(1<<i)) err=read_board(cr,i);
      else cr->brd[i].name[0]=0;
      if (err!=0) return 3;
  }
  
  return 0;
}

/* read the board parameters */

int read_board(crate *cr,unsigned short nb)
{
 int n,i,err=0;

  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
 for(i=0;i<NAME_BRD_SIZE;i++) 
     cr->brd[nb].name[i]=simCrateList[n].brd[nb].name[i];
     
 cr->brd[nb].CurrentUnits=simCrateList[n].brd[nb].CurrentUnits;
 cr->brd[nb].Ser_Num=simCrateList[n].brd[nb].Ser_Num;
 cr->brd[nb].Ver1=simCrateList[n].brd[nb].Ver1;
 cr->brd[nb].Ver2=simCrateList[n].brd[nb].Ver2;
 cr->brd[nb].Num_of_channels=simCrateList[n].brd[nb].Num_of_channels;
 cr->brd[nb].Vmax=simCrateList[n].brd[nb].Vmax;
 cr->brd[nb].Imax=simCrateList[n].brd[nb].Imax;
 cr->brd[nb].Ramp_min=simCrateList[n].brd[nb].Ramp_min;
 cr->brd[nb].Ramp_max=simCrateList[n].brd[nb].Ramp_max;
 cr->brd[nb].Vres=simCrateList[n].brd[nb].Vres;
 cr->brd[nb].Ires=simCrateList[n].brd[nb].Ires;
 cr->brd[nb].Vdec=simCrateList[n].brd[nb].Vdec;
 cr->brd[nb].Idec=simCrateList[n].brd[nb].Idec;
  
  for (i=0;i<cr->brd[nb].Num_of_channels;i++) 
  {
    err=read_channel(cr,nb,i);
    if (err!=0) return 1;
    if (INIT=0) cr->brd[nb].chan[i].used=0;
  }
 return 0;
}

/* read the channel parameters */

int read_channel(crate *cr,unsigned short nb,unsigned short nc)
{
 int n,i;
 
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
 for(i=0;i<NAME_BRD_SIZE;i++) 
     cr->brd[nb].chan[nc].name[i]=simCrateList[n].brd[nb].chan[nc].name[i];
 cr->brd[nb].chan[nc].V0set=simCrateList[n].brd[nb].chan[nc].V0set;
 cr->brd[nb].chan[nc].V1set=simCrateList[n].brd[nb].chan[nc].V1set;
 cr->brd[nb].chan[nc].I0set=simCrateList[n].brd[nb].chan[nc].I0set;
 cr->brd[nb].chan[nc].I1set=simCrateList[n].brd[nb].chan[nc].I1set;
 cr->brd[nb].chan[nc].Vmax=simCrateList[n].brd[nb].chan[nc].Vmax;
 cr->brd[nb].chan[nc].Rup=simCrateList[n].brd[nb].chan[nc].Rup;
 cr->brd[nb].chan[nc].Rdwn=simCrateList[n].brd[nb].chan[nc].Rdwn;
 cr->brd[nb].chan[nc].Trip=simCrateList[n].brd[nb].chan[nc].Trip;
 cr->brd[nb].chan[nc].Flag=simCrateList[n].brd[nb].chan[nc].Flag;
 
 return 0;
}


/* read the channel status */

int read_channel_status(crate *cr,unsigned short nb,unsigned short nc)
{
  int i,n;
 
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
    cr->brd[nb].chan[nc].Vmon=simCrateList[n].brd[nb].chan[nc].Vmon;
    cr->brd[nb].chan[nc].HVmax=simCrateList[n].brd[nb].chan[nc].HVmax;
    cr->brd[nb].chan[nc].Imon=simCrateList[n].brd[nb].chan[nc].Imon;
    cr->brd[nb].chan[nc].Status=simCrateList[n].brd[nb].chan[nc].Status;
    return 0;
}

/*********************************************************************************
*	 MODIFICATION DES PARAMETRES                                            *
*********************************************************************************/

int set_name(crate *cr,unsigned short nb,unsigned short nc,char *name)
{
  int i,n;
  
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
  for(i=0;i<12;i++)
     simCrateList[n].brd[nb].chan[nc].name[i]=name[i];
     
  return 0;
}

int set_V0set(crate *cr,unsigned short nb,unsigned short nc,int val)
{
  int i,n;
  
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
  simCrateList[n].brd[nb].chan[nc].V0set=val;
  return 0;
}

int set_I0set(crate *cr,unsigned short nb,unsigned short nc,unsigned short val)
{
  int i,n;
  
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
  simCrateList[n].brd[nb].chan[nc].I0set=val;
  return 0;
}


int set_Rup(crate *cr,unsigned short nb,unsigned short nc,unsigned short v)
{
  int i,n;
  
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
  simCrateList[n].brd[nb].chan[nc].Rup=v;
  return 0;
}

int set_Rdwn(crate *cr,unsigned short nb,unsigned short nc,unsigned short v)
{
  int i,n;
  
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
  simCrateList[n].brd[nb].chan[nc].Rdwn=v;
  return 0;
}
 

int set_flag(crate *cr,unsigned short nb,unsigned short nc,unsigned short f)
{
  int i,n;
  
  for(i=0,n=-1;i<NUMBER_OF_CRATE;i++)
    if(cr->nb==simCrateList[i].nb) n=i;
  if(n==-1) return 1; 
  
  simCrateList[n].brd[nb].chan[nc].Flag=f;
  simCrateList[n].brd[nb].chan[nc].Status=(f&8)<<12;
  if (f&8)
    simCrateList[n].brd[nb].chan[nc].Vmon=simCrateList[n].brd[nb].chan[nc].V0set;
  else simCrateList[n].brd[nb].chan[nc].Vmon=0; 
     
  return 0;
}
