/* jtagCommon.h		Jtag common functions header 
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	David Bonnet
 *	Date :		22/03/00
 *
 */
 

/* hard_reset() parameters */
#define TBC_INTERNAL    0       /* connect to on-board 8244 JTAG devices */
#define TBC_EXTERNAL    1       /* connect to off-board JTAG targets */

#define CLK_25MHZ       0       /* Internal 25MHz clock source */
#define CLK_781KHZ      1       /* Internal 781KHz clock source */
#define VMEBUS_TTLTRG   2       /* VMEbus TTLTRG signal */
#define EXTERNAL_CLOCK  3       /* External Clock input */

#define TBC_DIV_BY_2    0       /* divide 25MHz clock by 2 */
#define TBC_DIV_BY_4    1       /* divide 25MHz clock by 4 */
#define TBC_DIV_BY_6    2       /* divide 25MHz clock by 6 */
#define TBC_DIV_BY_8    3       /* divide 25MHz clock by 8 */
#define TBC_DIV_BY_10   4       /* divide 25MHz clock by 10 */
#define TBC_DIV_BY_12   5       /* divide 25MHz clock by 12 */
#define TBC_DIV_BY_14   6       /* divide 25MHz clock by 14 */
#define TBC_DIV_BY_16   7       /* divide 25MHz clock by 16 */
#define TBC_DIV_BY_18   8       /* divide 25MHz clock by 18 */
#define TBC_DIV_BY_20   9       /* divide 25MHz clock by 20 */
#define TBC_DIV_BY_22   10      /* divide 25MHz clock by 22 */
#define TBC_DIV_BY_24   11      /* divide 25MHz clock by 24 */
#define TBC_DIV_BY_26   12      /* divide 25MHz clock by 26 */
#define TBC_DIV_BY_28   13      /* divide 25MHz clock by 28 */
#define TBC_DIV_BY_30   14      /* divide 25MHz clock by 30 */
#define TBC_DIV_BY_32   15      /* divide 25MHz clock by 32 */

/* Logical constants */
#define TRUE        1
#define FALSE       0

/* Register offsets within the TBC register space */
#define VME_ID      0x00    /* VMEbus ID register */
#define VME_DEVTYPE 0x02    /* VMEbus Device Type Register */
#define VME_STATUS  0x04    /* VMEbus Status Register */
#define VME_TSR     0x06    /* Trigger-in Select Register */
#define PIO1_OUT    0x08    /* Parallel port output register #1 */
#define PIO1_IN     0x08    /* Parallel port input register #1 */
#define PIO2_OUT    0x0a    /* Parallel port output register #2 */
#define PIO2_IN     0x0a    /* Parallel port input register #2 */
#define CFG_REG1    0x0c    /* Configuration Register #1 */
#define CFG_REG2    0x0e    /* Configuration Register #2 */
#define CONTROL0    0x10    /* TBC Control Register 0 */
#define CONTROL1    0x12    /* TBC Control Register 1 */
#define CONTROL2    0x14    /* TBC Control Register 2 */
#define CONTROL3    0x16    /* TBC Control Register 3 */
#define CONTROL4    0x18    /* TBC Control Register 4 */
#define CONTROL5    0x1a    /* TBC Control Register 5 */
#define CONTROL6    0x1c    /* TBC Control Register 6 */
#define CONTROL7    0x1e    /* TBC Control Register 7 */
#define CONTROL8    0x20    /* TBC Control Register 8 */
#define CONTROL9    0x22    /* TBC Control Register 9 */
#define MINOR       0x24    /* TBC Minor command register */
#define MAJOR       0x26    /* TBC Major command register */
#define CNT1UP0     0x28    /* TBC counter 1 update 0 */
#define CNT1UP1     0x2a    /* TBC counter 1 update 1 */
#define CNT2UP0     0x2c    /* TBC counter 2 update 0 */
#define CNT2UP1     0x2e    /* TBC counter 2 update 1 */
#define STATUS0     0x30    /* TBC STATUS0 register */
#define STATUS1     0x32    /* TBC STATUS1 register */
#define STATUS2     0x34    /* TBC STATUS2 register */
#define STATUS3     0x36    /* TBC STATUS3 register */
#define CAPTURE0    0x38    /* TBC counter 1 capture 0 */
#define CAPTURE1    0x3a    /* TBC counter 1 capture 1 */
#define READBUF     0x3c    /* TBC Read buffer */
#define WRITEBUF    0x3e    /* TBC Write buffer */

/* JTAG State Definitions */
#define TEST_LOGIC_RESET        0x00
#define RUN_TEST_IDLE           0x02
#define SHIFT_DR                0x04
#define SHIFT_IR                0x05
#define PAUSE_DR                0x06
#define PAUSE_IR                0x07

#define IR_SZ			5
#define BYPASS_SZ		1
#define IDENT_SZ		8
#define NB_MAX_IR  		120
#define NB_MAX_BSR  		120
#define MAX_LENGTH_STREAM 	350
#define MAX_LENGTH_STREAM_IR	40
#define MAX_LENGTH_STREAM_8	60
#define MAX_LENGTH_STREAM_16	120
#define MAX_LENGTH_STREAM_1	8
#define LADDER_NB		40
#define MODULE_NB		16
#define MAX_CHIPS 		120
#define COMPRESSED_MAP_LENGTH   32

#define DEF_CO_ID 0
typedef enum {COSTAR,ALICE128C,END,CONNEXION,READOUT} chip;
/* Function definitions	*/

short 		writea16(unsigned short, unsigned short);
short 		reada16(unsigned short, unsigned short *);
short 		hard_reset(unsigned short, unsigned short, unsigned short);
void 		soft_reset(void);
void 		wait_for_ready(void);
short 		check_ready(void);
void 		write_tbc(unsigned short, unsigned short);
void 		read_tbc(unsigned short, unsigned short *);
void 		write_buf(unsigned short);
void 		read_buf(unsigned short *);
void 		wait_jtag_state(short);
void 		scan_ir(unsigned short *, unsigned short, unsigned short *);
void 		scan_dr(unsigned short *, unsigned short, unsigned short *);
void 		circulate_dr(unsigned short, unsigned short *);
unsigned short 	map_nb(chip *);
unsigned short 	bit_to_word(unsigned short);
void		init_stream(unsigned short *, unsigned short);
void		set_stream(unsigned short *, unsigned short, unsigned long, unsigned short);
unsigned long	get_stream(unsigned short *,unsigned short, unsigned short); 
void 		bypass_all_chips(chip *);
unsigned short 	detect_length();
void            clear_bypass();
int 		auto_init_map(chip *);
void 		init_map(chip *,char *);
int 		jtag_init(chip *,char *);
void 		uncompress_map(chip *,char *);
void 		compress_map(chip *,char *);
void 		ascii_map(chip *,char *);
short 		reset_jtag();
short reset_jtag_var(unsigned short i,unsigned short j,unsigned short k);
void 		connect_tap(char tap);
