/* jtagCommonSimul.c		Simulates the behaviour of ALICE128C, COSTAR and FPGA connexion chips
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	David Bonnet
 *	Date :		30/05/00
 *
 */
 
#include <string.h> 
#include "vxWorks.h"
#include "sysLib.h"
#include "stdio.h"
#include "jtagCommon.h"
#include "jtagCostar.h"
#include "jtagA128C.h"
#include "jtagConnexion.h"
#include "jtagReadout.h"


int JTAG_INIT_DONE=0;
unsigned short logical_address = 32;
long a16base = 0xc800;
unsigned char tck_delay = 0;
int nbLadder=40;
int nbModule=16;
unsigned short latchupInfoSimN0 = 0;
unsigned short latchupInfoSimP0 = 0;
unsigned short latchupInfoSimN1 = 0;
unsigned short latchupInfoSimP1 = 0;
unsigned long simScIr = 0;
unsigned long simScDr = 0;
unsigned long simCiDr = 0;
chip tcnx[3]={CONNEXION,READOUT,END};
chip tread[2]={READOUT,END};
void jtagSimulInit();

#define SIM_NB (2+MODULE_NB*7)

typedef struct
{
   unsigned char bias[7];
   unsigned char pulse;
   unsigned char onoff;
   unsigned char token;
   unsigned short test[8];   
   unsigned char bypass;
   unsigned char ident;
}a128c_reg ;

typedef struct
{
   unsigned char ctrl1;
   unsigned char ctrl2;
   unsigned char dinout;
   unsigned long dac;
   unsigned long adc0;
   unsigned long adc1;
   unsigned char bypass;
   unsigned char ident;
}costar_reg ;

typedef struct
{
   unsigned short pow_status;
   unsigned short pow_on;
   unsigned char latchup;
   unsigned char vref;
   unsigned short hyb_bypass;
   unsigned char bypass;
   unsigned char ident;
}connex_reg ;

typedef struct
{
   unsigned short version[3];
   unsigned int config;
   unsigned char latchup;
   unsigned short status;
   unsigned char chain;
   unsigned int temp1;
   unsigned char bypass;
   unsigned char ident;
}readout_reg ;

typedef struct
{
   chip   type;
   unsigned char hard_bypass;
   union
   {
   costar_reg c;
   a128c_reg  a;
   connex_reg x;
   readout_reg r;
   } typ;
} chip_reg;

chip_reg tab_simul[LADDER_NB][SIM_NB];

readout_reg readout[4];

unsigned char tab_simul_ir[LADDER_NB][SIM_NB];

unsigned char cnx_module[LADDER_NB][MODULE_NB];

unsigned short cvme_reg[0x3f];

unsigned short lad=0;

/*************************************************************************

       Function: writea16()

        Summary: Writes a 16-bit value to VMEbus A16 space

          Usage: short writea16(addr, data)
                 unsigned short addr;
                 unsigned short data;

    Description: This routine writes the specified 16-bit value to the
                 register offset specified by addr.

*************************************************************************/

short writea16(unsigned short addr,unsigned short data)
{
   /*printf(" writea16 : addr :%d data:%d\n",addr,data);*/
   cvme_reg[addr]=data;
   return 0;
}

/*************************************************************************

       Function: reada16()

        Summary: Reads a 16-bit value from VMEbus A16 space

          Usage: short writea16(addr, readval)
                 unsigned short addr;
                 unsigned short *readval;

    Description: This routine reads a 16-bit value from the
                 register offset specified by addr.

*************************************************************************/

short reada16(unsigned short addr,unsigned short *readval)
{
 /*printf(" reada16 : addr :%d \n",addr);*/
  *readval=cvme_reg[addr];
    return 0;
}


/*************************************************************************

       Function: wait_for_ready()

        Summary: Returns when ACT8990 is ready to be accessed

          Usage: void wait_for_ready(void)

    Description: This routine does not return until the ACT8990 *RDY line
                 goes low, indicating the chip is ready to be accessed.
                 This function has no parameters.

**************************************************************************/

void wait_for_ready()
{
    return;
}


/*************************************************************************

       Function: check_ready()

        Summary: Returns the value of the ACT8990 *RDY line

          Usage: void check_ready(void)

    Description: This routine returns the value of the ACT8990 *RDY line.
                 This function has no parameters.

**************************************************************************/

short check_ready()
{
    return 1;         /* return status of RDY bit */
}


/*************************************************************************

       Function: write_tbc()

        Summary: Writes a value to one of the TBC registers

          Usage: void write_tbc(register_offset, data)
                 unsigned short register_offset;
                 unsigned short data;

    Description: This routine first checks the register_offset to make sure
                 it is a valid offset.  If it is not, the function returns.
                 When the ACT8990 is ready to be accessed, the specified
                 data is written to the register.  This function has two
                 parameters.  The 'register_offset' parameter specifies
                 which of the ACT8990 registers is to be written to.  The
                 'data' parameter holds the data to be written.

**************************************************************************/

void write_tbc(unsigned short register_offset,unsigned short data)
{
   cvme_reg[register_offset]=data;
}


/*************************************************************************

       Function: read_tbc()

        Summary: Writes a value to one of the TBC registers

          Usage: void write_tbc(register_offset, data)
                 unsigned short register_offset;
                 unsigned short *data;

    Description: This routine first checks the register_offset to make sure
                 it is a valid offset.  If it is not, the function returns.
                 When the ACT8990 is ready to be accessed, the specified
                 register is read and the data is returned.  This function
                 has two parameters.  The 'register_offset' parameter 
                 specifies which ACT8990 register is to be read.  The 'data'
                 parameter contains the address where the read data is to
                 be stored.

**************************************************************************/

void read_tbc(unsigned short register_offset,unsigned short *data)
{
  *data=cvme_reg[register_offset];  
}


/*************************************************************************

       Function: write_buf()

        Summary: Writes data to the JTAG data output register

          Usage: void write_buf(data)
                 unsigned short data;

    Description: This routine polls the STATUS2 register until the
                 write_ok flag is clear.  At that time, it writes the
                 specified data to the JTAG write buffer.  This function
                 has one parameter.  The 'data' parameter specifies the
                 data to be written to the JTAG output buffer.

**************************************************************************/

void write_buf(unsigned short data)
{
  printf(" write_buf : data :%x\n",data);  
}


/*************************************************************************

       Function: read_buf()

        Summary: Reads data from the JTAG data input buffer

          Usage: void read_buf(data)
                 unsigned short *data;

    Description: This routine polls the STATUS2 register until the
                 read_ok flag is clear.  At that time, it reads the
                 JTAG read buffer, and stores the read data.  This
                 fuction has one parameter.  The 'data' parameter 
                 contains the address where to store the data read
                 from the JTAG input buffer.

**************************************************************************/

void read_buf(unsigned short *data)
{
  printf(" read_buf \n");
  *data=0;  
}

/*************************************************************************

       Function: wait_jtag_state()

        Summary: Waits for the JTAG bus to transition to a particular state

          Usage: void wait_jtag_state(state);
                 unsigned short state;

    Description: This routine polls the STATUS2 register until the
                 JTAG bus is in the specified state.

**************************************************************************/

void wait_jtag_state(short state)
{
   return;
}


/*************************************************************************

       Function: hard_reset()

        Summary: Provides a hardware reset of the ACT8990

          Usage: short hard_reset(target, divider)
                 unsigned short target;
                 unsigned short divider;

    Description: This routine causes the PC-AT bus board to provide a
                 hardware reset of the 8990 IC by pulsing the 8990
                 *TRST input.  This also pulses the target *TRST signal.
                 There are two inputs to this function.  The 'target'
                 parameters specifies whether the board is to connect
                 the JTAG path to the on-board JTAG 8244 devices or to the
                 external JTAG target.  The second parameter specifies the
                 TCLK divider to use.

                 This function can also be used to verify the existance
                 of the JTAG board in a system.  When values are written
                 to the board, they are read back to verify the board is
                 operating normally.  If the board is responding, this
                 function returns TRUE.  If the board is not responding,
                 this function returns FALSE.  Extensive checking is not
                 done to verify the board responding is indeed a PC-1149.1.

**************************************************************************/

short hard_reset(unsigned short target,unsigned short clock_source,unsigned short divider)
{
  jtagSimulInit();
  return FALSE;
}

/*************************************************************************

       Function: soft_reset()

        Summary: Provides a software reset of the ACT8990

          Usage: void soft_reset(void)

    Description: This routine causes the PC-AT bus board to provide a
                 software reset of the 8990 IC by issuing a MINOR reset
                 command.  This function has no parameters.

**************************************************************************/

void soft_reset()
{
   printf(" soft reset \n");  
}


unsigned short calculate_real_length(void)
{
  unsigned short i,rl=0;

   for(i=0;i<SIM_NB;i++)
    if (tab_simul[lad][i].hard_bypass&1)
    {
      if (tab_simul[lad][i].type==COSTAR)
      {
          switch(tab_simul_ir[lad][i])
          {
	  case CO_CTRL_1 : rl+=CO_CTRL_SZ; 
	     break;

	  case CO_CTRL_2 : rl+=CO_CTRL_SZ;
	     break;
	  
	  case CO_DAC_0 : rl+=CO_DAC_SZ;
	     break;
	  
	  case  CO_DINOUT : rl+=CO_DINOUT_SZ;
	     break;
	  
	  case CO_BYPASS : rl+=CO_BYPASS_SZ;
	     break;
	  
	  case CO_ADC_0 : rl+=CO_ADC_SZ;
	     break;
	  
	  case CO_ADC_1 : rl+=CO_ADC_SZ;
	     break;
	  
	  case CO_IDENT : rl+=CO_IDENT_SZ;
	     break;
	  
	  default : rl+=CO_BYPASS_SZ;
 	     break;
	  }
       }
       else if (tab_simul[lad][i].type==ALICE128C) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case AL_BYPASS : rl+=AL_BYPASS_SZ;
	     break;
	     
           case AL_BIAS : rl+=AL_BIAS_SZ;
	     break;
       
           case AL_PULSE : rl+=AL_PULSE_SZ;
             break;
	     
           case AL_ONOFF : rl+=AL_ONOFF_SZ;
             break;
	     
           case AL_TOKEN : rl+=AL_TOKEN_SZ;
             break;
          
	   case AL_TEST : rl+=AL_TEST_SZ;
	     break;
	     
	   case CO_IDENT : rl+=CO_IDENT_SZ;
	     break;
	  
	   default : rl+=AL_BYPASS_SZ; 
	     break;
	  }	     

       }
       else if (tab_simul[lad][i].type==CONNEXION) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case CX_BYPASS : rl+=CX_BYPASS_SZ;
	     break;
	     
           case CX_LATCHUP : rl+=CX_LATCHUP_SZ;
	     break;
       
           case CX_STATUS : rl+=CX_STATUS_SZ;
             break;
	     
           case CX_VREF : rl+=CX_VREF_SZ;
             break;
	     
           case CX_POWER : rl+=CX_POWER_SZ;
             break;
          
	   case CX_HYB_BYP : rl+=CX_HYB_BYP_SZ;
	     break;
	     
	   case CX_IDENT : rl+=CX_IDENT_SZ;
	     break;
	  
	   default : rl+=CX_BYPASS_SZ; 
	     break;
	  }	     

       }
       else if (tab_simul[lad][i].type==READOUT) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case RD_BYPASS : rl+=RD_BYPASS_SZ;
	     break;
	     
           case RD_LATCHUP : rl+=RD_LATCHUP_SZ;
	     break;
       
           case RD_STATUS : rl+=RD_STATUS_SZ;
             break;
	     
           case RD_CONFIG : rl+=RD_CONFIG_SZ;
             break;
	     
           case RD_VERSION : rl+=RD_VERSION_SZ;
             break;
          
	   case RD_CHAIN : rl+=RD_CHAIN_SZ;
	     break;
	     
	   case RD_TEMP1 : rl+=RD_TEMP1_SZ;
	     break;
	     
	   case RD_TEMP2 : rl+=RD_TEMP2_SZ;
	     break;
	     
	   case RD_IDENT : rl+=RD_IDENT_SZ;
	     break;
	  
	   default : rl+=RD_BYPASS_SZ; 
	     break;
	  }	     

       }
    }
    
    return rl;
}


void actu_bypass()
{
   unsigned char i,j=0;
   if (readout[lad/10].chain!=3) tab_simul[lad][7*MODULE_NB].hard_bypass=0; 
   else tab_simul[lad][7*MODULE_NB].hard_bypass=1;
   
   for(i=0;i<MODULE_NB;i++)
      for(;j<cnx_module[lad][i];j++)
         {
	   if((tab_simul[lad][j].hard_bypass==2)&&((tab_simul[lad][7*MODULE_NB].typ.x.hyb_bypass&(1<<i))==0))
	    	{
		   tab_simul[lad][j].hard_bypass=0;
		   tab_simul[lad][MODULE_NB*7].typ.x.pow_status|=(1<<i);
		   tab_simul[lad][MODULE_NB*7].typ.x.latchup=0;
    		   readout[lad/10].status&=~(1<<5);
		   readout[lad/10].latchup=0;
		}

	    tab_simul[lad][j].hard_bypass&=2;
	    if (readout[lad/10].chain==3)
	       tab_simul[lad][j].hard_bypass|=((tab_simul[lad][7*MODULE_NB].typ.x.hyb_bypass&(1<<i))!=0);
	 }
   /* printf("actu_bypass:%d ",tab_simul[lad][7*MODULE_NB].typ.x.hyb_bypass);
   for (i=0;i<NB_MAX_A128C;i++) printf("%d",tab_simul[lad][i].hard_bypass);
   printf("\n");*/ 
}

/*************************************************************************

       Function: scan_dr()

        Summary: Scans a bit stream out the TAP data path

          Usage: void scan_dr(output, length, input)
                 unsigned short *output;
                 unsigned short length;
                 unsigned short *input;

    Description: This routine scans an arbitrary length bit stream out
                 the target TAP controller data path.  When a scan starts,
                 the RX-FIFO holds garbage, so we always shift out a 
                 16-bit dummy value first to take out the garbage.
                 This function has three parameters.  The 'output' variable
                 is an address to an array of 16-bit values to be scanned
                 out the JTAG port.  The 'length' parameter specifies the
                 length of the bitstream.  The 'input' variable is an address
                 to an array of 16-bit values where the data scanned in is
                 to be stored.

**************************************************************************/

void scan_dr(unsigned short *output1,unsigned short length,unsigned short *input)
{
  unsigned short numregs,i,str,ws,j,real_length=0,output[MAX_LENGTH_STREAM];
  short str_f=-1;
  unsigned char a_b=0;
  int stroff;

  simScDr+=length;
    /* determine number of registers to use in data transfer */
    numregs = length / 16;

    if((numregs * 16) != length)
        ++numregs;

   /* if the data stream is shorter than than the "real" jtag chain, modify this data stream */   
   real_length=calculate_real_length();
   stroff=real_length-length;
   if(stroff>=0) 
      for(i=0;i<numregs;i++)
         set_stream(output,stroff+i*16,output1[i],16);
   	
    numregs = real_length / 16;

    if((numregs * 16) != real_length)
        ++numregs;

   for(i=0,str=0;i<SIM_NB;i++)
   {
      if ((tab_simul[lad][i].hard_bypass&1)==0) continue;
	    
      if ((tab_simul[lad][i].hard_bypass&2)==2) str_f=str;
      
      if (tab_simul[lad][i].type==COSTAR)
      {
          switch(tab_simul_ir[lad][i])
          {
	  case CO_CTRL_1 : 
	     set_stream(input,str,tab_simul[lad][i].typ.c.ctrl1,8);
	     tab_simul[lad][i].typ.c.ctrl1=get_stream(output,str,8);
	     str+=8;
	     break;

	  case CO_CTRL_2 :
	     set_stream(input,str,tab_simul[lad][i].typ.c.ctrl2,8);
	     tab_simul[lad][i].typ.c.ctrl2=get_stream(output,str,8);
	     str+=8;
	     break;
	  
	  case CO_DAC_0 :
	     set_stream(input,str,tab_simul[lad][i].typ.c.dac,32);
	     tab_simul[lad][i].typ.c.dac=get_stream(output,str,32);
	     str+=32;
	     break;
	  
	  case  CO_DINOUT : 
	     set_stream(input,str,tab_simul[lad][i].typ.c.dinout,8);
	     tab_simul[lad][i].typ.c.dinout=get_stream(output,str,8);
	     str+=8;
	     break;
	  
	  case CO_BYPASS : 
	     set_stream(input,str,tab_simul[lad][i].typ.c.bypass,1);
	     tab_simul[lad][i].typ.c.bypass=get_stream(output,str,1);
	     str+=1;
	     break;
	  
	  case CO_ADC_0 :
	     set_stream(input,str,tab_simul[lad][i].typ.c.adc0,32);
	     tab_simul[lad][i].typ.c.adc0=get_stream(output,str,32);
	     str+=32;
	     break;
	  
	  case CO_ADC_1 :
	     set_stream(input,str,tab_simul[lad][i].typ.c.adc1,32);
	     tab_simul[lad][i].typ.c.adc0=get_stream(output,str,32);
	     str+=32;
	     break;
	  
	  case CO_IDENT :
	     set_stream(input,str,tab_simul[lad][i].typ.c.ident,8);
	     tab_simul[lad][i].typ.c.ident=get_stream(output,str,8);
	     str+=8;
	     break;
	  
	  default : 
	     set_stream(input,str,tab_simul[lad][i].typ.c.bypass,1);
	     tab_simul[lad][i].typ.c.bypass=get_stream(output,str,1);
	     str+=1;
	     break;
	  }
       }
       else if (tab_simul[lad][i].type==ALICE128C) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case AL_BYPASS : 
	     set_stream(input,str,tab_simul[lad][i].typ.a.bypass,1);
	     tab_simul[lad][i].typ.a.bypass=get_stream(output,str,1);
	     str+=1;
	     break;
	     
           case AL_BIAS :
       	     for (j=0;j<7;j++)
	     {
	        set_stream(input,str,tab_simul[lad][i].typ.a.bias[j],8);
	        tab_simul[lad][i].typ.a.bias[j]=get_stream(output,str,8);
	        str+=8;
	     }
	     break;
       
           case AL_PULSE :
       	     set_stream(input,str,tab_simul[lad][i].typ.a.pulse,8);
	     tab_simul[lad][i].typ.a.pulse=get_stream(output,str,8);
	     str+=8;
             break;
	     
           case AL_ONOFF :
       	     set_stream(input,str,tab_simul[lad][i].typ.a.onoff,1);
	     tab_simul[lad][i].typ.a.onoff=get_stream(output,str,1);
	     str+=1;
             break;
	     
           case AL_TOKEN :
       	     set_stream(input,str,tab_simul[lad][i].typ.a.token,1);
	     tab_simul[lad][i].typ.a.token=get_stream(output,str,1);
	     str+=1;
             break;
          
	   case AL_TEST :
       	     for (j=0;j<8;j++)
	     {
       	        set_stream(input,str,tab_simul[lad][i].typ.a.test[j],8);
	        tab_simul[lad][i].typ.a.test[j]=get_stream(output,str,8);
	        str+=8;
       	     }
	     break;
	     
	   case CO_IDENT :
	     set_stream(input,str,tab_simul[lad][i].typ.a.ident,8);
	     tab_simul[lad][i].typ.a.ident=get_stream(output,str,8);
	     str+=8;
	     break;
	  
	   default : 
	     set_stream(input,str,tab_simul[lad][i].typ.a.bypass,1);
	     tab_simul[lad][i].typ.a.bypass=get_stream(output,str,1);
	     str+=1;
	     break;
	     
	  }	     

       }
       else if (tab_simul[lad][i].type==CONNEXION) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case CX_BYPASS : 
	     set_stream(input,str,tab_simul[lad][i].typ.x.bypass,CX_BYPASS_SZ);
	     tab_simul[lad][i].typ.x.bypass=get_stream(output,str,CX_BYPASS_SZ);
	     str+=CX_BYPASS_SZ;
	     break;
	     
           case CX_LATCHUP :
	     set_stream(input,str,tab_simul[lad][i].typ.x.latchup,CX_LATCHUP_SZ);
	     tab_simul[lad][i].typ.x.latchup=get_stream(output,str,CX_LATCHUP_SZ);
	     str+=CX_LATCHUP_SZ;
	     break;
       
           case CX_STATUS :
	     set_stream(input,str,tab_simul[lad][i].typ.x.pow_status,CX_STATUS_SZ);
	     tab_simul[lad][i].typ.x.pow_status=get_stream(output,str,CX_STATUS_SZ);
	     str+=CX_STATUS_SZ;
             break;
	     
           case CX_VREF : 
	     set_stream(input,str,tab_simul[lad][i].typ.x.vref,CX_VREF_SZ);
	     tab_simul[lad][i].typ.x.vref=get_stream(output,str,CX_VREF_SZ);
	     str+=CX_VREF_SZ;
             break;
	     
           case CX_POWER :
	     set_stream(input,str,tab_simul[lad][i].typ.x.pow_on,CX_POWER_SZ);
	     tab_simul[lad][i].typ.x.pow_on=get_stream(output,str,CX_POWER_SZ);
	     str+=CX_POWER_SZ;
             break;
          
	   case CX_HYB_BYP :
	     set_stream(input,str,tab_simul[lad][i].typ.x.hyb_bypass,CX_HYB_BYP_SZ);
	     tab_simul[lad][i].typ.x.hyb_bypass=get_stream(output,str,CX_HYB_BYP_SZ);
	     str+=CX_HYB_BYP_SZ;
	     a_b=1;
	     break;
	     
	   case CX_IDENT :
	     set_stream(input,str,tab_simul[lad][i].typ.x.ident,CX_IDENT_SZ);
	     tab_simul[lad][i].typ.x.ident=get_stream(output,str,CX_IDENT_SZ);
	     str+=CX_IDENT_SZ;
	     break;
	  
	   default : 
	     set_stream(input,str,tab_simul[lad][i].typ.x.bypass,CX_BYPASS_SZ);
	     tab_simul[lad][i].typ.x.bypass=get_stream(output,str,CX_BYPASS_SZ);
	     str+=CX_BYPASS_SZ;
	     break;
	  }	     

       }
       else if (tab_simul[lad][i].type==READOUT) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case RD_BYPASS : 
	     set_stream(input,str,readout[lad/10].bypass,RD_BYPASS_SZ);
	     readout[lad/10].bypass=get_stream(output,str,RD_BYPASS_SZ);
	     str+=RD_BYPASS_SZ;
	     break;
	     
           case RD_LATCHUP :
	     set_stream(input,str,readout[lad/10].latchup,RD_LATCHUP_SZ);
	     readout[lad/10].latchup=get_stream(output,str,RD_LATCHUP_SZ);
	     str+=RD_LATCHUP_SZ;
	     break;
       
           case RD_STATUS :
	     set_stream(input,str,readout[lad/10].status,RD_STATUS_SZ);
	     readout[lad/10].status=get_stream(output,str,RD_STATUS_SZ);
	     str+=RD_STATUS_SZ;
             break;
	     
           case RD_VERSION : 
	   for(j=0;j<3;j++)
	   {
	     set_stream(input,str,readout[lad/10].version[j],16);
	     readout[lad/10].version[j]=get_stream(output,str,16);
           }
	     str+=RD_VERSION;
	     break;
	     
           case RD_CONFIG :
	     set_stream(input,str,readout[lad/10].config,RD_CONFIG_SZ);
	     readout[lad/10].config=get_stream(output,str,RD_CONFIG_SZ);
	     str+=RD_CONFIG_SZ;
             break;
          
           case RD_CHAIN :
	     set_stream(input,str,readout[lad/10].config,RD_CHAIN_SZ);
	     readout[lad/10].chain=get_stream(output,str,RD_CHAIN_SZ);
	     str+=RD_CHAIN_SZ;
	     a_b=1;
             break;
          
	   case RD_TEMP1 :
	     set_stream(input,str,readout[lad/10].temp1,RD_TEMP1_SZ);
	     readout[lad/10].temp1=get_stream(output,str,RD_TEMP1_SZ);
	     str+=RD_TEMP1_SZ;
	     a_b=1;
	     break;
	     
	   case RD_IDENT :
	     set_stream(input,str,readout[lad/10].ident,RD_IDENT_SZ);
	     readout[lad/10].ident=get_stream(output,str,RD_IDENT_SZ);
	     str+=RD_IDENT_SZ;
	     break;
	  
	   default : 
	     set_stream(input,str,readout[lad/10].bypass,RD_BYPASS_SZ);
	     readout[lad/10].bypass=get_stream(output,str,RD_BYPASS_SZ);
	     str+=RD_BYPASS_SZ;
	     break;
	  }	     

       }
    }
    if(a_b) actu_bypass();

   /* if a dead chip is simulated (hard_bypass=2 or 3), modify the input stream */
   if (str_f!=-1)
   {
      ws=bit_to_word(str_f);
      set_stream(input,str_f,0xffff,16-(str_f%16));
      for(ws++;ws<numregs;ws++) input[ws]=0xffff;
   }
/*
   printf("scan dr :");
   for(x=0;x<numregs;x++)
    {
    printf("%4x ",output[x]);
    input[x]=output[x];
    }
   printf("\nlength = %d\n",length);*/
   
}

void scan_ir(unsigned short *output1,unsigned short length,unsigned short *input)
{
  unsigned short i,output[MAX_LENGTH_STREAM_IR];
  int sh=0,real_length=0,ioff;

  simScIr+=length;
   /* Calculate real length */
   for(i=0;i<SIM_NB;i++)
      if(tab_simul[lad][i].hard_bypass&1) real_length++;

   init_stream(output,real_length*5);
   
   ioff=real_length-(length/5);
   if(ioff>=0) 
      for(i=0;i<(length/5);i++)
         set_stream(output,(ioff+i)*5,get_stream(output1,i*5,5),5);
   	
   for(i=0;(i-sh)<real_length;i++)
   {
      if((tab_simul[lad][i].hard_bypass&1)==0) 
      {
         sh++;
         continue;
      }
      set_stream(input,(i-sh)*5,tab_simul_ir[lad][i],5);
      tab_simul_ir[lad][i]=(unsigned short) get_stream(output,(i-sh)*5,5);
   }
   
}

/*************************************************************************

       Function: circulate_dr()

        Summary: Circulates a bit stream thru the TAP controller data path

          Usage: void circulate_dr(length, data)
                 unsigned short length;
                 unsigned short *data;

    Description: This routine circulates an arbitrary length bit stream thru
                 the target TAP controller data path.  Data doesnt actually
                 start circulating until the first 32 bits of data are sent
                 out.  The first 16 bits are sent to get valid data into the
                 RX-FIFO, and the second 16 bits force the first valid data
                 out of the scan path.  This function has two parameters.
                 The 'length' parameter specifies how many bits to 
                 circulate.  The 'data' parameter is an address to an array
                 of 16-bit values where the data scanned in is to be stored.

**************************************************************************/

void circulate_dr(unsigned short length,unsigned short *output)
{
   unsigned short numregs,i,ws,str,j,x,*latchupInfo;
   short str_f=-1;
   
    if((lad&1)==0)
    {  
      if(lad<20) latchupInfo=&latchupInfoSimN0;
      else latchupInfo=&latchupInfoSimN1;
    }
    else
    {
      if(lad<20) latchupInfo=&latchupInfoSimP0;
      else latchupInfo=&latchupInfoSimP1;
    }

   simCiDr+=length;
    /* determine number of registers to use in data transfer */
    numregs = length / 16;

    if((numregs * 16) != length)
        ++numregs;

   for(i=0,str=0;i<SIM_NB;i++)
   {
      if ((tab_simul[lad][i].hard_bypass&1)==0) continue;
      if ((tab_simul[lad][i].hard_bypass)==3) str_f=str;
      if (tab_simul[lad][i].type==COSTAR)
      {
          switch(tab_simul_ir[lad][i])
          {
	  case CO_CTRL_1 : 
	     set_stream(output,str,tab_simul[lad][i].typ.c.ctrl1,8);
	     str+=8;
	     break;

	  case CO_CTRL_2 :
	     set_stream(output,str,tab_simul[lad][i].typ.c.ctrl2,8);
	     str+=8;
	     break;
	  
	  case CO_DAC_0 :
	     set_stream(output,str,tab_simul[lad][i].typ.c.dac,32);
	     str+=32;
	     break;
	  
	  case  CO_DINOUT : 
	     set_stream(output,str,tab_simul[lad][i].typ.c.dinout,8);
	     str+=8;
	     break;
	  
	  case CO_BYPASS : 
	     set_stream(output,str,tab_simul[lad][i].typ.c.bypass,1);
	     str+=1;
	     break;
	  
	  case CO_ADC_0 :
	     set_stream(output,str,tab_simul[lad][i].typ.c.adc0,32);
	     str+=32;
	     break;
	  
	  case CO_ADC_1 :
	     set_stream(output,str,tab_simul[lad][i].typ.c.adc1,32);
	     str+=32;
	     break;
	  
	  case CO_IDENT :
	     set_stream(output,str,tab_simul[lad][i].typ.c.ident,8);
	     str+=8;
	     break;
	  
	  default : /*printf("ERROR : instruction unknown %d chip %d\n",tab_simul_ir[lad][i],i);*/
	  	    break;
	  }
       }
       else if (tab_simul[lad][i].type==ALICE128C) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case AL_BYPASS : 
	     set_stream(output,str,tab_simul[lad][i].typ.a.bypass,1);
	     str+=1;
	     break;
	     
           case AL_BIAS :
       	     for (j=0;j<7;j++)
	     {
	        set_stream(output,str,tab_simul[lad][i].typ.a.bias[j],8);
	        str+=8;
	     }
	     break;
       
           case AL_PULSE :
       	     set_stream(output,str,tab_simul[lad][i].typ.a.pulse,8);
	     str+=8;
             break;
	     
           case AL_ONOFF :
       	     set_stream(output,str,tab_simul[lad][i].typ.a.onoff,1);
	     str+=1;
             break;
	     
           case AL_TOKEN :
       	     set_stream(output,str,tab_simul[lad][i].typ.a.token,1);
	     str+=1;
             break;
          
	   case AL_TEST :
       	     for (j=0;j<8;j++)
	     {
       	        set_stream(output,str,tab_simul[lad][i].typ.a.test[j],8);
	        str+=8;
       	     }
	     break;
	   
	   case CO_IDENT :
	     set_stream(output,str,tab_simul[lad][i].typ.a.ident,8);
	     str+=8;
	     break;
	     
	   default : break;
	  
	  }	     

       }
       else if (tab_simul[lad][i].type==CONNEXION) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case CX_BYPASS : 
	     set_stream(output,str,tab_simul[lad][i].typ.x.bypass,CX_BYPASS_SZ);
	     str+=CX_BYPASS_SZ;
	     break;
	     
           case CX_LATCHUP :
	     set_stream(output,str,tab_simul[lad][i].typ.x.latchup,CX_LATCHUP_SZ);
	     str+=CX_LATCHUP_SZ;
	     break;
       
           case CX_STATUS :
	     set_stream(output,str,tab_simul[lad][i].typ.x.pow_status,CX_STATUS_SZ);
	     /*printf("pow_status : %d\n",tab_simul[lad][i].typ.x.pow_status);*/
	     str+=CX_STATUS_SZ;
	     if(*latchupInfo==1) *latchupInfo=0;
             break;
	     
           case CX_VREF : 
	     set_stream(output,str,tab_simul[lad][i].typ.x.vref,CX_VREF_SZ);
	     str+=CX_VREF_SZ;
             break;
	     
           case CX_POWER :
	     set_stream(output,str,tab_simul[lad][i].typ.x.pow_on,CX_POWER_SZ);
	     str+=CX_POWER_SZ;
             break;
          
	   case CX_HYB_BYP :
	     set_stream(output,str,tab_simul[lad][i].typ.x.hyb_bypass,CX_HYB_BYP_SZ);
	     str+=CX_HYB_BYP_SZ;
	     break;
	     
	   case CX_IDENT :
	     set_stream(output,str,tab_simul[lad][i].typ.x.ident,CX_IDENT_SZ);
	     str+=CX_IDENT_SZ;
	     break;
	  
	   default : 
	     set_stream(output,str,tab_simul[lad][i].typ.x.bypass,CX_BYPASS_SZ);
	     str+=CX_BYPASS_SZ;
	     break;
	  }	     

       }
       else if (tab_simul[lad][i].type==READOUT) 
       {
         switch(tab_simul_ir[lad][i])
         {
           case RD_BYPASS : 
	     set_stream(output,str,readout[lad/10].bypass,RD_BYPASS_SZ);
	     str+=RD_BYPASS_SZ;
	     break;
	     
           case RD_LATCHUP :
	     set_stream(output,str,readout[lad/10].latchup,RD_LATCHUP_SZ);
	     str+=RD_LATCHUP_SZ;
	     break;
       
           case RD_STATUS :
	     set_stream(output,str,readout[lad/10].status,RD_STATUS_SZ);
	     str+=RD_STATUS_SZ;
             break;
	     
           case RD_VERSION : 
	   for(j=0;j<3;j++)
	   {
	     set_stream(output,str,readout[lad/10].version[j],16);
           }
	     str+=RD_VERSION_SZ;
	     break;
	     
           case RD_CONFIG :
	     set_stream(output,str,readout[lad/10].config,RD_CONFIG_SZ);
	     str+=RD_CONFIG_SZ;
             break;
          
           case RD_CHAIN :
	     set_stream(output,str,readout[lad/10].config,RD_CHAIN_SZ);
	     str+=RD_CHAIN_SZ;
             break;
          
	   case RD_TEMP1 :
	     set_stream(output,str,readout[lad/10].temp1,RD_TEMP1_SZ);
	     str+=RD_TEMP1_SZ;
	     break;
	     
	   case RD_IDENT :
	     set_stream(output,str,readout[lad/10].ident,RD_IDENT_SZ);
	     str+=RD_IDENT_SZ;
	     break;
	  
	   default : 
	     set_stream(output,str,readout[lad/10].bypass,RD_BYPASS_SZ);
	     str+=RD_BYPASS_SZ;
	     break;
	  }	     

       }
    }

   if (str_f!=-1)
   {
	 ws=bit_to_word(str_f);
	 set_stream(output,str_f,0xffff,16-(str_f%16));
	 for(ws++;ws<numregs;ws++) output[ws]=0xffff;
    }

  /* printf("circulate dr :");

   for(x=0;x<numregs;x++)
    {
    printf("%4x ",output[x]);
    }
   printf("\nlength = %d\n",length);*/
    
}

short reset_jtag()
{
    short err;

    err=hard_reset(TBC_EXTERNAL,CLK_781KHZ,TBC_DIV_BY_4);
    write_tbc(CONTROL0,0);
    write_tbc(CONTROL1,0);
    write_tbc(CONTROL2,1);
    write_tbc(CONTROL3,0x8f);
    write_tbc(CONTROL5,0);
    write_tbc(CONTROL6,0);
    write_tbc(CONTROL7,0);
    write_tbc(CONTROL8,4);
    write_tbc(CONTROL9,0);
    return err;
    
}

short reset_jtag_var(unsigned short i,unsigned short j,unsigned short k)
{
    short err;

    err=hard_reset(i,j,k);
    write_tbc(CONTROL0,0);
    write_tbc(CONTROL1,0);
    write_tbc(CONTROL2,1);
    write_tbc(CONTROL3,0x8f);
    write_tbc(CONTROL5,0);
    write_tbc(CONTROL6,0);
    write_tbc(CONTROL7,0);
    write_tbc(CONTROL8,4);
    write_tbc(CONTROL9,0);
    return err;
    
}

void connect_tap(char tap)
{
   if(tap>(LADDER_NB-1)) return;
   lad=tap;
}


/***************************************************************************************


	bit_to_word (bit_nb)
	
	unsigned short	bit_nb
	
	return : unsigned short word_nb
	
	
	Description : 	this function determines how many words are needed to represent
			bit_nb bits


****************************************************************************************/
unsigned short bit_to_word(unsigned short bit_nb)
{
    unsigned short word_nb;
    
    word_nb=bit_nb/16;
    if ((word_nb*16) == bit_nb) word_nb--;
    
    return word_nb;
}

/***************************************************************************************


	map_nb(map)
	
	chip 		*map
	
	return : unsigned short chip_nb
	
	Description :  	this function determines how many chip are in the chain, 
			according to the "map".


****************************************************************************************/

unsigned short map_nb(chip *map)
{
    unsigned short i=0;
    
    for (;map[i]!=END;i++);
    
    return i;
}


/***************************************************************************************


	init_stream(stream,lenght)
	
	unsigned short 	*stream
	unsigned short 	length
	
	return :	nothing
	
	Description : this function sets the stream to be all ones
		     (all ones instruction = BYPASS instruction, according to IEEE 1149.1)	


******************************************************************************************/

void init_stream(unsigned short *stream,unsigned short length)
{
   
   int i,n;

   n=bit_to_word(length);      
   for (i=0;i<(n+1);i++)
       stream[i]=0xffff;
       
}

/***************************************************************************************


	set_stream(stream,loc_bit,data,data_sz)
	
	unsigned short 	*stream
	unsigned short 	loc_bit
	unsigned long 	data
	unsigned short 	data_sz
	
	return :	nothing
	
	
	Description : 	this function puts "data" in the stream at the location 
			corresponding to "loc_bit". "data_sz" is the size (in bits) of
			"data".	

******************************************************************************************/

void set_stream(unsigned short *stream,unsigned short loc_bit,unsigned long data,unsigned short data_sz)
{

unsigned short w1,w2,b1,b2,data_mask,i;
    
    w1 = loc_bit/16;		/* nb of word where data begins 			*/
    b1 = loc_bit - w1*16;		/* nb of bit where data begins in w1 			*/
    w2 = bit_to_word(loc_bit+data_sz);	/* nb of word where data ends 				*/			
    b2 = (loc_bit+data_sz) - w2*16;	/* nb of bit where data ends in w2			*/
    
   /* printf("set stream : str:%p loc:%d data:%x  siz:%d\n",stream,loc_bit,data,data_sz); */
    
    if (w1==w2)  /* if the data fits in a single word of the stream */
    {
       for (data_mask=0,i=0;i<16;i++)
	  if ((i<b1)||(i>=b2)) data_mask+=1<<i;
       stream[w1]=(stream[w1]&data_mask)+(data<<b1);	/* fill this single word */
       return;
    }
    else
    {
       for (data_mask=0,i=0;i<16;i++)
          if (i<b1) data_mask+=1<<i;
       stream[w1]=(stream[w1]&data_mask)+((data<<b1)&0xffff);	/* fill the first word */
       
       for (data_mask=0,i=0;i<16;i++)
          if (i>=b2) data_mask+=1<<i;
       stream[w2]=(stream[w2]&data_mask)+(data>>(data_sz-b2));	/* fill the last word */
       
       if ((w1+1)<w2) 	/* if the data fits in three words of the stream */
          stream[w1+1]=(data>>(16-b1))&0xffff;			/* fill the middle word */
    }
       
}  
   
/***************************************************************************************


	get_stream(stream,loc_bit,data_sz)
	
	unsigned short 	*stream
	unsigned short 	loc_bit
	unsigned short 	data_sz
	
	return :	unsigned long data
	
	
	Description : 	this function gets the data from the stream at the location 
			corresponding to "loc_bit" (data_sz : number of bits of the data)
				

******************************************************************************************/

unsigned long	get_stream(unsigned short *stream,unsigned short loc_bit,unsigned short data_sz)
{

unsigned short w1,w2,b1,b2,data_mask,i;
unsigned long data;


    /*printf("get stream : str:%p loc:%d siz:%d\n",stream,loc_bit,data_sz);*/

    w1 = loc_bit/16;			/* word where data begins 			*/
    b1 = loc_bit - w1*16;		/* nb of bit where data begins in w1 			*/
    w2 = bit_to_word(loc_bit+data_sz);	/* nb of word where data ends 				*/
    b2 = (loc_bit+data_sz) - w2*16;	/* nb of bit where data ends in w2			*/
    
   /* printf("get stream : w1:%d w2:%d b1:%d b2:%d\n",w1,w2,b1,b2); */
    
    if (w1==w2)  /* if the data fits in a single word of the stream */
    {
       for (data_mask=0,i=0;i<16;i++)
	  if ((i>=b1)&&(i<b2)) data_mask+=1<<i;
       data=(stream[w1]&data_mask)>>b1;		/* get the data */ 
       return data;
    }
    else
    {
       for (data_mask=0,i=0;i<16;i++)
          if (i>=b1) data_mask+=1<<i;
       data=(stream[w1]&data_mask)>>b1;	/* get the part of the data from the first word */
  
       /*printf("get stream : data1:%x",(stream[w1]&data_mask)>>b1);*/
           
       for (data_mask=0,i=0;i<16;i++)
          if (i<b2) data_mask+=1<<i;
       data+=((unsigned long)(stream[w2]&data_mask))<<((w2-w1)*16-b1);	/* get the part of the data from the last word */
       /*printf(" data2:%x",((unsigned long)(stream[w2]&data_mask))<<((w2-w1)*16-b1));*/
       
       if ((w1+1)<w2) 	/* if the data fits in three words of the stream */
          {data+=((unsigned long)stream[w1+1])<<(16-b1);			/* get the part of the data from the middle word */
       /*printf(" data3:%x\n",((unsigned long)stream[w1+1])<<(16-b1));*/
    	}
    }
    return data;
}

/***************************************************************************************

	bypass_all_chips(map)
	
	Description : 	this function send a bypass instruction to all the chips 
			(map descibes the chain).
				

******************************************************************************************/
void bypass_all_chips(chip *map)
{
    unsigned short stream_in[MAX_LENGTH_STREAM_IR],stream_out[MAX_LENGTH_STREAM_IR];
    unsigned short nb_chip;
    
    nb_chip = map_nb(map);
    
    init_stream(stream_out,IR_SZ*nb_chip);	/* fill the output stream with BYPASS instructions */
    
    scan_ir(stream_out,IR_SZ*nb_chip,stream_in);
    
}

/***************************************************************************************

	jtag_check()
	
	return : int
		
	Description : 	this function check the continuity of the jtag chain and return 1
			if the chain is not interrupted.
						

******************************************************************************************/ 
int jtag_check()
{
return 1;
}
   
   
   
/***************************************************************************************

	detect_length()
	
	return : unsigned short length
		
	Description : 	this function detect automatically the length of the JTAG chain and 
			return the number of chips in the chain.
			
	note : if this function is not called just after a reset, it should be called twice. 
			

******************************************************************************************/ 
unsigned short detect_length()
{
   int i,n=0;
   for(i=0;i<SIM_NB;i++) 
   {
      if ((tab_simul[lad][i].hard_bypass&1)==1)
      {
         n++;
         if ((tab_simul[lad][i].hard_bypass&2)==2) return 0;
      }
   }
   return n;       
}
   
   
/***************************************************************************************

	auto_init_map(map)
	
	chip *map
	
	return : nothing
	
	Description : 	this function initialize the chip map automatically by sending EXTEST
			commands to distinguish between COSTAR and ALICE128C chips.
			

******************************************************************************************/ 

int auto_init_map(chip *map)
{
   unsigned short nb_chip; 
   unsigned short stream_in[MAX_LENGTH_STREAM_IR],stream_out[MAX_LENGTH_STREAM_IR];
   int i;
      
   if ((nb_chip=detect_length())==0||!jtag_check()) {map[0]=END; return 0;}
   
   for (i=0;i<nb_chip;i++)
   {
       init_stream(stream_out,IR_SZ*nb_chip);
       
       set_stream(stream_out,i*IR_SZ,0x1b,IR_SZ);   
   
       scan_ir(stream_out,IR_SZ*nb_chip,stream_in); 	/* send ID command */
   
       circulate_dr(nb_chip*BYPASS_SZ+IDENT_SZ-1,stream_in);	/* read the ID register (result in stream_in) */			
      
     /* if(!i)
      {
   	for(x=0;x<bit_to_word(nb_chip*BYPASS_SZ+IDENT_SZ-1);x++)
   	 {
   	 printf("%4x ",stream_in[x]);
  	  }
  	 printf("\nlength = %d\n",nb_chip*BYPASS_SZ+IDENT_SZ-1);
      printf("ir:%d hb:%d id:%2x\n",tab_simul_ir[lad][i],tab_simul[lad][i].hard_bypass,tab_simul[lad][i].typ.c.ident);
      }
         
      printf("%2x",get_stream(stream_in,i*BYPASS_SZ,8)&0xff);*/
      if (get_stream(stream_in,i*BYPASS_SZ,8)==0xaf)	     /* if the ID register is the costar one */
      {
         map[i]=COSTAR;					/* it's a COSTAR */
      }
      else if ((get_stream(stream_in,i*BYPASS_SZ,8)&0xf0)==0x90)	     /* if the ID register is the FPGA connexion one */
      {
         map[i]=CONNEXION;					/* it's a FPGA connexion */
      }
      else if ((get_stream(stream_in,i*BYPASS_SZ,8)&0xf0)==0x60)	     /* if the ID register is the FPGA connexion one */
      {
         map[i]=READOUT;					/* it's a FPGA connexion */
      }
      else
      {
         map[i]=ALICE128C;				/* else it's a ALICE128C */
      }
   }
   /*printf(" nc:%d\n",nb_chip);*/
   map[nb_chip]=END;
   return nb_chip;
}
       

   
/***************************************************************************************

	init_map(map,name)
	
	chip *map
	char *name
	
	return : nothing
	
	Description : 	this function initialize the chip map by reading a ascii file which
			name is in the string 'name'. 
			
			
	note : 		in the ascii file :
			
			'a' or 'A' means Alice128C chip
			'c' or 'C' means Costar chip
			'.' means end of description.

******************************************************************************************/ 

void init_map(chip *map,char *name)
{
   FILE *map_file;
   int i,c,type;
   
   map[0]=COSTAR;
   type=0;
   for(i=0;name[i]!='\0';i++)
      if ((name[i]!='c')&&(name[i]!='a')&&(name[i]!='C')&&(name[i]!='A')&&(name[i]!='.')&&(name[i]!='x')&&(name[i]!='X')&&(name[i]!='r')&&(name[i]!='R'))
         type=1;
   if (type==1)
   {
      name=strcat(name,".map");
      map_file = fopen(name,"r");
      if (map_file==NULL)
        {
	   printf("\nERROR : can't open map file.");
	   map[0]=END;
	   return;
 	}
      fgets(name,128,map_file);
      fclose(map_file);
   }
   i=0;
   for(i=0;name[i]!='\0';i++)
   {
      switch(c=name[i])
      {
	 case '.' : map[i]=END;
	 	    break;
		    
	 case 'c' : map[i]=COSTAR;
	 	    break;
		    
	 case 'C' : map[i]=COSTAR;
	 	    break;
		    
	 case 'a' : map[i]=ALICE128C;
	 	    break;
		    
	 case 'A' : map[i]=ALICE128C;
	 	    break;
		    
	 case 'x' : map[i]=CONNEXION;
	 	    break;
		    
	 case 'X' : map[i]=CONNEXION;
	 	    break;
		    
	 case 'r' : map[i]=READOUT;
	 	    break;
		    
	 case 'R' : map[i]=READOUT;
	 	    break;
		    
	 default  : break;
       }
     
    }
    if((i>0)&&(map[i-1]!=END)) map[i]=END;
    if(!i) map[i]=END;
     
    
    return;
}

/***************************************************************************************

	jtag_init(map,chain)
	
	chip *map
	char *chain
	
	return : nothing
	
	Description : 	this function try to initialize the jtag map (automatically) and
			put it in "map". If it doesn't works, it uses the default map
			described by "chain". "chain" is the name of the ascii file 
			describing the map, or the map itself.
	
	ex :		jtag_init(map,"default")  use the file "default.map" as default map
			jtag_init(map,"aaaaaac.") the default map is 6 A128C and 1 COSTAR.  
			
			
	note : 		in the ascii file :
			
			'a' or 'A' means Alice128C chip
			'c' or 'C' means Costar chip
			'x' or 'X' means FPGA of connexion board
			'r' or 'R' means FPGA of readout board
			'.' means end of description.
			
			the name of this file must contain at least one character different
			from 'a' , 'c' , 'x' or 'r'

******************************************************************************************/ 
   
int jtag_init(chip *map,char *chain)
{
   
/*    if (!reset_jtag())
      printf("reset JTAG : OK\n");
   else 
   {
      printf(" ERROR : JTAG Board not present.\n");
      return 1;
   }
 */   
   if (!auto_init_map(map))
      {
      printf(" ERROR : JTAG chain not connected.\nUsing default map\n");
      init_map(map,chain);
      }
   if (map[0]==END) return 1;
   else return 0;
}   
    
void uncompress_map(chip *map,char *cmap)
{
   int i,j,fin=0;
   
   for (i=0;i<COMPRESSED_MAP_LENGTH;i++)
   {
      for (j=0;j<4;j++)
         if ((map[i*4+j]=(cmap[i]&(3<<(2*j)))>>(2*j))==END) 
	 {
	    fin=1;
	    break;
	 }
   if (fin) break;
   }
   
}

void compress_map(chip *map,char *cmap)
{
    int i,j,fin=0;
    chip c;
    
   for (i=0;i<COMPRESSED_MAP_LENGTH;i++)
   {
      for (j=0,cmap[i]=0;j<4;j++)
      {
         c=map[i*4+j];
	 if (c>END) c=ALICE128C;
	 cmap[i]|=(c&3)<<(2*j);
	 if (c==END)
	 {
	    fin=1;
	    break;
	 }
      }
   if(fin) break;
   }
}

void ascii_map(chip *map,char *amap)
{
    int i;
    
   for (i=0;(i<MAX_CHIPS)&&(map[i]!=END);i++)
   {
      switch(map[i])
      {
      case COSTAR    : amap[i]='C';
      		       break;
      case ALICE128C : amap[i]='A';
      		       break;
      case CONNEXION : amap[i]='X';
      		       break;
      case READOUT   : amap[i]='R';
      		       break;
      default        : break;
      }
   }	    
   amap[i++]='.';
   amap[i]=0;
}

void jtagSimulInit()
{
   int i,j,k,l;
   
   for (i=0;i<LADDER_NB;i++)
   {
      for (j=0;j<MODULE_NB;j++)
      {
	 tab_simul[i][j*7].type=COSTAR;
	 tab_simul[i][j*7].typ.c.ctrl1=0;
	 tab_simul[i][j*7].typ.c.ctrl2=0;
	 tab_simul[i][j*7].typ.c.dinout=0;
	 tab_simul[i][j*7].typ.c.dac=0;
	 tab_simul[i][j*7].typ.c.adc0=128+(128<<8)+(128<<16)+(128<<24);
	 tab_simul[i][j*7].typ.c.adc1=128+(26<<8)+(77<<16)+(104<<24);
	 tab_simul[i][j*7].typ.c.bypass=0;
	 tab_simul[i][j*7].typ.c.ident=0xaf;
	 tab_simul[i][j*7].hard_bypass=0;
	 cnx_module[i][j]=(j+1)*7;
	 for (k=1;k<=6;k++)
	 {
	    tab_simul[i][j*7+k].type=ALICE128C;
	    tab_simul[i][j*7+k].typ.a.pulse=0;
	    tab_simul[i][j*7+k].typ.a.onoff=0;
	    tab_simul[i][j*7+k].typ.a.token=0;
	    tab_simul[i][j*7+k].typ.a.bypass=0;
	    tab_simul[i][j*7+k].typ.a.ident=0x00;
	    for (l=0;l<7;l++)
	    {
	       tab_simul[i][j*7+k].typ.a.bias[l]=0;
	       tab_simul[i][j*7+k].typ.a.test[l]=0;
	    }
	    tab_simul[i][j*7+k].typ.a.test[7]=0;
	    tab_simul[i][j*7+k].hard_bypass=0;
	 }
	 tab_simul[i][MODULE_NB*7].type=CONNEXION;
	 tab_simul[i][MODULE_NB*7].typ.x.pow_status=0xffff;
	 tab_simul[i][MODULE_NB*7].typ.x.pow_on=0;
	 tab_simul[i][MODULE_NB*7].typ.x.latchup=0;
	 tab_simul[i][MODULE_NB*7].typ.x.vref=2;
	 tab_simul[i][MODULE_NB*7].typ.x.hyb_bypass=0x00;
	 tab_simul[i][MODULE_NB*7].typ.x.bypass=0;
	 tab_simul[i][MODULE_NB*7].typ.x.ident=0x97;
	 tab_simul[i][MODULE_NB*7+1].type=READOUT;	 
      }     
   }
   for (i=0;i<4;i++)
   {
      readout[i].version[0]=2001;
      readout[i].version[1]=0x0101;
      readout[i].version[2]=0x0210;
      readout[i].latchup=0;
      readout[i].status=i+128*(i&1)+256+512;
      readout[i].temp1=27+4096*27;
      readout[i].chain=0;	 
      readout[i].ident=0x60;	 
   }
   for (i=0;i<nbLadder;i++)
   {
      for (j=0;j<nbModule;j++)
      { 
         for (k=0;k<7;k++)
	    tab_simul[i][j*7+k].hard_bypass=1;
	 tab_simul[i][MODULE_NB*7].typ.x.hyb_bypass|=1<<j;
      }
      tab_simul[i][MODULE_NB*7].hard_bypass=1;
      tab_simul[i][MODULE_NB*7+1].hard_bypass=1;
   }
   
}	 
	 
void simTemp(unsigned char i,unsigned char temp)
{
   int j;
   float jf;
   unsigned long adc1;
   
      for (j=0;j<MODULE_NB;j++)
      { 
         jf=(float)j;
   	 adc1=tab_simul[i][j*7].typ.c.adc1;
	 tab_simul[i][j*7].typ.c.adc1=adc1&0x00ffffff;
	 tab_simul[i][j*7].typ.c.adc1|=(unsigned char)((104*(1-(jf/MODULE_NB))+(temp*(jf/MODULE_NB))))<<24;
      }
}
   
void simBias(unsigned char bias)
{
   int i,j;
   float jf;
   unsigned long adc0;
   
   for (i=0;i<LADDER_NB;i++)
   {
      for (j=0;j<MODULE_NB;j++)
      { 
         jf=(float)j;
   	 adc0=tab_simul[i][j*7].typ.c.adc0;
	 tab_simul[i][j*7].typ.c.adc0=adc0&0xff00ffff;
	 tab_simul[i][j*7].typ.c.adc0|=bias<<16;
      }
   }
}
   
int simChipFail(unsigned short lad, unsigned short nchip)
{
    tab_simul[lad][nchip].hard_bypass|=2;
    return tab_simul[lad][nchip].hard_bypass;
}

int simChipFix(unsigned short lad, unsigned short nchip)
{
    tab_simul[lad][nchip].hard_bypass=0;
    return tab_simul[lad][nchip].hard_bypass;
}

unsigned short simLatchup(unsigned short lad, unsigned short nchip)
{
    simChipFail(lad,nchip);
    tab_simul[lad][MODULE_NB*7].typ.x.pow_status&=~(1<<(char)(nchip/7));
    tab_simul[lad][MODULE_NB*7].typ.x.latchup=1;
    readout[lad/10].status|=1<<5;
    readout[lad/10].latchup+=1;
    /*if((lad&1)==0)
    {  
      if(lad<20) latchupInfoSimN0=1;
      else latchupInfoSimN1=1;
    }
    else
    {
      if(lad<20) latchupInfoSimP0=1;
      else latchupInfoSimP1=1;
    }*/
    return tab_simul[lad][MODULE_NB*7].typ.x.pow_status;
}    


void viewHb(unsigned short lad)
{
      int i;
      
      for(i=0;i<SIM_NB;i++)
         printf("%d",tab_simul[lad][i].hard_bypass);
      printf("\n");
}

void simCount(void)
{
	printf("\nInstruction Register : %d\n",simScIr);
	printf("Data Registers       : %d\n",simScDr+simCiDr);
	printf("Total                : %d\n",simScDr+simCiDr+simScIr);
}

void simCountR(void)
{
   simScIr=0;
   simScDr=0;
   simCiDr=0;
}
