/* jtagCostar.h		Jtag costar functions header 
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	David Bonnet
 *	Date :		22/03/00
 *
 */
 
/* COSTAR REGISTERS SIZE */

  #define CO_IR_SZ				5
  #define CO_BSR_SZ				20
  #define CO_BYPASS_SZ				1
  #define CO_CTRL_SZ				8
  #define CO_IDENT_SZ				8
  #define CO_ADC_SZ				32
  #define CO_ADC_TEST_SZ			4
  #define CO_DAC_SZ				32
  #define CO_DINOUT_SZ				8

/* COSTAR REGISTERS NUMBER */

  #define CO_NUMBER				4

  #define CO_IR_NB				1
  #define CO_BSR_NB				1
  #define CO_BYPASS_NB				1
  #define CO_CTRL_NB				1
  #define CO_IDENT_NB				1
  #define CO_ADC_NB				1
  #define CO_ADC_TEST_NB			1
  #define CO_DAC_NB				1
  #define CO_DINOUT_NB				1

/* COSTAR COMMANDS */

  #define CO_EXT_TEST				0x00
  #define CO_HIGHZ				0x01
  #define CO_BSR_SP				0x02
  #define CO_IN_TEST				0x03
  #define CO_CLAMP				0x04
  #define CO_ID					0x05   /* supp 10/02/99 cf doc */
  #define CO_CTRL_1				0x10
  #define CO_ADC_TEST_0				0x11
  #define CO_ADC_0				0x12
  #define CO_ADC_TEST_1				0x13
  #define CO_ADC_1				0x14
  #define CO_DINOUT				0x17
  #define CO_DAC_0				0x18
  #define CO_CTRL_2				0x1A
  #define CO_IDENT				0x1B
  #define CO_BYPASS				0x1F 
  
#define CONTROL_REG1 0x32 		/* Internal Clock divided by 8 */

#define ASYN_COSTAR_TASK_DELAY 120 	/* 2 seconds */
#define ASYN_COSTAR_TASK_PRIORITY 210
#define ASYN_COSTAR_TASK_STACK 10000

typedef struct
{
   unsigned char adc0[4];
   unsigned char adc1[4];
   unsigned char vref;
   unsigned char soft_bypass;
   unsigned char hard_bypass;
}costar ;


unsigned short  map_costar_nb(chip *);
void 		send_all_costar(unsigned long, unsigned short, unsigned short *, unsigned short *,chip *);
void 		send_some_costar(unsigned long, unsigned short, unsigned short *,unsigned short *,char *,chip *);
unsigned long 	send_one_costar(unsigned long, unsigned short, unsigned long, unsigned short, chip *);
void 		get_all_costar(unsigned long, unsigned short, unsigned short *, chip *);
unsigned long 	get_one_costar(unsigned long, unsigned short, unsigned short, chip *);
void  		set_ctrl1_all_costar(unsigned char, chip *);
void  		set_ctrl2_all_costar(unsigned char, chip *);
unsigned char 	set_ctrl1_one_costar(unsigned char, unsigned short, chip *);
unsigned char 	set_ctrl2_one_costar(unsigned char, unsigned short, chip *);
void  		set_dac_all_costar(unsigned char [][], unsigned char [][], chip *);
void  		set_dac_one_costar(unsigned char *, unsigned char *, unsigned short, chip *);
void 		get_adc0_all_costar(unsigned char [][], chip *);
void 		get_adc1_all_costar(unsigned char [][], chip *);
void 		get_adc0_one_costar(unsigned char *, unsigned short, chip *);
void 		get_adc1_one_costar(unsigned char *, unsigned short, chip *);
void 		get_adc0_one_costar_debug(unsigned char *, unsigned short, chip *);
void 		get_adc1_one_costar_debug(unsigned char *, unsigned short, chip *);
void 		start_conversion_all_costar(chip *);
void 		stop_conversion_all_costar(chip *);
void 		start_conversion_one_costar(unsigned short, chip *);
void 		stop_conversion_one_costar(unsigned short, chip *);

