program ssd_ladder

option +r; /* make code reentrant */

int ladder;
assign ladder to "ssd_lad{lad}_jtag";
monitor ladder;

int power;
assign power to "ssd_lad{lad}_power_status";
monitor power; 

float current;
assign current to "ssd_lad{lad}_lv_current";
monitor current;

float limit;
assign limit to "ssd_lad{lad}_lv_limit";

int controlMode;
assign controlMode to "ssd_control_mode";
monitor controlMode;

short jtag;
assign jtag to "ssd_main_jtag";
monitor jtag;

short refresh;
assign refresh to "ssd_main_costar";
monitor refresh;

short cancel;
assign cancel to "ssd_main_cancel";
monitor cancel;

int init_state;
assign init_state to "ssd_lad{lad}_state";

int readout_state;
assign readout_state to "ssd_q{sect}_state";
monitor readout_state;

ss ladder_control
{
   state init
   {
      when((readout_state==4)&&power&&(init_state==0)&&(ladder!=10)) /* when readout board is ready and the ladder power supply is on*/
      {
	 init_state=2; /* state : init */
	 pvPut(init_state);
	 ladder=1; /* init */
	 pvPut(ladder);
      } state test1

      when(jtag&&init_state)
      {
         init_state=0; /* not ready */
	 pvPut(init_state);
      } state init

      when(refresh&&init_state)
      {
         init_state=0; /* not ready */
	 pvPut(init_state);
      } state init

      when(((init_state==11)||(init_state==12))&&((readout_state==4)||(readout_state==8)))
      {
	readout_state=0;
	pvPut(readout_state);
      } state init

   }

   state test1
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 ladder=3; /* init */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
      } state init
      
      when(!ladder)
      {
      } state bias

      when(ladder==3) /* connection board not detected */
      {
         ladder=1; /* init */
	 pvPut(ladder);
      } state test2

      when(ladder==10) /* all the hybrids are bypassed */
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
      } state init 

   }

   state test2
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
      } state init
      
      when(!ladder)
      {
      } state bias

      when(ladder==3) /* connection board not detected */
      {
         ladder=1; /* init */
	 pvPut(ladder);
      } state test3

      when(ladder==10) /* all the hybrids are bypassed */
      {
      } state init

   }

   state test3
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
      } state init
      
      when(!ladder)
      {
      } state bias

      when(ladder==3) /* connection board not detected */
      {
         init_state=12; /* error */
	 pvPut(init_state);
      } state init

      when(ladder==10) /* all the hybrids are bypassed */
      {
      } state init

   }

   state bias
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
      } state init
      
      when(!ladder)
      {
         init_state=3; /* Bias */
	 pvPut(init_state);
         ladder=2; /* bias */
	 pvPut(ladder);
	 limit=1.5;
	 pvPut(limit);
      } state costar

      when(ladder==3)
      {
         init_state=12; /* error */
	 pvPut(init_state);
      } state init

      when(ladder==10) /* all the hybrids are bypassed */
      {
      } state init


   }
   state costar
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
      } state init
      
      when(!ladder)
      {
         init_state=4; /* Read costars */
	 pvPut(init_state);
         ladder=7; /* read costars */
	 pvPut(ladder);
      } state ready

      when(ladder==1) /* need to init */
      {
         init_state=2; /* Init */
	 pvPut(init_state);
         ladder=1; /* init */
	 pvPut(ladder);
      } state bias

      when(ladder==3)
      {
         init_state=12; /* error */
	 pvPut(init_state);
      } state init

      when(ladder==10) /* all the hybrids are bypassed */
      {
      } state init

   }

   state ready
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 ladder=3; /* reset */
	 pvPut(ladder);
	 limit=-0.1;
	 pvPut(limit);	
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
      } state init
      
      when((!ladder)&&(init_state!=5))
      {
         init_state=5; /* Ready */
	 pvPut(init_state);
      } state ready

      when(ladder==3)
      {
         init_state=12; /* error */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
      } state init

      when(jtag) 
      {
	 init_state=1; /* check */
	 pvPut(init_state);
	 ladder=0; /* reset */
	 pvPut(ladder);
	 limit=-0.1;
	 pvPut(limit);	
      } state init     

      when(refresh&&!pvSeverity(current)) 
      {
	 init_state=6; /* reload chips */
	 pvPut(init_state);
	 ladder=6; /* latchup */
	 pvPut(ladder);
      } state costar     

      when(refresh&&pvSeverity(current)) 
      {
	 init_state=6; /* reload chips */
	 pvPut(init_state);
	 ladder=6; /* latchup */
	 pvPut(ladder);
      } state bias     

      when(controlMode==1)
      {
	 init_state=8; /* state : daq */
	 pvPut(init_state);
      } state daq       

      when(controlMode==2)
      {
	 init_state=9; /* state : calibration */
	 pvPut(init_state);
	 ladder=4; /* test pulse */
	 pvPut(ladder);
      } state goto_calib       

      when(controlMode==3)
      {
         init_state=4; /* Read costars */
         pvPut(init_state);
         ladder=7; /* read costars */
         pvPut(ladder);
      } state monit

   }

   state daq
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
      } state init
      
      when(controlMode==0)
      {
	 init_state=7; /* state : full control */
	 pvPut(init_state);
      } state ready       

   }

   state monit
   {
      when(!power)
      {
         init_state=11; /* power is off */
         pvPut(init_state);
         limit=-0.1;
         pvPut(limit);
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init

      when(cancel)
      {
         init_state=10; /* cancelled */
         pvPut(init_state);
         limit=-0.1;
         pvPut(limit);
      } state init

      when(controlMode==0)
      {
         init_state=7; /* state : full control */
         pvPut(init_state);
      } state ready

      when(delay(30))
      {
         ladder=7; /* read costars */
         pvPut(ladder);
      } state monit

   }

   state goto_calib
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
      } state init

      when(!ladder)
      {
	 init_state=5; /* state : ready */
	 pvPut(init_state);
      } state calib      
   }

   state calib
   {
      when(!power)
      {
         init_state=11; /* power is off */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
	 ladder=3; /* reset */
	 pvPut(ladder);
      } state init
      
      when(cancel)
      {
         init_state=10; /* cancelled */
	 pvPut(init_state);
	 limit=-0.1;
	 pvPut(limit);	
      } state init

      when(controlMode==0)
      {
	 init_state=7; /* state : full control */
	 pvPut(init_state);
      } state ready       
     
      when(controlMode==1)
      {
	 init_state=8; /* state : daq */
	 pvPut(init_state);
      } state daq       
   }


}
