program ssd_readout

option +r; /* make code reentrant */

int readout;
assign readout to "ssd_q{sect}_readout";
monitor readout;

int main_readout;
assign main_readout to "ssd_q1P_readout";
monitor main_readout;

int controlMode;
assign controlMode to "ssd_control_mode";
monitor controlMode;

short jtag;
assign jtag to "ssd_main_jtag";
monitor jtag;

short refresh;
assign refresh to "ssd_main_costar";
monitor refresh;

short cancel;
assign cancel to "ssd_main_cancel";
monitor cancel;

int init_state;
assign init_state to "ssd_q{sect}_state";
monitor init_state;

int lv_status;
assign lv_status to "ssd_lad1N_lv_status";
monitor lv_status;

ss readout_board
{
   state init
   {
      when(jtag&&(controlMode==0)&&(init_state!=11)&&(init_state!=5))
      {
	 init_state=1; /* state : check board */
	 pvPut(init_state);
	 readout=4; /* full control */
	 pvPut(readout);
      } state test_fpga

      when(jtag&&(controlMode==0)&&((init_state==11)||(init_state==5)))
      {
	 init_state=2; /* state : load FPGA */
	 pvPut(init_state);
	 readout=6; /* Program FPGA*/
	 pvPut(readout);
      } state test2_fpga
      
      when(jtag&&(controlMode!=0))
      {
	 init_state=9; /* NO WAY ! */
	 pvPut(init_state);
      } state warning_init

      when((lv_status&64)&&(init_state!=5)) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when((!(lv_status&64))&&(init_state==5)) /* end of interlock */
      {
	 init_state=0; /* state : not ready */
	 pvPut(init_state);
      } state init
   }

   state test_fpga
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(readout==5) /* if readout board not responding */
      {
	 init_state=2; /* state : load FPGA */
	 pvPut(init_state);
	 readout=6; /* Program FPGA*/
	 pvPut(readout);
      } state test2_fpga
      
      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init
      
      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when(readout==0) /* if readout board is ok */
      {
	 init_state=3; /* state : init */
	 pvPut(init_state);
      } state reset
   }
         
   state test2_fpga
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init
      
      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when(readout==1) /* if readout board not responding */
      {
	 init_state=12; /* state : cannot program */
	 pvPut(init_state);
      } state init
      
      when(readout==0) /* if readout board is ok */
      {
	 init_state=3; /* state : init */
	 pvPut(init_state);
	 /*readout=7;*/ /* reset */
	 readout=4; /* get full control */
	 pvPut(readout);
      } state get_version
   }

   state reset
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init
      
      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when(readout==5) /* if readout board not responding */
      {
	 init_state=12; /* state : cannot program */
	 pvPut(init_state);
      } state init
      
      when(readout==0) /* if readout board is ok */
      {
	 readout=4; /* get full control */
	 pvPut(readout);

      } state get_version
   }



   state get_version
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init
      
      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when(readout==5) /* if readout board not responding */
      {
	 init_state=12; /* state : cannot program */
	 pvPut(init_state);
      } state init
      
      when(readout==0) /* if readout board is ok */
      {
	 readout=3; /* get version */
	 pvPut(readout);

      } state set_config
   }

   state set_config
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init
      
      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when(readout==0) /* if readout board is ok */
      {
	 readout=1; /* set config */
	 pvPut(readout);

      } state get_status
   }

   state get_status
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init
      
      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when(readout==0) /* if readout board is ok */
      {
	 init_state=4; /* Ready */
	 pvPut(init_state);
	 readout=2; /* get status */
	 pvPut(readout);

      } state ready
   }

   state ready
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(main_readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
	 readout=7; /* Power OFF */
	 pvPut(readout);
      } state init

      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init

      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when((jtag|refresh)&&!init_state) 
      {
	 init_state=2; /* state : load FPGA */
	 pvPut(init_state);
	 readout=6; /* Program FPGA*/
	 pvPut(readout);
      } state test2_fpga
      
      when((jtag|refresh)&&init_state) 
      {
	 init_state=3; /* state : init */
	 pvPut(init_state);
	 readout=4; /* go to full control mode */
	 pvPut(readout);
      } state test_fpga
      

      when(controlMode==1)
      {
	 init_state=8; /* state : daq */
	 pvPut(init_state);
	 readout=5; /* go to daq mode */
	 pvPut(readout);
      } state daq  
      
/*      when(delay(1))
      {
      readout=4;
      pvPut(readout);
      } state ready  */
   }      

   state daq
   {
      when(cancel) 
      {
	 init_state=10; /* state : cancelled */
	 pvPut(init_state);
      } state init
      
      when(main_readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
	 readout=7; /* Power OFF */
	 pvPut(readout);
      } state init

      when(readout==9) /* power is off */
      {
	 init_state=11; /* state : Power OFF */
	 pvPut(init_state);
      } state init

      when(lv_status&64) /* interlock */
      {
	 init_state=5; /* state : Interlock */
	 pvPut(init_state);
	 readout=9; /* power OFF */
	 pvPut(readout);
      } state init
      
      when(refresh||jtag) 
      {
	 init_state=9; /* NO WAY ! */
	 pvPut(init_state);
	 readout=0; /* do nothing */
	 pvPut(readout);
      } state warning_daq
      
      when(controlMode==0)
      {
	 if(init_state==8){
	    init_state=7; /* state : full control */
	    pvPut(init_state);
         }
	 readout=4; /* go to full control */
	 pvPut(readout);
      } state ready      

      when(delay(5))
      {
	 readout=2; /* get status */
	 pvPut(readout);
      } state daq
      
   }


   state warning_init
   {
      when(delay(1))
      {
	 init_state=6;
	 pvPut(init_state);
      } state init      
   }

   state warning_daq
   {
      when(delay(1))
      {
	 init_state=8;
	 pvPut(init_state);
      } state daq      
   }

}
