#ifndef INCcaenH
#define INCcaenH
#include "epicsTypes.h"
#include "link.h"
#include "epicsMutex.h"
#include "ellLib.h"
#include "epicsTime.h"
typedef struct caenRecord {
	char		name[61];	/* Record Name */
	char		desc[41];	/* Descriptor */
	char		asg[29];	/* Access Security Group */
	epicsEnum16	scan;	/* Scan Mechanism */
	epicsEnum16	pini;	/* Process at iocInit */
	epicsInt16	phas;	/* Scan Phase */
	epicsInt16	evnt;	/* Event Number */
	epicsInt16	tse;	/* Time Stamp Event */
	DBLINK		tsel;	/* Time Stamp Link */
	epicsEnum16	dtyp;	/* Device Type */
	epicsInt16	disv;	/* Disable Value */
	epicsInt16	disa;	/* Disable */
	DBLINK		sdis;	/* Scanning Disable */
	epicsMutexId	mlok;	/* Monitor lock */
	ELLLIST		mlis;	/* Monitor List */
	epicsUInt8	disp;	/* Disable putField */
	epicsUInt8	proc;	/* Force Processing */
	epicsEnum16	stat;	/* Alarm Status */
	epicsEnum16	sevr;	/* Alarm Severity */
	epicsEnum16	nsta;	/* New Alarm Status */
	epicsEnum16	nsev;	/* New Alarm Severity */
	epicsEnum16	acks;	/* Alarm Ack Severity */
	epicsEnum16	ackt;	/* Alarm Ack Transient */
	epicsEnum16	diss;	/* Disable Alarm Sevrty */
	epicsUInt8	lcnt;	/* Lock Count */
	epicsUInt8	pact;	/* Record active */
	epicsUInt8	putf;	/* dbPutField process */
	epicsUInt8	rpro;	/* Reprocess  */
	struct asgMember *asp;	/* Access Security Pvt */
	struct putNotify *ppn;	/* addr of PUTNOTIFY */
	struct putNotifyRecord *ppnr;	/* pputNotifyRecord */
	struct scan_element *spvt;	/* Scan Private */
	struct rset	*rset;	/* Address of RSET */
	struct dset	*dset;	/* DSET address */
	void		*dpvt;	/* Device Private */
	struct dbRecordType *rdes;	/* Address of dbRecordType */
	struct lockRecord *lset;	/* Lock Set */
	epicsEnum16	prio;	/* Scheduling Priority */
	epicsUInt8	tpro;	/* Trace Processing */
	char bkpt;	/* Break Point */
	epicsUInt8	udf;	/* Undefined */
	epicsTimeStamp	time;	/* Time */
	DBLINK		flnk;	/* Forward Process Link */
	epicsEnum16	val;	/* Power */
	epicsInt8	dcha;	/* Disable Channel */
	epicsInt16	crat;	/* Crate Number */
	epicsInt16	brd;	/* Slot Number */
	epicsInt16	chan;	/* Channel Number */
	char		btyp[6];	/* Board Type */
	epicsUInt16	chst;	/* Channel status */
	epicsEnum16	pwst;	/* Power status */
	epicsFloat32	vmon;	/* Monitored Voltage */
	epicsFloat32	imon;	/* Monitored Current */
	epicsFloat32	vset;	/* Set Voltage */
	epicsFloat32	iset;	/* Set Current */
	epicsInt16	rup;	/* Ramp Up */
	epicsInt16	rdwn;	/* Ramp Down */
	epicsFloat32	lvm;	/* Last Monit. Volt. */
	epicsFloat32	lim;	/* Last Monit. Curr. */
	epicsInt16	lst;	/* Last Channel status */
	epicsFloat32	vade;	/* Volt. Arch. Deadband */
	epicsFloat32	vmde;	/* Volt. monit. Deadband */
	epicsFloat32	vlar;	/* Volt. Last Val. Arch. */
	epicsFloat32	iade;	/* Curr. Arch. Deadband */
	epicsFloat32	imde;	/* Curr. monit. Deadband */
	epicsFloat32	ilar;	/* Curr. Last Val. Arch. */
	epicsEnum16	stsv;	/* Status Trip Severity */
	epicsEnum16	socs;	/* Stat. Over Curr. Sev. */
	epicsEnum16	sovs;	/* Stat. Over Volt. Sev. */
	epicsEnum16	suvs;	/* Stat. Und. Volt. Sev. */
	epicsEnum16	sosv;	/* Stat. other Sev. */
	epicsEnum16	cfsv;	/* Comm. Failure Sev. */
	epicsInt16	nbcf;	/* Nb comm. fail. alarm */
} caenRecord;
#define caenRecordNAME	0
#define caenRecordDESC	1
#define caenRecordASG	2
#define caenRecordSCAN	3
#define caenRecordPINI	4
#define caenRecordPHAS	5
#define caenRecordEVNT	6
#define caenRecordTSE	7
#define caenRecordTSEL	8
#define caenRecordDTYP	9
#define caenRecordDISV	10
#define caenRecordDISA	11
#define caenRecordSDIS	12
#define caenRecordMLOK	13
#define caenRecordMLIS	14
#define caenRecordDISP	15
#define caenRecordPROC	16
#define caenRecordSTAT	17
#define caenRecordSEVR	18
#define caenRecordNSTA	19
#define caenRecordNSEV	20
#define caenRecordACKS	21
#define caenRecordACKT	22
#define caenRecordDISS	23
#define caenRecordLCNT	24
#define caenRecordPACT	25
#define caenRecordPUTF	26
#define caenRecordRPRO	27
#define caenRecordASP	28
#define caenRecordPPN	29
#define caenRecordPPNR	30
#define caenRecordSPVT	31
#define caenRecordRSET	32
#define caenRecordDSET	33
#define caenRecordDPVT	34
#define caenRecordRDES	35
#define caenRecordLSET	36
#define caenRecordPRIO	37
#define caenRecordTPRO	38
#define caenRecordBKPT	39
#define caenRecordUDF	40
#define caenRecordTIME	41
#define caenRecordFLNK	42
#define caenRecordVAL	43
#define caenRecordDCHA	44
#define caenRecordCRAT	45
#define caenRecordBRD	46
#define caenRecordCHAN	47
#define caenRecordBTYP	48
#define caenRecordCHST	49
#define caenRecordPWST	50
#define caenRecordVMON	51
#define caenRecordIMON	52
#define caenRecordVSET	53
#define caenRecordISET	54
#define caenRecordRUP	55
#define caenRecordRDWN	56
#define caenRecordLVM	57
#define caenRecordLIM	58
#define caenRecordLST	59
#define caenRecordVADE	60
#define caenRecordVMDE	61
#define caenRecordVLAR	62
#define caenRecordIADE	63
#define caenRecordIMDE	64
#define caenRecordILAR	65
#define caenRecordSTSV	66
#define caenRecordSOCS	67
#define caenRecordSOVS	68
#define caenRecordSUVS	69
#define caenRecordSOSV	70
#define caenRecordCFSV	71
#define caenRecordNBCF	72
#endif /*INCcaenH*/
#ifdef GEN_SIZE_OFFSET
#ifdef __cplusplus
extern "C" {
#endif
#include <epicsExport.h>
static int caenRecordSizeOffset(dbRecordType *pdbRecordType)
{
    caenRecord *prec = 0;
  pdbRecordType->papFldDes[0]->size=sizeof(prec->name);
  pdbRecordType->papFldDes[0]->offset=(short)((char *)&prec->name - (char *)prec);
  pdbRecordType->papFldDes[1]->size=sizeof(prec->desc);
  pdbRecordType->papFldDes[1]->offset=(short)((char *)&prec->desc - (char *)prec);
  pdbRecordType->papFldDes[2]->size=sizeof(prec->asg);
  pdbRecordType->papFldDes[2]->offset=(short)((char *)&prec->asg - (char *)prec);
  pdbRecordType->papFldDes[3]->size=sizeof(prec->scan);
  pdbRecordType->papFldDes[3]->offset=(short)((char *)&prec->scan - (char *)prec);
  pdbRecordType->papFldDes[4]->size=sizeof(prec->pini);
  pdbRecordType->papFldDes[4]->offset=(short)((char *)&prec->pini - (char *)prec);
  pdbRecordType->papFldDes[5]->size=sizeof(prec->phas);
  pdbRecordType->papFldDes[5]->offset=(short)((char *)&prec->phas - (char *)prec);
  pdbRecordType->papFldDes[6]->size=sizeof(prec->evnt);
  pdbRecordType->papFldDes[6]->offset=(short)((char *)&prec->evnt - (char *)prec);
  pdbRecordType->papFldDes[7]->size=sizeof(prec->tse);
  pdbRecordType->papFldDes[7]->offset=(short)((char *)&prec->tse - (char *)prec);
  pdbRecordType->papFldDes[8]->size=sizeof(prec->tsel);
  pdbRecordType->papFldDes[8]->offset=(short)((char *)&prec->tsel - (char *)prec);
  pdbRecordType->papFldDes[9]->size=sizeof(prec->dtyp);
  pdbRecordType->papFldDes[9]->offset=(short)((char *)&prec->dtyp - (char *)prec);
  pdbRecordType->papFldDes[10]->size=sizeof(prec->disv);
  pdbRecordType->papFldDes[10]->offset=(short)((char *)&prec->disv - (char *)prec);
  pdbRecordType->papFldDes[11]->size=sizeof(prec->disa);
  pdbRecordType->papFldDes[11]->offset=(short)((char *)&prec->disa - (char *)prec);
  pdbRecordType->papFldDes[12]->size=sizeof(prec->sdis);
  pdbRecordType->papFldDes[12]->offset=(short)((char *)&prec->sdis - (char *)prec);
  pdbRecordType->papFldDes[13]->size=sizeof(prec->mlok);
  pdbRecordType->papFldDes[13]->offset=(short)((char *)&prec->mlok - (char *)prec);
  pdbRecordType->papFldDes[14]->size=sizeof(prec->mlis);
  pdbRecordType->papFldDes[14]->offset=(short)((char *)&prec->mlis - (char *)prec);
  pdbRecordType->papFldDes[15]->size=sizeof(prec->disp);
  pdbRecordType->papFldDes[15]->offset=(short)((char *)&prec->disp - (char *)prec);
  pdbRecordType->papFldDes[16]->size=sizeof(prec->proc);
  pdbRecordType->papFldDes[16]->offset=(short)((char *)&prec->proc - (char *)prec);
  pdbRecordType->papFldDes[17]->size=sizeof(prec->stat);
  pdbRecordType->papFldDes[17]->offset=(short)((char *)&prec->stat - (char *)prec);
  pdbRecordType->papFldDes[18]->size=sizeof(prec->sevr);
  pdbRecordType->papFldDes[18]->offset=(short)((char *)&prec->sevr - (char *)prec);
  pdbRecordType->papFldDes[19]->size=sizeof(prec->nsta);
  pdbRecordType->papFldDes[19]->offset=(short)((char *)&prec->nsta - (char *)prec);
  pdbRecordType->papFldDes[20]->size=sizeof(prec->nsev);
  pdbRecordType->papFldDes[20]->offset=(short)((char *)&prec->nsev - (char *)prec);
  pdbRecordType->papFldDes[21]->size=sizeof(prec->acks);
  pdbRecordType->papFldDes[21]->offset=(short)((char *)&prec->acks - (char *)prec);
  pdbRecordType->papFldDes[22]->size=sizeof(prec->ackt);
  pdbRecordType->papFldDes[22]->offset=(short)((char *)&prec->ackt - (char *)prec);
  pdbRecordType->papFldDes[23]->size=sizeof(prec->diss);
  pdbRecordType->papFldDes[23]->offset=(short)((char *)&prec->diss - (char *)prec);
  pdbRecordType->papFldDes[24]->size=sizeof(prec->lcnt);
  pdbRecordType->papFldDes[24]->offset=(short)((char *)&prec->lcnt - (char *)prec);
  pdbRecordType->papFldDes[25]->size=sizeof(prec->pact);
  pdbRecordType->papFldDes[25]->offset=(short)((char *)&prec->pact - (char *)prec);
  pdbRecordType->papFldDes[26]->size=sizeof(prec->putf);
  pdbRecordType->papFldDes[26]->offset=(short)((char *)&prec->putf - (char *)prec);
  pdbRecordType->papFldDes[27]->size=sizeof(prec->rpro);
  pdbRecordType->papFldDes[27]->offset=(short)((char *)&prec->rpro - (char *)prec);
  pdbRecordType->papFldDes[28]->size=sizeof(prec->asp);
  pdbRecordType->papFldDes[28]->offset=(short)((char *)&prec->asp - (char *)prec);
  pdbRecordType->papFldDes[29]->size=sizeof(prec->ppn);
  pdbRecordType->papFldDes[29]->offset=(short)((char *)&prec->ppn - (char *)prec);
  pdbRecordType->papFldDes[30]->size=sizeof(prec->ppnr);
  pdbRecordType->papFldDes[30]->offset=(short)((char *)&prec->ppnr - (char *)prec);
  pdbRecordType->papFldDes[31]->size=sizeof(prec->spvt);
  pdbRecordType->papFldDes[31]->offset=(short)((char *)&prec->spvt - (char *)prec);
  pdbRecordType->papFldDes[32]->size=sizeof(prec->rset);
  pdbRecordType->papFldDes[32]->offset=(short)((char *)&prec->rset - (char *)prec);
  pdbRecordType->papFldDes[33]->size=sizeof(prec->dset);
  pdbRecordType->papFldDes[33]->offset=(short)((char *)&prec->dset - (char *)prec);
  pdbRecordType->papFldDes[34]->size=sizeof(prec->dpvt);
  pdbRecordType->papFldDes[34]->offset=(short)((char *)&prec->dpvt - (char *)prec);
  pdbRecordType->papFldDes[35]->size=sizeof(prec->rdes);
  pdbRecordType->papFldDes[35]->offset=(short)((char *)&prec->rdes - (char *)prec);
  pdbRecordType->papFldDes[36]->size=sizeof(prec->lset);
  pdbRecordType->papFldDes[36]->offset=(short)((char *)&prec->lset - (char *)prec);
  pdbRecordType->papFldDes[37]->size=sizeof(prec->prio);
  pdbRecordType->papFldDes[37]->offset=(short)((char *)&prec->prio - (char *)prec);
  pdbRecordType->papFldDes[38]->size=sizeof(prec->tpro);
  pdbRecordType->papFldDes[38]->offset=(short)((char *)&prec->tpro - (char *)prec);
  pdbRecordType->papFldDes[39]->size=sizeof(prec->bkpt);
  pdbRecordType->papFldDes[39]->offset=(short)((char *)&prec->bkpt - (char *)prec);
  pdbRecordType->papFldDes[40]->size=sizeof(prec->udf);
  pdbRecordType->papFldDes[40]->offset=(short)((char *)&prec->udf - (char *)prec);
  pdbRecordType->papFldDes[41]->size=sizeof(prec->time);
  pdbRecordType->papFldDes[41]->offset=(short)((char *)&prec->time - (char *)prec);
  pdbRecordType->papFldDes[42]->size=sizeof(prec->flnk);
  pdbRecordType->papFldDes[42]->offset=(short)((char *)&prec->flnk - (char *)prec);
  pdbRecordType->papFldDes[43]->size=sizeof(prec->val);
  pdbRecordType->papFldDes[43]->offset=(short)((char *)&prec->val - (char *)prec);
  pdbRecordType->papFldDes[44]->size=sizeof(prec->dcha);
  pdbRecordType->papFldDes[44]->offset=(short)((char *)&prec->dcha - (char *)prec);
  pdbRecordType->papFldDes[45]->size=sizeof(prec->crat);
  pdbRecordType->papFldDes[45]->offset=(short)((char *)&prec->crat - (char *)prec);
  pdbRecordType->papFldDes[46]->size=sizeof(prec->brd);
  pdbRecordType->papFldDes[46]->offset=(short)((char *)&prec->brd - (char *)prec);
  pdbRecordType->papFldDes[47]->size=sizeof(prec->chan);
  pdbRecordType->papFldDes[47]->offset=(short)((char *)&prec->chan - (char *)prec);
  pdbRecordType->papFldDes[48]->size=sizeof(prec->btyp);
  pdbRecordType->papFldDes[48]->offset=(short)((char *)&prec->btyp - (char *)prec);
  pdbRecordType->papFldDes[49]->size=sizeof(prec->chst);
  pdbRecordType->papFldDes[49]->offset=(short)((char *)&prec->chst - (char *)prec);
  pdbRecordType->papFldDes[50]->size=sizeof(prec->pwst);
  pdbRecordType->papFldDes[50]->offset=(short)((char *)&prec->pwst - (char *)prec);
  pdbRecordType->papFldDes[51]->size=sizeof(prec->vmon);
  pdbRecordType->papFldDes[51]->offset=(short)((char *)&prec->vmon - (char *)prec);
  pdbRecordType->papFldDes[52]->size=sizeof(prec->imon);
  pdbRecordType->papFldDes[52]->offset=(short)((char *)&prec->imon - (char *)prec);
  pdbRecordType->papFldDes[53]->size=sizeof(prec->vset);
  pdbRecordType->papFldDes[53]->offset=(short)((char *)&prec->vset - (char *)prec);
  pdbRecordType->papFldDes[54]->size=sizeof(prec->iset);
  pdbRecordType->papFldDes[54]->offset=(short)((char *)&prec->iset - (char *)prec);
  pdbRecordType->papFldDes[55]->size=sizeof(prec->rup);
  pdbRecordType->papFldDes[55]->offset=(short)((char *)&prec->rup - (char *)prec);
  pdbRecordType->papFldDes[56]->size=sizeof(prec->rdwn);
  pdbRecordType->papFldDes[56]->offset=(short)((char *)&prec->rdwn - (char *)prec);
  pdbRecordType->papFldDes[57]->size=sizeof(prec->lvm);
  pdbRecordType->papFldDes[57]->offset=(short)((char *)&prec->lvm - (char *)prec);
  pdbRecordType->papFldDes[58]->size=sizeof(prec->lim);
  pdbRecordType->papFldDes[58]->offset=(short)((char *)&prec->lim - (char *)prec);
  pdbRecordType->papFldDes[59]->size=sizeof(prec->lst);
  pdbRecordType->papFldDes[59]->offset=(short)((char *)&prec->lst - (char *)prec);
  pdbRecordType->papFldDes[60]->size=sizeof(prec->vade);
  pdbRecordType->papFldDes[60]->offset=(short)((char *)&prec->vade - (char *)prec);
  pdbRecordType->papFldDes[61]->size=sizeof(prec->vmde);
  pdbRecordType->papFldDes[61]->offset=(short)((char *)&prec->vmde - (char *)prec);
  pdbRecordType->papFldDes[62]->size=sizeof(prec->vlar);
  pdbRecordType->papFldDes[62]->offset=(short)((char *)&prec->vlar - (char *)prec);
  pdbRecordType->papFldDes[63]->size=sizeof(prec->iade);
  pdbRecordType->papFldDes[63]->offset=(short)((char *)&prec->iade - (char *)prec);
  pdbRecordType->papFldDes[64]->size=sizeof(prec->imde);
  pdbRecordType->papFldDes[64]->offset=(short)((char *)&prec->imde - (char *)prec);
  pdbRecordType->papFldDes[65]->size=sizeof(prec->ilar);
  pdbRecordType->papFldDes[65]->offset=(short)((char *)&prec->ilar - (char *)prec);
  pdbRecordType->papFldDes[66]->size=sizeof(prec->stsv);
  pdbRecordType->papFldDes[66]->offset=(short)((char *)&prec->stsv - (char *)prec);
  pdbRecordType->papFldDes[67]->size=sizeof(prec->socs);
  pdbRecordType->papFldDes[67]->offset=(short)((char *)&prec->socs - (char *)prec);
  pdbRecordType->papFldDes[68]->size=sizeof(prec->sovs);
  pdbRecordType->papFldDes[68]->offset=(short)((char *)&prec->sovs - (char *)prec);
  pdbRecordType->papFldDes[69]->size=sizeof(prec->suvs);
  pdbRecordType->papFldDes[69]->offset=(short)((char *)&prec->suvs - (char *)prec);
  pdbRecordType->papFldDes[70]->size=sizeof(prec->sosv);
  pdbRecordType->papFldDes[70]->offset=(short)((char *)&prec->sosv - (char *)prec);
  pdbRecordType->papFldDes[71]->size=sizeof(prec->cfsv);
  pdbRecordType->papFldDes[71]->offset=(short)((char *)&prec->cfsv - (char *)prec);
  pdbRecordType->papFldDes[72]->size=sizeof(prec->nbcf);
  pdbRecordType->papFldDes[72]->offset=(short)((char *)&prec->nbcf - (char *)prec);
    pdbRecordType->rec_size = sizeof(*prec);
    return(0);
}
epicsExportRegistrar(caenRecordSizeOffset);
#ifdef __cplusplus
}
#endif
#endif /*GEN_SIZE_OFFSET*/
