#ifndef INCcostarH
#define INCcostarH
#include "epicsTypes.h"
#include "link.h"
#include "epicsMutex.h"
#include "ellLib.h"
#include "epicsTime.h"
typedef struct costarRecord {
	char		name[61];	/* Record Name */
	char		desc[41];	/* Descriptor */
	char		asg[29];	/* Access Security Group */
	epicsEnum16	scan;	/* Scan Mechanism */
	epicsEnum16	pini;	/* Process at iocInit */
	epicsInt16	phas;	/* Scan Phase */
	epicsInt16	evnt;	/* Event Number */
	epicsInt16	tse;	/* Time Stamp Event */
	DBLINK		tsel;	/* Time Stamp Link */
	epicsEnum16	dtyp;	/* Device Type */
	epicsInt16	disv;	/* Disable Value */
	epicsInt16	disa;	/* Disable */
	DBLINK		sdis;	/* Scanning Disable */
	epicsMutexId	mlok;	/* Monitor lock */
	ELLLIST		mlis;	/* Monitor List */
	epicsUInt8	disp;	/* Disable putField */
	epicsUInt8	proc;	/* Force Processing */
	epicsEnum16	stat;	/* Alarm Status */
	epicsEnum16	sevr;	/* Alarm Severity */
	epicsEnum16	nsta;	/* New Alarm Status */
	epicsEnum16	nsev;	/* New Alarm Severity */
	epicsEnum16	acks;	/* Alarm Ack Severity */
	epicsEnum16	ackt;	/* Alarm Ack Transient */
	epicsEnum16	diss;	/* Disable Alarm Sevrty */
	epicsUInt8	lcnt;	/* Lock Count */
	epicsUInt8	pact;	/* Record active */
	epicsUInt8	putf;	/* dbPutField process */
	epicsUInt8	rpro;	/* Reprocess  */
	struct asgMember *asp;	/* Access Security Pvt */
	struct putNotify *ppn;	/* addr of PUTNOTIFY */
	struct putNotifyRecord *ppnr;	/* pputNotifyRecord */
	struct scan_element *spvt;	/* Scan Private */
	struct rset	*rset;	/* Address of RSET */
	struct dset	*dset;	/* DSET address */
	void		*dpvt;	/* Device Private */
	struct dbRecordType *rdes;	/* Address of dbRecordType */
	struct lockRecord *lset;	/* Lock Set */
	epicsEnum16	prio;	/* Scheduling Priority */
	epicsUInt8	tpro;	/* Trace Processing */
	char bkpt;	/* Break Point */
	epicsUInt8	udf;	/* Undefined */
	epicsTimeStamp	time;	/* Time */
	DBLINK		flnk;	/* Forward Process Link */
	epicsFloat32	val;	/* Temperature */
	epicsInt16	ladr;	/* Ladder Number */
	epicsInt16	modu;	/* Module Number */
	epicsEnum16	byps;	/* Chip Soft Bypass */
	epicsEnum16	hbyp;	/* Chip Hard Bypass */
	epicsInt16	jini;	/* Jtag Init */
	epicsFloat32	avdd;	/* Positive Low Voltage */
	epicsFloat32	avss;	/* Negative Low Voltage */
	epicsFloat32	bias;	/* Bias Current */
	epicsFloat32	guar;	/* Guard Current */
	epicsFloat32	leak;	/* Bias+Guard Current */
	epicsFloat32	refv;	/* An. Out Ref. Voltage */
	epicsFloat32	vrp;	/* Positive Ref. Voltage */
	epicsFloat32	vrn;	/* Negative Ref. Voltage */
	epicsFloat32	cfa;	/* Temp. Probe Slope */
	epicsFloat32	cfb;	/* Temp. Probe Offset */
	epicsFloat32	ires;	/* Resistor value */
	epicsFloat32	ltp;	/* Last Temperature */
	epicsFloat32	lvdd;	/* Last Posit. L. Volt. */
	epicsFloat32	lvss;	/* Last Negat. L. Volt. */
	epicsFloat32	lbia;	/* Last Bias Current */
	epicsFloat32	lgua;	/* Last Guard Current */
	epicsFloat32	thh;	/* T. HIHI Alarm */
	epicsFloat32	thi;	/* T. HIGH Alarm */
	epicsEnum16	thhs;	/* T. HIHI Severity */
	epicsEnum16	ths;	/* T. HIGH Severity */
	epicsFloat32	thys;	/* T. Alarm Deadband */
	epicsFloat32	tade;	/* T. Arch. Deadband */
	epicsFloat32	tmde;	/* T. monit. Deadband */
	epicsFloat32	tlal;	/* T. Last Val. Alrm. */
	epicsFloat32	tlar;	/* T. Last Val. Arch. */
	epicsFloat32	bhh;	/* B. HIHI Alarm */
	epicsFloat32	bhi;	/* B. HIGH Alarm */
	epicsEnum16	bhhs;	/* B. HIHI Severity */
	epicsEnum16	bhs;	/* B. HIGH Severity */
	epicsFloat32	bll;	/* B. LOLO Alarm */
	epicsFloat32	blo;	/* B. LOW Alarm */
	epicsEnum16	blls;	/* B. LOLO Severity */
	epicsEnum16	bls;	/* B. LOW Severity */
	epicsFloat32	bhys;	/* B. Alarm Deadband */
	epicsFloat32	bade;	/* B. Arch. Deadband */
	epicsFloat32	bmde;	/* B. monit. Deadband */
	epicsFloat32	blal;	/* B. Last Val. Alrm. */
	epicsFloat32	blar;	/* B. Last Val. Arch. */
	epicsFloat32	ghh;	/* G. HIHI Alarm */
	epicsFloat32	ghi;	/* G. HIGH Alarm */
	epicsEnum16	ghhs;	/* G. HIHI Severity */
	epicsEnum16	ghs;	/* G. HIGH Severity */
	epicsFloat32	gll;	/* G. LOLO Alarm */
	epicsFloat32	glo;	/* G. LOW Alarm */
	epicsEnum16	glls;	/* G. LOLO Severity */
	epicsEnum16	gls;	/* G. LOW Severity */
	epicsFloat32	ghys;	/* G. Alarm Deadband */
	epicsFloat32	gade;	/* G. Arch. Deadband */
	epicsFloat32	gmde;	/* G. monit. Deadband */
	epicsFloat32	glal;	/* G. Last Val. Alrm. */
	epicsFloat32	glar;	/* G. Last Val. Arch. */
	epicsFloat32	vhh;	/* V. HIHI Alarm */
	epicsFloat32	vhi;	/* V. HIGH Alarm */
	epicsEnum16	vhhs;	/* V. HIHI Severity */
	epicsEnum16	vhs;	/* V. HIGH Severity */
	epicsFloat32	vll;	/* V. LOLO Alarm */
	epicsFloat32	vlo;	/* V. LOW Alarm */
	epicsEnum16	vlls;	/* V. LOLO Severity */
	epicsEnum16	vls;	/* V. LOW Severity */
	epicsFloat32	vhys;	/* V. Alarm Deadband */
	epicsFloat32	vade;	/* V. Arch. Deadband */
	epicsFloat32	vmde;	/* V. monit. Deadband */
	epicsFloat32	vlal;	/* V. Last Val. Alrm. */
	epicsFloat32	dlar;	/* Last VDD Arch. */
	epicsFloat32	slar;	/* Last VSS Arch. */
	epicsUInt8	alst;	/* Alarm Status */
	epicsUInt8	lals;	/* Last Alarm Status */
	epicsUInt8	muxm;	/* Mux choice input -  */
	epicsUInt8	muxp;	/* Mux choice input +  */
	epicsUInt8	muxa;	/* Mux A address  */
	epicsUInt8	muxb;	/* Mux B address  */
} costarRecord;
#define costarRecordNAME	0
#define costarRecordDESC	1
#define costarRecordASG	2
#define costarRecordSCAN	3
#define costarRecordPINI	4
#define costarRecordPHAS	5
#define costarRecordEVNT	6
#define costarRecordTSE	7
#define costarRecordTSEL	8
#define costarRecordDTYP	9
#define costarRecordDISV	10
#define costarRecordDISA	11
#define costarRecordSDIS	12
#define costarRecordMLOK	13
#define costarRecordMLIS	14
#define costarRecordDISP	15
#define costarRecordPROC	16
#define costarRecordSTAT	17
#define costarRecordSEVR	18
#define costarRecordNSTA	19
#define costarRecordNSEV	20
#define costarRecordACKS	21
#define costarRecordACKT	22
#define costarRecordDISS	23
#define costarRecordLCNT	24
#define costarRecordPACT	25
#define costarRecordPUTF	26
#define costarRecordRPRO	27
#define costarRecordASP	28
#define costarRecordPPN	29
#define costarRecordPPNR	30
#define costarRecordSPVT	31
#define costarRecordRSET	32
#define costarRecordDSET	33
#define costarRecordDPVT	34
#define costarRecordRDES	35
#define costarRecordLSET	36
#define costarRecordPRIO	37
#define costarRecordTPRO	38
#define costarRecordBKPT	39
#define costarRecordUDF	40
#define costarRecordTIME	41
#define costarRecordFLNK	42
#define costarRecordVAL	43
#define costarRecordLADR	44
#define costarRecordMODU	45
#define costarRecordBYPS	46
#define costarRecordHBYP	47
#define costarRecordJINI	48
#define costarRecordAVDD	49
#define costarRecordAVSS	50
#define costarRecordBIAS	51
#define costarRecordGUAR	52
#define costarRecordLEAK	53
#define costarRecordREFV	54
#define costarRecordVRP	55
#define costarRecordVRN	56
#define costarRecordCFA	57
#define costarRecordCFB	58
#define costarRecordIRES	59
#define costarRecordLTP	60
#define costarRecordLVDD	61
#define costarRecordLVSS	62
#define costarRecordLBIA	63
#define costarRecordLGUA	64
#define costarRecordTHH	65
#define costarRecordTHI	66
#define costarRecordTHHS	67
#define costarRecordTHS	68
#define costarRecordTHYS	69
#define costarRecordTADE	70
#define costarRecordTMDE	71
#define costarRecordTLAL	72
#define costarRecordTLAR	73
#define costarRecordBHH	74
#define costarRecordBHI	75
#define costarRecordBHHS	76
#define costarRecordBHS	77
#define costarRecordBLL	78
#define costarRecordBLO	79
#define costarRecordBLLS	80
#define costarRecordBLS	81
#define costarRecordBHYS	82
#define costarRecordBADE	83
#define costarRecordBMDE	84
#define costarRecordBLAL	85
#define costarRecordBLAR	86
#define costarRecordGHH	87
#define costarRecordGHI	88
#define costarRecordGHHS	89
#define costarRecordGHS	90
#define costarRecordGLL	91
#define costarRecordGLO	92
#define costarRecordGLLS	93
#define costarRecordGLS	94
#define costarRecordGHYS	95
#define costarRecordGADE	96
#define costarRecordGMDE	97
#define costarRecordGLAL	98
#define costarRecordGLAR	99
#define costarRecordVHH	100
#define costarRecordVHI	101
#define costarRecordVHHS	102
#define costarRecordVHS	103
#define costarRecordVLL	104
#define costarRecordVLO	105
#define costarRecordVLLS	106
#define costarRecordVLS	107
#define costarRecordVHYS	108
#define costarRecordVADE	109
#define costarRecordVMDE	110
#define costarRecordVLAL	111
#define costarRecordDLAR	112
#define costarRecordSLAR	113
#define costarRecordALST	114
#define costarRecordLALS	115
#define costarRecordMUXM	116
#define costarRecordMUXP	117
#define costarRecordMUXA	118
#define costarRecordMUXB	119
#endif /*INCcostarH*/
#ifdef GEN_SIZE_OFFSET
#ifdef __cplusplus
extern "C" {
#endif
#include <epicsExport.h>
static int costarRecordSizeOffset(dbRecordType *pdbRecordType)
{
    costarRecord *prec = 0;
  pdbRecordType->papFldDes[0]->size=sizeof(prec->name);
  pdbRecordType->papFldDes[0]->offset=(short)((char *)&prec->name - (char *)prec);
  pdbRecordType->papFldDes[1]->size=sizeof(prec->desc);
  pdbRecordType->papFldDes[1]->offset=(short)((char *)&prec->desc - (char *)prec);
  pdbRecordType->papFldDes[2]->size=sizeof(prec->asg);
  pdbRecordType->papFldDes[2]->offset=(short)((char *)&prec->asg - (char *)prec);
  pdbRecordType->papFldDes[3]->size=sizeof(prec->scan);
  pdbRecordType->papFldDes[3]->offset=(short)((char *)&prec->scan - (char *)prec);
  pdbRecordType->papFldDes[4]->size=sizeof(prec->pini);
  pdbRecordType->papFldDes[4]->offset=(short)((char *)&prec->pini - (char *)prec);
  pdbRecordType->papFldDes[5]->size=sizeof(prec->phas);
  pdbRecordType->papFldDes[5]->offset=(short)((char *)&prec->phas - (char *)prec);
  pdbRecordType->papFldDes[6]->size=sizeof(prec->evnt);
  pdbRecordType->papFldDes[6]->offset=(short)((char *)&prec->evnt - (char *)prec);
  pdbRecordType->papFldDes[7]->size=sizeof(prec->tse);
  pdbRecordType->papFldDes[7]->offset=(short)((char *)&prec->tse - (char *)prec);
  pdbRecordType->papFldDes[8]->size=sizeof(prec->tsel);
  pdbRecordType->papFldDes[8]->offset=(short)((char *)&prec->tsel - (char *)prec);
  pdbRecordType->papFldDes[9]->size=sizeof(prec->dtyp);
  pdbRecordType->papFldDes[9]->offset=(short)((char *)&prec->dtyp - (char *)prec);
  pdbRecordType->papFldDes[10]->size=sizeof(prec->disv);
  pdbRecordType->papFldDes[10]->offset=(short)((char *)&prec->disv - (char *)prec);
  pdbRecordType->papFldDes[11]->size=sizeof(prec->disa);
  pdbRecordType->papFldDes[11]->offset=(short)((char *)&prec->disa - (char *)prec);
  pdbRecordType->papFldDes[12]->size=sizeof(prec->sdis);
  pdbRecordType->papFldDes[12]->offset=(short)((char *)&prec->sdis - (char *)prec);
  pdbRecordType->papFldDes[13]->size=sizeof(prec->mlok);
  pdbRecordType->papFldDes[13]->offset=(short)((char *)&prec->mlok - (char *)prec);
  pdbRecordType->papFldDes[14]->size=sizeof(prec->mlis);
  pdbRecordType->papFldDes[14]->offset=(short)((char *)&prec->mlis - (char *)prec);
  pdbRecordType->papFldDes[15]->size=sizeof(prec->disp);
  pdbRecordType->papFldDes[15]->offset=(short)((char *)&prec->disp - (char *)prec);
  pdbRecordType->papFldDes[16]->size=sizeof(prec->proc);
  pdbRecordType->papFldDes[16]->offset=(short)((char *)&prec->proc - (char *)prec);
  pdbRecordType->papFldDes[17]->size=sizeof(prec->stat);
  pdbRecordType->papFldDes[17]->offset=(short)((char *)&prec->stat - (char *)prec);
  pdbRecordType->papFldDes[18]->size=sizeof(prec->sevr);
  pdbRecordType->papFldDes[18]->offset=(short)((char *)&prec->sevr - (char *)prec);
  pdbRecordType->papFldDes[19]->size=sizeof(prec->nsta);
  pdbRecordType->papFldDes[19]->offset=(short)((char *)&prec->nsta - (char *)prec);
  pdbRecordType->papFldDes[20]->size=sizeof(prec->nsev);
  pdbRecordType->papFldDes[20]->offset=(short)((char *)&prec->nsev - (char *)prec);
  pdbRecordType->papFldDes[21]->size=sizeof(prec->acks);
  pdbRecordType->papFldDes[21]->offset=(short)((char *)&prec->acks - (char *)prec);
  pdbRecordType->papFldDes[22]->size=sizeof(prec->ackt);
  pdbRecordType->papFldDes[22]->offset=(short)((char *)&prec->ackt - (char *)prec);
  pdbRecordType->papFldDes[23]->size=sizeof(prec->diss);
  pdbRecordType->papFldDes[23]->offset=(short)((char *)&prec->diss - (char *)prec);
  pdbRecordType->papFldDes[24]->size=sizeof(prec->lcnt);
  pdbRecordType->papFldDes[24]->offset=(short)((char *)&prec->lcnt - (char *)prec);
  pdbRecordType->papFldDes[25]->size=sizeof(prec->pact);
  pdbRecordType->papFldDes[25]->offset=(short)((char *)&prec->pact - (char *)prec);
  pdbRecordType->papFldDes[26]->size=sizeof(prec->putf);
  pdbRecordType->papFldDes[26]->offset=(short)((char *)&prec->putf - (char *)prec);
  pdbRecordType->papFldDes[27]->size=sizeof(prec->rpro);
  pdbRecordType->papFldDes[27]->offset=(short)((char *)&prec->rpro - (char *)prec);
  pdbRecordType->papFldDes[28]->size=sizeof(prec->asp);
  pdbRecordType->papFldDes[28]->offset=(short)((char *)&prec->asp - (char *)prec);
  pdbRecordType->papFldDes[29]->size=sizeof(prec->ppn);
  pdbRecordType->papFldDes[29]->offset=(short)((char *)&prec->ppn - (char *)prec);
  pdbRecordType->papFldDes[30]->size=sizeof(prec->ppnr);
  pdbRecordType->papFldDes[30]->offset=(short)((char *)&prec->ppnr - (char *)prec);
  pdbRecordType->papFldDes[31]->size=sizeof(prec->spvt);
  pdbRecordType->papFldDes[31]->offset=(short)((char *)&prec->spvt - (char *)prec);
  pdbRecordType->papFldDes[32]->size=sizeof(prec->rset);
  pdbRecordType->papFldDes[32]->offset=(short)((char *)&prec->rset - (char *)prec);
  pdbRecordType->papFldDes[33]->size=sizeof(prec->dset);
  pdbRecordType->papFldDes[33]->offset=(short)((char *)&prec->dset - (char *)prec);
  pdbRecordType->papFldDes[34]->size=sizeof(prec->dpvt);
  pdbRecordType->papFldDes[34]->offset=(short)((char *)&prec->dpvt - (char *)prec);
  pdbRecordType->papFldDes[35]->size=sizeof(prec->rdes);
  pdbRecordType->papFldDes[35]->offset=(short)((char *)&prec->rdes - (char *)prec);
  pdbRecordType->papFldDes[36]->size=sizeof(prec->lset);
  pdbRecordType->papFldDes[36]->offset=(short)((char *)&prec->lset - (char *)prec);
  pdbRecordType->papFldDes[37]->size=sizeof(prec->prio);
  pdbRecordType->papFldDes[37]->offset=(short)((char *)&prec->prio - (char *)prec);
  pdbRecordType->papFldDes[38]->size=sizeof(prec->tpro);
  pdbRecordType->papFldDes[38]->offset=(short)((char *)&prec->tpro - (char *)prec);
  pdbRecordType->papFldDes[39]->size=sizeof(prec->bkpt);
  pdbRecordType->papFldDes[39]->offset=(short)((char *)&prec->bkpt - (char *)prec);
  pdbRecordType->papFldDes[40]->size=sizeof(prec->udf);
  pdbRecordType->papFldDes[40]->offset=(short)((char *)&prec->udf - (char *)prec);
  pdbRecordType->papFldDes[41]->size=sizeof(prec->time);
  pdbRecordType->papFldDes[41]->offset=(short)((char *)&prec->time - (char *)prec);
  pdbRecordType->papFldDes[42]->size=sizeof(prec->flnk);
  pdbRecordType->papFldDes[42]->offset=(short)((char *)&prec->flnk - (char *)prec);
  pdbRecordType->papFldDes[43]->size=sizeof(prec->val);
  pdbRecordType->papFldDes[43]->offset=(short)((char *)&prec->val - (char *)prec);
  pdbRecordType->papFldDes[44]->size=sizeof(prec->ladr);
  pdbRecordType->papFldDes[44]->offset=(short)((char *)&prec->ladr - (char *)prec);
  pdbRecordType->papFldDes[45]->size=sizeof(prec->modu);
  pdbRecordType->papFldDes[45]->offset=(short)((char *)&prec->modu - (char *)prec);
  pdbRecordType->papFldDes[46]->size=sizeof(prec->byps);
  pdbRecordType->papFldDes[46]->offset=(short)((char *)&prec->byps - (char *)prec);
  pdbRecordType->papFldDes[47]->size=sizeof(prec->hbyp);
  pdbRecordType->papFldDes[47]->offset=(short)((char *)&prec->hbyp - (char *)prec);
  pdbRecordType->papFldDes[48]->size=sizeof(prec->jini);
  pdbRecordType->papFldDes[48]->offset=(short)((char *)&prec->jini - (char *)prec);
  pdbRecordType->papFldDes[49]->size=sizeof(prec->avdd);
  pdbRecordType->papFldDes[49]->offset=(short)((char *)&prec->avdd - (char *)prec);
  pdbRecordType->papFldDes[50]->size=sizeof(prec->avss);
  pdbRecordType->papFldDes[50]->offset=(short)((char *)&prec->avss - (char *)prec);
  pdbRecordType->papFldDes[51]->size=sizeof(prec->bias);
  pdbRecordType->papFldDes[51]->offset=(short)((char *)&prec->bias - (char *)prec);
  pdbRecordType->papFldDes[52]->size=sizeof(prec->guar);
  pdbRecordType->papFldDes[52]->offset=(short)((char *)&prec->guar - (char *)prec);
  pdbRecordType->papFldDes[53]->size=sizeof(prec->leak);
  pdbRecordType->papFldDes[53]->offset=(short)((char *)&prec->leak - (char *)prec);
  pdbRecordType->papFldDes[54]->size=sizeof(prec->refv);
  pdbRecordType->papFldDes[54]->offset=(short)((char *)&prec->refv - (char *)prec);
  pdbRecordType->papFldDes[55]->size=sizeof(prec->vrp);
  pdbRecordType->papFldDes[55]->offset=(short)((char *)&prec->vrp - (char *)prec);
  pdbRecordType->papFldDes[56]->size=sizeof(prec->vrn);
  pdbRecordType->papFldDes[56]->offset=(short)((char *)&prec->vrn - (char *)prec);
  pdbRecordType->papFldDes[57]->size=sizeof(prec->cfa);
  pdbRecordType->papFldDes[57]->offset=(short)((char *)&prec->cfa - (char *)prec);
  pdbRecordType->papFldDes[58]->size=sizeof(prec->cfb);
  pdbRecordType->papFldDes[58]->offset=(short)((char *)&prec->cfb - (char *)prec);
  pdbRecordType->papFldDes[59]->size=sizeof(prec->ires);
  pdbRecordType->papFldDes[59]->offset=(short)((char *)&prec->ires - (char *)prec);
  pdbRecordType->papFldDes[60]->size=sizeof(prec->ltp);
  pdbRecordType->papFldDes[60]->offset=(short)((char *)&prec->ltp - (char *)prec);
  pdbRecordType->papFldDes[61]->size=sizeof(prec->lvdd);
  pdbRecordType->papFldDes[61]->offset=(short)((char *)&prec->lvdd - (char *)prec);
  pdbRecordType->papFldDes[62]->size=sizeof(prec->lvss);
  pdbRecordType->papFldDes[62]->offset=(short)((char *)&prec->lvss - (char *)prec);
  pdbRecordType->papFldDes[63]->size=sizeof(prec->lbia);
  pdbRecordType->papFldDes[63]->offset=(short)((char *)&prec->lbia - (char *)prec);
  pdbRecordType->papFldDes[64]->size=sizeof(prec->lgua);
  pdbRecordType->papFldDes[64]->offset=(short)((char *)&prec->lgua - (char *)prec);
  pdbRecordType->papFldDes[65]->size=sizeof(prec->thh);
  pdbRecordType->papFldDes[65]->offset=(short)((char *)&prec->thh - (char *)prec);
  pdbRecordType->papFldDes[66]->size=sizeof(prec->thi);
  pdbRecordType->papFldDes[66]->offset=(short)((char *)&prec->thi - (char *)prec);
  pdbRecordType->papFldDes[67]->size=sizeof(prec->thhs);
  pdbRecordType->papFldDes[67]->offset=(short)((char *)&prec->thhs - (char *)prec);
  pdbRecordType->papFldDes[68]->size=sizeof(prec->ths);
  pdbRecordType->papFldDes[68]->offset=(short)((char *)&prec->ths - (char *)prec);
  pdbRecordType->papFldDes[69]->size=sizeof(prec->thys);
  pdbRecordType->papFldDes[69]->offset=(short)((char *)&prec->thys - (char *)prec);
  pdbRecordType->papFldDes[70]->size=sizeof(prec->tade);
  pdbRecordType->papFldDes[70]->offset=(short)((char *)&prec->tade - (char *)prec);
  pdbRecordType->papFldDes[71]->size=sizeof(prec->tmde);
  pdbRecordType->papFldDes[71]->offset=(short)((char *)&prec->tmde - (char *)prec);
  pdbRecordType->papFldDes[72]->size=sizeof(prec->tlal);
  pdbRecordType->papFldDes[72]->offset=(short)((char *)&prec->tlal - (char *)prec);
  pdbRecordType->papFldDes[73]->size=sizeof(prec->tlar);
  pdbRecordType->papFldDes[73]->offset=(short)((char *)&prec->tlar - (char *)prec);
  pdbRecordType->papFldDes[74]->size=sizeof(prec->bhh);
  pdbRecordType->papFldDes[74]->offset=(short)((char *)&prec->bhh - (char *)prec);
  pdbRecordType->papFldDes[75]->size=sizeof(prec->bhi);
  pdbRecordType->papFldDes[75]->offset=(short)((char *)&prec->bhi - (char *)prec);
  pdbRecordType->papFldDes[76]->size=sizeof(prec->bhhs);
  pdbRecordType->papFldDes[76]->offset=(short)((char *)&prec->bhhs - (char *)prec);
  pdbRecordType->papFldDes[77]->size=sizeof(prec->bhs);
  pdbRecordType->papFldDes[77]->offset=(short)((char *)&prec->bhs - (char *)prec);
  pdbRecordType->papFldDes[78]->size=sizeof(prec->bll);
  pdbRecordType->papFldDes[78]->offset=(short)((char *)&prec->bll - (char *)prec);
  pdbRecordType->papFldDes[79]->size=sizeof(prec->blo);
  pdbRecordType->papFldDes[79]->offset=(short)((char *)&prec->blo - (char *)prec);
  pdbRecordType->papFldDes[80]->size=sizeof(prec->blls);
  pdbRecordType->papFldDes[80]->offset=(short)((char *)&prec->blls - (char *)prec);
  pdbRecordType->papFldDes[81]->size=sizeof(prec->bls);
  pdbRecordType->papFldDes[81]->offset=(short)((char *)&prec->bls - (char *)prec);
  pdbRecordType->papFldDes[82]->size=sizeof(prec->bhys);
  pdbRecordType->papFldDes[82]->offset=(short)((char *)&prec->bhys - (char *)prec);
  pdbRecordType->papFldDes[83]->size=sizeof(prec->bade);
  pdbRecordType->papFldDes[83]->offset=(short)((char *)&prec->bade - (char *)prec);
  pdbRecordType->papFldDes[84]->size=sizeof(prec->bmde);
  pdbRecordType->papFldDes[84]->offset=(short)((char *)&prec->bmde - (char *)prec);
  pdbRecordType->papFldDes[85]->size=sizeof(prec->blal);
  pdbRecordType->papFldDes[85]->offset=(short)((char *)&prec->blal - (char *)prec);
  pdbRecordType->papFldDes[86]->size=sizeof(prec->blar);
  pdbRecordType->papFldDes[86]->offset=(short)((char *)&prec->blar - (char *)prec);
  pdbRecordType->papFldDes[87]->size=sizeof(prec->ghh);
  pdbRecordType->papFldDes[87]->offset=(short)((char *)&prec->ghh - (char *)prec);
  pdbRecordType->papFldDes[88]->size=sizeof(prec->ghi);
  pdbRecordType->papFldDes[88]->offset=(short)((char *)&prec->ghi - (char *)prec);
  pdbRecordType->papFldDes[89]->size=sizeof(prec->ghhs);
  pdbRecordType->papFldDes[89]->offset=(short)((char *)&prec->ghhs - (char *)prec);
  pdbRecordType->papFldDes[90]->size=sizeof(prec->ghs);
  pdbRecordType->papFldDes[90]->offset=(short)((char *)&prec->ghs - (char *)prec);
  pdbRecordType->papFldDes[91]->size=sizeof(prec->gll);
  pdbRecordType->papFldDes[91]->offset=(short)((char *)&prec->gll - (char *)prec);
  pdbRecordType->papFldDes[92]->size=sizeof(prec->glo);
  pdbRecordType->papFldDes[92]->offset=(short)((char *)&prec->glo - (char *)prec);
  pdbRecordType->papFldDes[93]->size=sizeof(prec->glls);
  pdbRecordType->papFldDes[93]->offset=(short)((char *)&prec->glls - (char *)prec);
  pdbRecordType->papFldDes[94]->size=sizeof(prec->gls);
  pdbRecordType->papFldDes[94]->offset=(short)((char *)&prec->gls - (char *)prec);
  pdbRecordType->papFldDes[95]->size=sizeof(prec->ghys);
  pdbRecordType->papFldDes[95]->offset=(short)((char *)&prec->ghys - (char *)prec);
  pdbRecordType->papFldDes[96]->size=sizeof(prec->gade);
  pdbRecordType->papFldDes[96]->offset=(short)((char *)&prec->gade - (char *)prec);
  pdbRecordType->papFldDes[97]->size=sizeof(prec->gmde);
  pdbRecordType->papFldDes[97]->offset=(short)((char *)&prec->gmde - (char *)prec);
  pdbRecordType->papFldDes[98]->size=sizeof(prec->glal);
  pdbRecordType->papFldDes[98]->offset=(short)((char *)&prec->glal - (char *)prec);
  pdbRecordType->papFldDes[99]->size=sizeof(prec->glar);
  pdbRecordType->papFldDes[99]->offset=(short)((char *)&prec->glar - (char *)prec);
  pdbRecordType->papFldDes[100]->size=sizeof(prec->vhh);
  pdbRecordType->papFldDes[100]->offset=(short)((char *)&prec->vhh - (char *)prec);
  pdbRecordType->papFldDes[101]->size=sizeof(prec->vhi);
  pdbRecordType->papFldDes[101]->offset=(short)((char *)&prec->vhi - (char *)prec);
  pdbRecordType->papFldDes[102]->size=sizeof(prec->vhhs);
  pdbRecordType->papFldDes[102]->offset=(short)((char *)&prec->vhhs - (char *)prec);
  pdbRecordType->papFldDes[103]->size=sizeof(prec->vhs);
  pdbRecordType->papFldDes[103]->offset=(short)((char *)&prec->vhs - (char *)prec);
  pdbRecordType->papFldDes[104]->size=sizeof(prec->vll);
  pdbRecordType->papFldDes[104]->offset=(short)((char *)&prec->vll - (char *)prec);
  pdbRecordType->papFldDes[105]->size=sizeof(prec->vlo);
  pdbRecordType->papFldDes[105]->offset=(short)((char *)&prec->vlo - (char *)prec);
  pdbRecordType->papFldDes[106]->size=sizeof(prec->vlls);
  pdbRecordType->papFldDes[106]->offset=(short)((char *)&prec->vlls - (char *)prec);
  pdbRecordType->papFldDes[107]->size=sizeof(prec->vls);
  pdbRecordType->papFldDes[107]->offset=(short)((char *)&prec->vls - (char *)prec);
  pdbRecordType->papFldDes[108]->size=sizeof(prec->vhys);
  pdbRecordType->papFldDes[108]->offset=(short)((char *)&prec->vhys - (char *)prec);
  pdbRecordType->papFldDes[109]->size=sizeof(prec->vade);
  pdbRecordType->papFldDes[109]->offset=(short)((char *)&prec->vade - (char *)prec);
  pdbRecordType->papFldDes[110]->size=sizeof(prec->vmde);
  pdbRecordType->papFldDes[110]->offset=(short)((char *)&prec->vmde - (char *)prec);
  pdbRecordType->papFldDes[111]->size=sizeof(prec->vlal);
  pdbRecordType->papFldDes[111]->offset=(short)((char *)&prec->vlal - (char *)prec);
  pdbRecordType->papFldDes[112]->size=sizeof(prec->dlar);
  pdbRecordType->papFldDes[112]->offset=(short)((char *)&prec->dlar - (char *)prec);
  pdbRecordType->papFldDes[113]->size=sizeof(prec->slar);
  pdbRecordType->papFldDes[113]->offset=(short)((char *)&prec->slar - (char *)prec);
  pdbRecordType->papFldDes[114]->size=sizeof(prec->alst);
  pdbRecordType->papFldDes[114]->offset=(short)((char *)&prec->alst - (char *)prec);
  pdbRecordType->papFldDes[115]->size=sizeof(prec->lals);
  pdbRecordType->papFldDes[115]->offset=(short)((char *)&prec->lals - (char *)prec);
  pdbRecordType->papFldDes[116]->size=sizeof(prec->muxm);
  pdbRecordType->papFldDes[116]->offset=(short)((char *)&prec->muxm - (char *)prec);
  pdbRecordType->papFldDes[117]->size=sizeof(prec->muxp);
  pdbRecordType->papFldDes[117]->offset=(short)((char *)&prec->muxp - (char *)prec);
  pdbRecordType->papFldDes[118]->size=sizeof(prec->muxa);
  pdbRecordType->papFldDes[118]->offset=(short)((char *)&prec->muxa - (char *)prec);
  pdbRecordType->papFldDes[119]->size=sizeof(prec->muxb);
  pdbRecordType->papFldDes[119]->offset=(short)((char *)&prec->muxb - (char *)prec);
    pdbRecordType->rec_size = sizeof(*prec);
    return(0);
}
epicsExportRegistrar(costarRecordSizeOffset);
#ifdef __cplusplus
}
#endif
#endif /*GEN_SIZE_OFFSET*/
