/* recReadoutFPGA.c - record support routines for the SSD readout board FPGA chip (STAR/SSD) */
/*
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author:		David Bonnet
 *	Date:		20/06/01
 */

#include <stdio.h>
#include <math.h>
#include	<string.h>
#include "dbEvent.h"
#include "recGbl.h"
#include	<alarm.h>
#include 	<special.h>
#include	<dbDefs.h>
#include	<dbAccess.h>
#include	<dbScan.h>
#include	<dbFldTypes.h>
#include	<devSup.h>
#include	<errMdef.h>
#include	<recSup.h>
#include 	<jtagCommon.h>
#include 	<jtagReadout.h>
#define GEN_SIZE_OFFSET
#include 	<ReadoutFPGARecord.h>
#undef GEN_SIZE_OFFSET

/* Create RSET - Record Support Entry Table*/
#define report NULL
static long initialize();
static long init_record();
static long process();
#define special NULL
static long get_value(); 
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
#define get_precision NULL
static long get_enum_str();
static long get_enum_strs();
static long put_enum_str();
static long get_graphic_double();
#define get_control_double NULL
#define get_alarm_double NULL


rset ReadoutFPGARSET={
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double
};
epicsExportAddress(rset,ReadoutFPGARSET);

int async_readout(ReadoutFPGARecord *);
static long Readout_io();
static void monitor();
static void alarm();

/*extern SEM_ID sem_jtag;*/ /* Coming from recA128C.c */
unsigned short relay=1;

static long initialize()
{
	return 0;
}

static long init_record(ReadoutFPGARecord *pReadout,int pass)
{
	return 0;
}

static long process(ReadoutFPGARecord *pReadout)
{
	long status;
	unsigned char pact=pReadout->pact;
	
	status=Readout_io(pReadout); /* Do Readout Board I/O operation */
	if(!pact && pReadout->pact) return 0;
	
	pReadout->pact=TRUE;
	
	alarm(pReadout);
	monitor(pReadout);
	recGblGetTimeStamp(pReadout);
	recGblFwdLink(pReadout);

	pReadout->pact=FALSE;
	return(status);
}
	
static long get_value(ReadoutFPGARecord *pReadout,struct valueDes *pvdes)
{
    pvdes->field_type = DBF_ENUM;
    pvdes->no_elements=1;
    pvdes->pvalue = &pReadout->val;
    return(0);
}

static long get_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char	  *pstring;
{
    struct ReadoutFPGARecord	*pReadout=(struct ReadoutFPGARecord *)paddr->precord;

    if(paddr->pfield==(void *)&pReadout->val)
    {
       if(pReadout->val==0) {
	  strcpy(pstring,"NOP");
       } else if(pReadout->val==1) {
	  strcpy(pstring,"CONFIG");
       } else if(pReadout->val==2) {
	  strcpy(pstring,"STATUS");
       } else if(pReadout->val==3) {
	  strcpy(pstring,"VERSION");
       } else if(pReadout->val==4) {
	  strcpy(pstring,"CONTROL");
       } else if(pReadout->val==5) {
	  strcpy(pstring,"DAQ");
       } else if(pReadout->val==6) {
	  strcpy(pstring,"FPGA");
       } else if(pReadout->val==7) {
	  strcpy(pstring,"RESET");
       } else if(pReadout->val==8) {
	  strcpy(pstring,"ON");
       } else if(pReadout->val==9) {
	  strcpy(pstring,"OFF");
       } else if(pReadout->val==10) {
	  strcpy(pstring,"CRAT1_ON");
       } else if(pReadout->val==11) {
	  strcpy(pstring,"CRAT1_OFF");
       } else if(pReadout->val==12) {
	  strcpy(pstring,"CRAT2_ON");
       } else if(pReadout->val==13) {
	  strcpy(pstring,"CRAT2_OFF");
       } else {
	  strcpy(pstring,"Illegal_Value");
       }
    }
    return(0);
}

static long get_enum_strs(paddr,pes)
    struct dbAddr *paddr;
    struct dbr_enumStrs *pes;
{
    struct ReadoutFPGARecord	*pReadout=(struct ReadoutFPGARecord *)paddr->precord;

    if(paddr->pfield==(void *)&pReadout->val)
    {
       pes->no_str = 14;
       memset(pes->strs,'\0',sizeof(pes->strs));
       strcpy(pes->strs[0],"NOP");
       strcpy(pes->strs[1],"CONFIG");   
       strcpy(pes->strs[2],"STATUS");   
       strcpy(pes->strs[3],"VERSION");   
       strcpy(pes->strs[4],"CONTROL");   
       strcpy(pes->strs[5],"DAQ");   
       strcpy(pes->strs[6],"FPGA");   
       strcpy(pes->strs[7],"RESET");   
       strcpy(pes->strs[8],"ON");   
       strcpy(pes->strs[9],"OFF");   
       strcpy(pes->strs[10],"CRAT1_ON");   
       strcpy(pes->strs[11],"CRAT1_OFF");   
       strcpy(pes->strs[12],"CRAT2_ON");   
       strcpy(pes->strs[13],"CRAT2_OFF");   
    }
    return(0);   
}
static long get_graphic_double(struct dbAddr *paddr,struct dbr_grDouble	*pgd)
{
    	ReadoutFPGARecord	*pReadout=(ReadoutFPGARecord *)paddr->precord;
	
     	if(paddr->pfield==(void *)&pReadout->tmp1 || 
           paddr->pfield==(void *)&pReadout->tmp2)
	{ 
         	pgd->upper_disp_limit = 60; 
         	pgd->lower_disp_limit = 0;
	} 
        else recGblGetGraphicDouble(paddr,pgd);
	 
    	return(0);
}


static long put_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char          *pstring;
{
    struct ReadoutFPGARecord     *pReadout=(struct ReadoutFPGARecord *)paddr->precord;

    if(paddr->pfield==(void *)&pReadout->val)
    {
       if(strcmp(pstring,"NOP")==0) pReadout->val = 0;
       else  if(strcmp(pstring,"CONFIG")==0) pReadout->val = 1;
       else  if(strcmp(pstring,"STATUS")==0) pReadout->val = 2;
       else  if(strcmp(pstring,"VERSION")==0) pReadout->val = 3;
       else  if(strcmp(pstring,"CONTROL")==0) pReadout->val = 4;
       else  if(strcmp(pstring,"DAQ")==0) pReadout->val = 5;
       else  if(strcmp(pstring,"FPGA")==0) pReadout->val = 6;
       else  if(strcmp(pstring,"RESET")==0) pReadout->val = 7;
       else  if(strcmp(pstring,"ON")==0) pReadout->val = 8;
       else  if(strcmp(pstring,"OFF")==0) pReadout->val = 9;
       else  if(strcmp(pstring,"CRAT1_ON")==0) pReadout->val = 10;
       else  if(strcmp(pstring,"CRAT1_OFF")==0) pReadout->val = 11;
       else  if(strcmp(pstring,"CRAT2_ON")==0) pReadout->val = 12;
       else  if(strcmp(pstring,"CRAT2_OFF")==0) pReadout->val = 13;
       else return(S_db_badChoice);
    }    
    return(0);
}
static void alarm(ReadoutFPGARecord *pReadout)
{	
	
   if(/*(pReadout->udf == TRUE)||(pReadout->rdpw == 0)||*/(pReadout->fpst != 0))
   {
      recGblSetSevr(pReadout,UDF_ALARM,INVALID_ALARM);
      return;
   }
	
   if(pReadout->tcks&&pReadout->ver2&&!pReadout->trck)
   {
      recGblSetSevr(pReadout,DISABLE_ALARM,pReadout->tcks);
      return;
   }	
   
   if(pReadout->glas&&pReadout->ver2&&pReadout->gler)
   {
      recGblSetSevr(pReadout,COMM_ALARM,pReadout->glas);
      return;
   }	

   if(pReadout->luev&&pReadout->ver2)
   {
      recGblSetSevr(pReadout,HIGH_ALARM,pReadout->luas);
      return;
   }	
   
   return;	
}	

static void monitor(ReadoutFPGARecord *pReadout)
{
	unsigned short	monitor_mask,mm;
	float		delta;

        monitor_mask = mm = recGblResetAlarms(pReadout);
	/* check for value change */
	if (pReadout->val!=0) {
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->trsb,monitor_mask);
	   db_post_events(pReadout,&pReadout->fets,monitor_mask);
	   db_post_events(pReadout,&pReadout->val,monitor_mask);	
	   db_post_events(pReadout,&pReadout->itlk,monitor_mask);	
	}
	
	monitor_mask = mm;
	
	if (pReadout->val==2) /*STATUS*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->rdlc,monitor_mask);
	   db_post_events(pReadout,&pReadout->luev,monitor_mask);
	   db_post_events(pReadout,&pReadout->gler,monitor_mask);
	   db_post_events(pReadout,&pReadout->dtsd,monitor_mask);
	   db_post_events(pReadout,&pReadout->tmpv,monitor_mask);
	   db_post_events(pReadout,&pReadout->trck,monitor_mask);
	   db_post_events(pReadout,&pReadout->trbb,monitor_mask);
	   db_post_events(pReadout,&pReadout->luct,monitor_mask);
	   db_post_events(pReadout,&pReadout->pmen,monitor_mask);
	   db_post_events(pReadout,&pReadout->pcen,monitor_mask);
	   db_post_events(pReadout,&pReadout->cxid,monitor_mask);
	   db_post_events(pReadout,&pReadout->nbev,monitor_mask);
	   db_post_events(pReadout,&pReadout->neve,monitor_mask);
	   db_post_events(pReadout,&pReadout->neva,monitor_mask);
	   db_post_events(pReadout,&pReadout->dsta,monitor_mask);
	   db_post_events(pReadout,&pReadout->ovar,monitor_mask);
	   if (pReadout->tmpv)
	   {
	      db_post_events(pReadout,&pReadout->tmp1,monitor_mask);
	      db_post_events(pReadout,&pReadout->tmp2,monitor_mask);
	   }
	}
        
	monitor_mask = mm;
	if (pReadout->val==3) /*VERSION*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->lmyr,monitor_mask);
	   db_post_events(pReadout,&pReadout->lmmh,monitor_mask);
	   db_post_events(pReadout,&pReadout->lmdy,monitor_mask);
	   db_post_events(pReadout,&pReadout->ver1,monitor_mask);
	   db_post_events(pReadout,&pReadout->ver2,monitor_mask);
	   db_post_events(pReadout,&pReadout->btyp,monitor_mask);
	   db_post_events(pReadout,&pReadout->ovar,monitor_mask);
	   db_post_events(pReadout,&pReadout->ofab,monitor_mask);
	}

	monitor_mask = mm;
	if (pReadout->val==6) /*FPGA programming*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->fpst,monitor_mask);
	}

	monitor_mask = mm;
	if (pReadout->val==7) /*Reset*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->jclk,monitor_mask);
	   db_post_events(pReadout,&pReadout->fpst,monitor_mask);
	   db_post_events(pReadout,&pReadout->ver1,monitor_mask);
	   db_post_events(pReadout,&pReadout->ver2,monitor_mask);
	   db_post_events(pReadout,&pReadout->gler,monitor_mask);
	   db_post_events(pReadout,&pReadout->trck,monitor_mask);
	}

	monitor_mask = mm;
	if ((pReadout->val==8)||(pReadout->val==9)) /*Power on or off*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->rdpw,monitor_mask);
	   db_post_events(pReadout,&pReadout->fpst,monitor_mask);
	   db_post_events(pReadout,&pReadout->ver1,monitor_mask);
	   db_post_events(pReadout,&pReadout->ver2,monitor_mask);
	   db_post_events(pReadout,&pReadout->gler,monitor_mask);
	   db_post_events(pReadout,&pReadout->trck,monitor_mask);
	}

	monitor_mask = mm;
	if ((pReadout->val==10)||(pReadout->val==11)) /*Caen crate 1  power on or off*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->c1pw,monitor_mask);
	}

	monitor_mask = mm;
	if ((pReadout->val==12)||(pReadout->val==13)) /*Caen crate 1  power on or off*/{
		/* post events for value change */
		monitor_mask |= DBE_VALUE;
	}

	/* send out monitors connected to the value field */
	if (monitor_mask)
	{
	   db_post_events(pReadout,&pReadout->c2pw,monitor_mask);
	}
}

static long Readout_io(ReadoutFPGARecord *pReadout)
{
   long status;
   
   if(pReadout->pact) return 0;
   else
   {
      pReadout->pact=TRUE;
        async_readout(pReadout);     
      /*taskSpawn("tRead",ASYN_READOUT_TASK_PRIORITY,0,ASYN_READOUT_TASK_STACK,&async_readout,(int)pReadout,0,0,0,0,0,0,0,0,0);*/
   }  
   return 0;
}

int async_readout(ReadoutFPGARecord *pReadout)
{
   struct rset *prset=(struct rset *)(pReadout->rset);
   chip rd_chip[3]={READOUT,END};
   unsigned char adj,chs,FPGA_program_filename[128],temp[20];
   unsigned int  cfg1,cfg2,tempo=1,i,j;
   unsigned short status,version[4];
   unsigned long ped,cfg;
   char took_sem_jtag=0;

   if(pReadout->val<8){
     /*semTake(sem_jtag,-1);	 */  	/* Protect this process from the others, wait forever */
     connect_tap(pReadout->rdnb*10);
     took_sem_jtag=1;
   }
  /* printf("readout:%d\n",pReadout->val);*/
   switch(pReadout->val)
   {
   /* NOP */
   case 0 : 
	    break;
   
   /* CONFIG */
   case 1 : if(!(relay&0x20)) break;
            set_chain_readout(0,rd_chip); 	/* Readout board alone in the jtag chain */
	    cfg1=pReadout->thdl&0xff;		/* Hold delay for internal pulse */
	    cfg1|=(pReadout->cadl&0xff)<<8;	/* Hold delay for calibration */ 
	    cfg1|=(pReadout->phdl&0xff)<<16;	/* Hold delay for trigger event */
	    cfg1|=(pReadout->tml1&0x0f)<<24;	/* Ladder number 1 for transparent mode */
	    cfg1|=(pReadout->tml2&0x0f)<<28;	/* Ladder number 2 for transparent mode */
	    cfg2=(pReadout->itxo&0x1);		/* Init txo readout */
	    cfg2|=(pReadout->fill&0x3)<<1;	/* Daq filling type */
	    cfg2|=(pReadout->l2ac&0x1)<<3;	/* L2 accept */
	    cfg2|=(pReadout->togs&0x1)<<5;	/* Toggle side (0 = side selected by switch on the board
	    						1 = other side  + Force the two other bits to zero */
	    cfg2|=(pReadout->hsca&0x1)<<6;
	    set_config_readout(cfg1,cfg2,rd_chip);	/* set config */
/*	    printf("config set : cfg1:%x cfg2:%x\n",cfg1,cfg2);*/
	    
	    ped=pReadout->ped0|(pReadout->ped1<<2)|(pReadout->ped2<<4)|(pReadout->ped3<<6)|
	    (pReadout->ped4<<8)|(pReadout->ped5<<10)|(pReadout->ped6<<12)|
	    (pReadout->ped7<<14)|(pReadout->ped8<<16)|(pReadout->ped9<<18);
	    set_pedestal_readout(ped,rd_chip);
            adj=pReadout->tpos|(pReadout->pclk<<3);
	    set_adjust_readout(adj,rd_chip);
	    set_chain_readout(pReadout->trsb|(2*pReadout->fets),rd_chip);/* get back to the previous state */
	    break;   

   /* STATUS */	    
   case 2 : if(!(relay&0x20)) break;
            set_chain_readout(0,rd_chip); 	/* Readout board alone in the jtag chain */
   	    
	    status=get_status_readout(rd_chip); /* get status */
	    /*printf("status %d : %d\n",pReadout->rdnb,status);*/
	    pReadout->rdlc=status&0x1f;		/* readout board location */
	    pReadout->luev=(status>>5)&1;	/* latchup ? */
	    pReadout->gler=(status>>6)&1;	/* Giga-link error ? */
	    pReadout->dtsd=(status>>7)&1;	/* Detector side (P or N) */
	    pReadout->tmpv=(status>>8)&1;	/* Temperature validity */
	    pReadout->trck=(status>>9)&1;	/* Trigger clock ok ? */
	    pReadout->trbb=(status>>10)&1;	/* Trigger busy back */
	    pReadout->pcen=(status>>11)&1;	/* PC enable */
	    pReadout->pmen=(status>>12)&1;	/* PM enable */
	    pReadout->luct=get_latchup_readout(rd_chip);  /* latchup counter */
	    pReadout->cxid=get_id_readout(rd_chip);

/*	    pReadout->nbev=get_nb_event_readout(rd_chip);*/ /*nbr of event start*/

	    get_nb_event2_readout(version,rd_chip); /*nbr of event start,end and abort*/
	    pReadout->nbev=version[0];
	    pReadout->neve=version[1];
	    pReadout->neva=version[2];
/*	    printf("number of events : version2:%x version1:%x version0:%x\n",version[2],version[1],version[0]); */
	    get_optic_var_readout(version,rd_chip);
	    for(i=0;i<3;i++) sprintf(pReadout->ovar+i*4,"%04x",version[2-i]);

	    get_daq_status_readout(version,rd_chip); /* get the daq register */
/*	    printf("daq register : version2:%x version1:%x version0:%x\n",version[2],version[1],version[0]);*/
	  
	    for(i=0;i<3;i++) sprintf(pReadout->dsta+i*4,"%04x",version[2-i]);
	    /*for(i=2;i<12;i++) if(pReadout->dsta[i]==0x20) pReadout->dsta[i]=0x30;*/  
	    if(pReadout->tmpv)			/* if temperature is valid, get it */
	    {
	       cfg=get_temp1_readout(rd_chip);
	       if(cfg!=0)
	       {
	          pReadout->tmp1=(cfg&0xfff)/10-273;
/* 		  printf("Readout TMP2 avant :%d\n",cfg); */
		  cfg=cfg>>12;
/* 		  printf("Readout TMP2 apres shift :%d\n",cfg); */
	          pReadout->tmp2=(cfg&0xfff)/10-273;
		  if(fabs(pReadout->tmp2-pReadout->tmp1)>1000) /* sometimes the tmp2 value is bad...*/
		    pReadout->tmp2=pReadout->tmp1;   /* in this case just replace it with tmp1 ! */
	       }
	    }
	    
	    set_chain_readout(pReadout->trsb|(2*pReadout->fets),rd_chip);/* get back to the previous state */
   	    
	    break;   
   
   /* VERSION */	     	    
   case 3 : if(!(relay&0x20)) break;
            set_chain_readout(0,rd_chip); 	/* Readout board alone in the jtag chain */
   	    
	    get_version_readout(version,rd_chip);
	    sprintf(temp,"%x",version[0]); 
	    sscanf(temp,"%d",&cfg);   
	    pReadout->lmyr=cfg;	/* date of last modification : year */	    
	    sprintf(temp,"%x",version[1]&0xff);    
	    sscanf(temp,"%d",&cfg);   
	    pReadout->lmmh=cfg;	/* month */
            sprintf(temp,"%x",(version[1]>>8)&0xff);    
	    sscanf(temp,"%d",&cfg);   
            pReadout->lmdy=cfg;/* day */
	    pReadout->ver1=version[2]&0x0f;	/* code version */
	    pReadout->ver2=(version[2]>>4)&0x0f;/* code sub-version */
	    pReadout->btyp=(version[2]>>8)&0xff;/* board type */
	    
	    get_optic_fab_readout(version,rd_chip);
	    for(i=0;i<4;i++) sprintf(pReadout->ofab+i*4,"%04x",version[3-i]);
	    	    
	    set_chain_readout(pReadout->trsb|(2*pReadout->fets),rd_chip);/* get back to the previous state */
   	    
	    break;
   
   /* CONTROL */		
   case 4 : if(!(relay&0x20)) break;
            set_chain_readout(0,rd_chip);  	/* Readout board alone in the jtag chain */
   	    tempo=1000; 
	    while (((get_status_readout(rd_chip)>>10)&1)&&(--tempo)); /* wait for readout board ready (trg_busy_back=0) */
	    if(!tempo) break; 			/* if timeout, give up */
   	    set_chain_readout(3,rd_chip);	/* full jtag chain control */
   	    pReadout->trsb=1;
	    pReadout->fets=1;	     
   	    break;
   
   /* DAQ */	     
   case 5 : if(!(relay&0x20)) break;
            set_chain_readout(0,rd_chip);	/* give address bus control to the daq */
   	    pReadout->trsb=0;
	    pReadout->fets=0;	     
	    break;	     
   
   /* FPGA */	/* Program FPGA */
   case 6 : 
   	    if(!(relay&0x20)) break;
	    connect_tap(64+(pReadout->rdnb<<4)); 	/* connect to FPGAs Jtag chain */
   	    strcpy(FPGA_program_filename,pReadout->fppt);
	    strcat(FPGA_program_filename,pReadout->fpnm); /* path + filename */
	    pReadout->fpst=jbi(FPGA_program_filename,"configure");
   	    connect_tap(pReadout->rdnb*10);	   	/* connect to readout board Jtag chain */	
   	    break;

   /* RESET */	/* Reset all jtag chain and set jtag clock */
   case 7 : 
	    pReadout->fpst=0; 
	    pReadout->ver2=0; 
	    pReadout->ver1=0; 
	    pReadout->trck=1; 
	    pReadout->gler=0; 
	    if(!(relay&0x20)) break;
            
	     /* calculate reset paremeters for the given jtag clock value */
   	     if (pReadout->jclk>390500) 
             {
	        i=0;
		for(j=0;(j<15)&&(pReadout->jclk<(25000000/(2*(j+1))));j++);
	        pReadout->jclk=25000000/(2*(j+1));  
	     }
             else if(pReadout->jclk>24405) 
             {
	        i=1;
		for(j=0;(j<15)&&(pReadout->jclk<(781000/(2*(j+1))));j++);
	        pReadout->jclk=781000/(2*(j+1));  
	     }
   	     else 
	     {
	        i=3;
		for(j=0;(j<15)&&(pReadout->jclk<(55000/(2*(j+1))));j++);
	        pReadout->jclk=55000/(2*(j+1));  
	     }
	    
	     if (!reset_jtag_var(1,i,j)) 	/* Hard reset */
		printf("reset JTAG : OK (%d, %d)\n",i,j);
   	     else 
		printf(" ERROR : JTAG Board not detected.\n");
   break;
   
   /* ON */ /* Power on the readout board */
   case 8 :  
	    relay|=0x20;
	    writea16(PIO2_OUT,relay);
	    pReadout->rdpw=1;   
   break;
   	
   /* OFF */ /* Power off the readout board */
   case 9 :  
            relay&=0xffff-0x20;
	    writea16(PIO2_OUT,relay);   
	    pReadout->rdpw=0;   
	    pReadout->fpst=0; 
	    pReadout->ver2=0; 
	    pReadout->ver1=0; 
	    pReadout->trck=1; 
	    pReadout->gler=0; 
   break;
   
   /* CRAT1_ON */ /* Power on the first Caen Crate */
   case 10 :  
            relay&=0xffff-0x80;
	    writea16(PIO2_OUT,relay);
	    pReadout->c1pw=1;   
   break;
   	
   /* CRAT1_OFF */ /* Power off the first Caen Crate */
   case 11 :  
	    relay|=0x80;
	    writea16(PIO2_OUT,relay);   
	    pReadout->c1pw=0;   
   break;
   
   /* CRAT2_ON */ /* Power on the first Caen Crate */
   case 12 :  
            relay&=0xffff-0x40;
	    writea16(PIO2_OUT,relay);
	    pReadout->c2pw=1;   
   break;
   	
   /* CRAT2_OFF */ /* Power off the first Caen Crate */
   case 13 :  
	    relay|=0x40;
	    writea16(PIO2_OUT,relay);   
	    pReadout->c2pw=0;   
   break;
   	    
   default : break;
   }
   reada16(PIO2_IN,&status);
   if(status&1)
     pReadout->itlk=1;
   else
     pReadout->itlk=0;
   /*if(took_sem_jtag)
     semGive(sem_jtag);	*/		   		/* release the protection */
   dbScanLock((struct dbCommon *)pReadout);
   (*prset->process)(pReadout);
   dbScanUnlock((struct dbCommon *)pReadout);

   if(pReadout->val>9)
     pReadout->val=0;
   else if(!(relay&0x20)){ /* if power is off */
      pReadout->val=9;
      pReadout->rdpw=0;   
   }
   else {
      pReadout->rdpw=1;  
      if((pReadout->val==6)&&(pReadout->fpst!=0)) pReadout->val=1; /* if fpga programmation failed */ 
      else if(tempo) pReadout->val=0; /* everything is ok */
      else      pReadout->val=5; /* if failure while getting full control */ 
   }
   db_post_events(pReadout,&pReadout->val,DBE_VALUE);
   
   return 0;
}

