/* jtagA128C.h		Jtag A128C functions header 
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	David Bonnet
 *	Date :		22/03/00
 *
 */

/* Alice128C registers size */

#define AL_IR_SZ	5
#define AL_BSR_SZ	8
#define AL_BYPASS_SZ	1
#define AL_BIAS_SZ	56
#define AL_PULSE_SZ 	8
#define AL_TEST_SZ	128
#define AL_READ_SZ	258
#define AL_ONOFF_SZ 	1
#define AL_TOKEN_SZ	1

/* Alice128C commands */

#define AL_BYPASS	0x1f
#define AL_EXT_TEST	0x00
#define AL_BSR_SP	0x05
#define AL_BIAS		0x09
#define AL_PULSE	0x0d
#define AL_TEST		0x11
#define AL_READ		0x15
#define AL_ONOFF	0x19
#define AL_TOKEN	0x1d

#define NB_MAX_A128C    96
#define ASYN_A128C_TASK_DELAY 120 	/* 2 seconds */
#define ASYN_A128C_TASK_PRIORITY 209
#define ASYN_A128C_TASK_STACK 10000
#define MAP_MAX_SIZE    150

typedef struct
{
   unsigned char bias[7];
   unsigned char pulse;
   unsigned char onoff;
   unsigned char token;
   unsigned short test[8];
   unsigned short read[17];
   
}a128c ;

/* Function prototypes */

unsigned short map_a128c_nb(chip *);
void send_all_a128c(unsigned long, unsigned short, unsigned short *, unsigned short *, chip *);
unsigned long send_one_a128c(unsigned long ,unsigned short , unsigned long ,unsigned short ,chip *);
void get_all_a128c(unsigned long ,unsigned short ,unsigned short *,chip *);
unsigned long get_one_a128c(unsigned long ,unsigned short ,unsigned short ,chip *);
void set_pulse_all_a128c(unsigned char ,chip *);
unsigned char set_pulse_one_a128c(unsigned char ,unsigned short ,chip *);
void set_bias_all_a128c(unsigned char [][7],unsigned char [][7],chip *);
void set_bias_one_a128c(unsigned char [7],unsigned char [7],unsigned short ,chip *);
void set_onoff_all_a128c(unsigned char *,chip *);
unsigned char set_onoff_one_a128c(unsigned char ,unsigned short ,chip *);
void set_token_all_a128c(unsigned char *,chip *);
unsigned char set_token_one_a128c(unsigned char ,unsigned short ,chip *);
void  set_test_one_a128c(unsigned short *,unsigned short *,unsigned short ,chip *);
void transp_chan_a128c(unsigned char ,unsigned short ,chip *);
