/* jtagCostar.c		Jtag costar functions 
 *
 *	Institut de Recherche Subatomique de Strasbourg
 *
 *	Author :	David Bonnet
 *	Date :		22/03/00
 *
 */

#include "stdio.h"
#include "jtagCommon.h"
#include "jtagCostar.h"

 

/***************************************************************************************


	map_costar_nb(map)
	
	chip 		*map
	
	return : 	unsigned short costar_nb
	
	Description :  	this function determines how many costars are in the chain, 
			according to the "map".


****************************************************************************************/

unsigned short map_costar_nb(chip *map)
{
    unsigned short i=0,icost=0;
    
    for (;(map[i]!=END)&&(i<MAX_CHIPS);i++)
        if (map[i]==COSTAR) icost++;
    if(i==MAX_CHIPS) icost=0;
    return icost;
}
  
/***************************************************************************************

	send_all_costar(instruction,instr_siz,list_in,list_out,map)
	
	unsigned long 	instruction
	unsigned short 	instr_siz
	unsigned short 	*list_in
	unsigned short 	*list_out
	chip 		*map
	
	return : 	nothing
	
	Description : 	this function send the instruction to all the costar in the chain 
			(map descibes the chain), then send the data from list_out. the data returned
			by the costars are in list_in.
				

******************************************************************************************/

 
void send_all_costar(unsigned long instruction,unsigned short instr_siz,
		     unsigned short *list_in,unsigned short *list_out,chip *map)
{
    unsigned short stream_in[MAX_LENGTH_STREAM];
    unsigned short stream_out[MAX_LENGTH_STREAM];
    unsigned short nb_chip,i,istr,idat;    


    if (map_costar_nb(map)==0) return ;
    nb_chip = map_nb(map);
#if OUT_PUT
    printf("\n%s, %d: %s start\n", __FILE__, __LINE__, __func__);
    print_map(map);
#endif      
    init_stream(stream_out,CO_IR_SZ*nb_chip);	/* fill the output stream with BYPASS instructions */
    
    for (i=0;i<nb_chip;i++)					/* put the instruction for the COSTARs in */
        if (map[i]==COSTAR)					/* the input stream. 			  */
	    set_stream(stream_out,i*CO_IR_SZ,instruction,CO_IR_SZ); 
	       
    scan_ir(stream_out,CO_IR_SZ*nb_chip,stream_in);		/* send the instruction stream */
    
    for (i=0,istr=0,idat=0;i<nb_chip;i++)			/* create the data stream to be sent */
        if (map[i]==COSTAR)
	   {
	   set_stream(stream_out,istr,get_stream(list_out,idat,instr_siz),instr_siz);
	   idat+=instr_siz;
	   istr+=instr_siz;
	   }
	else istr+=CO_BYPASS_SZ; 
    
    scan_dr(stream_out,istr,stream_in);				/* send the data stream */
    
    for (i=0,istr=0,idat=0;i<nb_chip;i++)			/* put the data from stream_in into list_in */
        if (map[i]==COSTAR)
	   {
	   set_stream(list_in,idat,get_stream(stream_in,istr,instr_siz),instr_siz);
	   idat+=instr_siz;
	   istr+=instr_siz;
	   }
	else istr+=CO_BYPASS_SZ;
    /*print_stream(list_in,map_costar_nb(map)*instr_siz);*/
#if OUT_PUT
    printf("===========================================\n");
#endif   
    return;
}    
    
/***************************************************************************************

	send_some_costar(instruction,instr_siz,list_in,list_out,list_chip,map)
	
	unsigned long 	instruction
	unsigned short 	instr_siz
	unsigned short 	*list_in
	unsigned short 	*list_out
	char		*list_chip
	chip 		*map
	
	return : 	nothing
	
	Description : 	this function send the instruction to some of the costar in the chain 
			(map descibes the chain, list_chip tells which chip is concerned ), then send the data 
			from list_out. the data returned by the costars are in list_in.
				

******************************************************************************************/

 
void send_some_costar(unsigned long instruction,unsigned short instr_siz,
		     unsigned short *list_in,unsigned short *list_out,char *list_chip,chip *map)
{
    unsigned short stream_in[MAX_LENGTH_STREAM];
    unsigned short stream_out[MAX_LENGTH_STREAM];
    unsigned short nb_chip,i,j,istr,idat;    


    if (map_costar_nb(map)==0) return ;
    nb_chip = map_nb(map);
#if OUT_PUT
    printf("\n%s, %d: %s start\n", __FILE__, __LINE__, __func__);
    print_map(map);
#endif      
    init_stream(stream_out,CO_IR_SZ*nb_chip);	/* fill the output stream with BYPASS instructions */
    
    for (i=0,j=0;i<nb_chip;i++)					/* put the instruction for the COSTARs in */
        if ((map[i]==COSTAR)&&list_chip[j++])			/* the input stream. 			  */
	    set_stream(stream_out,i*CO_IR_SZ,instruction,CO_IR_SZ);
	       
    scan_ir(stream_out,CO_IR_SZ*nb_chip,stream_in);		/* send the instruction stream */
    
    for (i=0,istr=0,idat=0,j=0;i<nb_chip;i++)			/* create the data stream to be sent */
        if ((map[i]==COSTAR)&&list_chip[j++])
	   {
	   set_stream(stream_out,istr,get_stream(list_out,idat,instr_siz),instr_siz);
	   idat+=instr_siz;
	   istr+=instr_siz;
	   }
	else istr+=CO_BYPASS_SZ; 
    
    scan_dr(stream_out,istr,stream_in);				/* send the data stream */
    
    for (i=0,istr=0,idat=0,j=0;i<nb_chip;i++)			/* put the data from stream_in into list_in */
        if ((map[i]==COSTAR)&&list_chip[j++])
	   {
	   set_stream(list_in,idat,get_stream(stream_in,istr,instr_siz),instr_siz);
	   idat+=instr_siz;
	   istr+=instr_siz;
	   }
	else istr+=CO_BYPASS_SZ;
    /*print_stream(list_in,map_costar_nb(map)*instr_siz);*/
#if OUT_PUT
    printf("===========================================\n");
#endif   
    return;
}    
    
/***************************************************************************************

	get_some_costar(instruction,instr_siz,list_in,list_chip,map)
	
	unsigned long 	instruction
	unsigned short 	instr_siz
	unsigned short 	*list_in
	char		*list_chip
	chip 		*map
	
	return : 	nothing
	
	Description : 	this function send the instruction to some of the costar in the chain 
			(map descibes the chain, list_chip tells which chip is concerned )
			the data returned by the costars are in list_in.
				

******************************************************************************************/

 
void get_some_costar(unsigned long instruction,unsigned short instr_siz,
		     unsigned short *list_in,char *list_chip,chip *map)
{
    unsigned short stream_in[MAX_LENGTH_STREAM];
    unsigned short stream_out[MAX_LENGTH_STREAM];
    unsigned short nb_chip,i,j,istr,idat,nb_cost;    


    if ((nb_cost=map_costar_nb(map))==0) return ;
    nb_chip = map_nb(map);
#if OUT_PUT
    printf("\n%s, %d: %s start\n", __FILE__, __LINE__, __func__);
    print_map(map);
#endif      
    init_stream(stream_out,CO_IR_SZ*nb_chip);	/* fill the output stream with BYPASS instructions */
    
    for (i=0,j=0;i<nb_chip;i++)					/* put the instruction for the COSTARs in */
        if ((map[i]==COSTAR)&&list_chip[j++])			/* the input stream. 			  */
	    set_stream(stream_out,i*CO_IR_SZ,instruction,CO_IR_SZ);
	       
    scan_ir(stream_out,CO_IR_SZ*nb_chip,stream_in);		/* send the instruction stream */
    
    for (i=0,istr=0,j=0;i<nb_chip;i++)			/* calculate the number of bit received */
        if ((map[i]==COSTAR)&&list_chip[j++])		/* j++ when map[i]==COSTAR */
	   istr+=instr_siz;
	else istr+=CO_BYPASS_SZ; 
    
    circulate_dr(istr,stream_in);				/* get the data stream */
    for(i=0,j=0;i<nb_cost;i++) j+=!list_chip[i]; /* calculate the number of bypassed costars */
    for (i=0,idat=(nb_cost-1-j)*instr_siz,istr=0,j=0;i<nb_chip;i++)			/* put the data from stream_in into list_in */
        if ((map[i]==COSTAR)&&list_chip[j++])
	   {
	   set_stream(list_in,idat,get_stream(stream_in,istr,instr_siz),instr_siz);
	   idat-=instr_siz;
	   istr+=instr_siz;
	   }
	else istr+=CO_BYPASS_SZ;
    /*print_stream(list_in,map_costar_nb(map)*instr_siz);*/
#if OUT_PUT
    printf("===========================================\n");
#endif   
    return;
}    
    
  
/***************************************************************************************

	send_one_costar(instruction,instr_siz,data,costar_nb,map)
	
	unsigned long 	instruction
	unsigned short 	instr_siz
	unsigned long 	data
	unsigned short 	costar_nb
	chip 		*map
	
	return : 	unsigned long old_data
	
	
	Description : 	this function send the instruction to the costar "costar_nb" 
			(map descibes the chain), then send the data. the data returned
			by the costar (the old data) is returned by this function.
				

******************************************************************************************/


unsigned long send_one_costar(unsigned long instruction,unsigned short instr_siz,
			      unsigned long data,unsigned short costar_nb,chip *map)
{
    unsigned short stream_in[MAX_LENGTH_STREAM],stream_out[MAX_LENGTH_STREAM];
    unsigned short nb_chip,i,icost,cost_i=0;    


    if (map_costar_nb(map)==0) return 0;
    nb_chip = map_nb(map);
    
    for (i=0,icost=0;i<nb_chip;i++)            /* find the costar in the chain */ 
        if (map[i]==COSTAR)
	{
	    if (icost==costar_nb)
	    {
	        cost_i=i; 
                break;
	    }
	    else icost++; 
	}
#if OUT_PUT
    printf("\n%s, %d: %s start\n", __FILE__, __LINE__, __func__);
    print_map(map);
#endif  	
    init_stream(stream_out,CO_IR_SZ*nb_chip);	/* fill the output stream with BYPASS instructions */
    
    set_stream(stream_out,cost_i*CO_IR_SZ,instruction,CO_IR_SZ); /* put the instruction for the COSTAR in the output stream. */
	    
    scan_ir(stream_out,CO_IR_SZ*nb_chip,stream_in);		/* send the instruction stream */
    
    set_stream(stream_out,cost_i*CO_BYPASS_SZ,data,instr_siz);	/* create the data stream to be sent */
        
    scan_dr(stream_out,nb_chip*CO_BYPASS_SZ+instr_siz-1,stream_in); /* send the data stream */
    
    /*print_stream(stream_in,nb_chip*CO_BYPASS_SZ+instr_siz-1);*/
#if OUT_PUT
    printf("===========================================\n");
#endif       
    return get_stream(stream_in,cost_i*CO_BYPASS_SZ,instr_siz);   /* return the old data (returned by the chip) */
}       

  
/***************************************************************************************

	get_all_costar(instruction,instr_siz,list,map)
	
	unsigned long 	instruction
	unsigned short 	instr_siz
	unsigned short 	*list
	chip 		*map
	
	return :	nothing
	
	
	Description : 	this function send the instruction to all the costar in the chain 
			(map descibes the chain), then get the data from the costars. 
				

******************************************************************************************/

 
void get_all_costar(unsigned long instruction,unsigned short instr_siz,
		     unsigned short *list,chip *map)
{
    unsigned short stream[MAX_LENGTH_STREAM];
    unsigned short stream_in[MAX_LENGTH_STREAM];
    unsigned short nb_chip,i,istr,idat;    


    if (map_costar_nb(map)==0) return ;
    nb_chip = map_nb(map);
#if OUT_PUT
    printf("\n%s, %d: %s start\n", __FILE__, __LINE__, __func__);
    print_map(map);
#endif      
    init_stream(stream,CO_IR_SZ*nb_chip);	/* fill the output stream with BYPASS instructions */
    
    for (i=0;i<nb_chip;i++)					/* put the instruction for the COSTARs in */
        if (map[i]==COSTAR)					/* the input stream. 			  */
	    set_stream(stream,i*CO_IR_SZ,instruction,CO_IR_SZ); 
	       
    scan_ir(stream,CO_IR_SZ*nb_chip,stream_in);		/* send the instruction stream */
    
    circulate_dr(map_costar_nb(map)*(instr_siz-1)+nb_chip,stream);	/* get the data stream */
    
    for (i=0,istr=0,idat=0;i<nb_chip;i++)			/* put the data from stream into list */
        if (map[i]==COSTAR)
	   {
	   set_stream(list,idat,get_stream(stream,istr,instr_siz),instr_siz);
	   idat+=instr_siz;
	   istr+=instr_siz;
	   }
	else istr+=CO_BYPASS_SZ;
    /*print_stream(list,map_costar_nb(map)*instr_siz);*/
#if OUT_PUT
    printf("===========================================\n");
#endif   
    return;
}    
    
  
/***************************************************************************************

	get_one_costar(instruction,instr_siz,costar_nb,map)
	
	unsigned long 	instruction
	unsigned short 	instr_siz
	unsigned short 	costar_nb
	chip 		*map
	
	return :	unsigned long data
	
	Description : 	this function send the instruction to the costar "costar_nb" 
			(map descibes the chain), then get the data from this costar.
				

******************************************************************************************/


unsigned long get_one_costar(unsigned long instruction,unsigned short instr_siz,
			      unsigned short costar_nb,chip *map)
{
    unsigned short stream_in[MAX_LENGTH_STREAM],stream[MAX_LENGTH_STREAM];
    unsigned short nb_chip,i,icost,cost_i=0;    


    if (map_costar_nb(map)==0) return 0;
    nb_chip = map_nb(map);
    
    for (i=0,icost=0;i<nb_chip;i++)            /* find the costar in the chain */ 
        if (map[i]==COSTAR)
	{
	    if (icost==costar_nb)
	    {
	        cost_i=i; 
                break;
	    }
	    else icost++; 
	}
#if OUT_PUT
    printf("\n%s, %d: %s start\n", __FILE__, __LINE__, __func__);
    print_map(map);
#endif  	
    init_stream(stream,CO_IR_SZ*nb_chip);
    set_stream(stream,cost_i*CO_IR_SZ,instruction,CO_IR_SZ);
    scan_ir(stream,CO_IR_SZ*nb_chip,stream_in);
    circulate_dr(nb_chip*CO_BYPASS_SZ+instr_siz-1,stream); 
#if OUT_PUT
    printf("===========================================\n");
#endif   
    return get_stream(stream,cost_i*CO_BYPASS_SZ,instr_siz);
}       


/***************************************************************************************

	set_ctrl1_all_costar(data,map)
	
	unsigned char 	data
	chip 		*map
	
	return : nothing
	
	Description : 	this function set the control1 register of all the costars with
			the same data (map descibes the chain).
				

******************************************************************************************/ 

void  set_ctrl1_all_costar(unsigned char data,chip *map)
{
    unsigned short list_in[MAX_LENGTH_STREAM],list_out[MAX_LENGTH_STREAM];
    unsigned short nb_costar,i;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    
    for(i=0;i<nb_costar;i++)     			/* the same data for all the costars */
        set_stream(list_out,i*CO_CTRL_SZ,data,CO_CTRL_SZ);
    
    send_all_costar(CO_CTRL_1,CO_CTRL_SZ,list_in,list_out,map);
    
}


/***************************************************************************************

	set_ctrl2_all_costar(data,map)
	
	unsigned char 	data
	chip 		*map
	
	return : nothing
	
	Description : 	this function set the control1 register of all the costars with
			the same data (map descibes the chain).
				

******************************************************************************************/ 

void  set_ctrl2_all_costar(unsigned char data,chip *map)
{
    unsigned short list_in[MAX_LENGTH_STREAM],list_out[MAX_LENGTH_STREAM];
    unsigned short nb_costar,i;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    
    for(i=0;i<nb_costar;i++)    		   /* the same data for all the costars */
        set_stream(list_out,i*CO_CTRL_SZ,data,CO_CTRL_SZ);
    
    send_all_costar(CO_CTRL_2,CO_CTRL_SZ,list_in,list_out,map);
    
}


/***************************************************************************************

	set_ctrl1_one_costar(data,costar_nb,map)
	
	unsigned char 	data
	unsigned short 	costar_nb
	chip 		*map
	
	return : 	unsigned short old_data
	
	Description : 	this function set the control1 register of the costar "costar_nb". 
			
				      return the old data.	

******************************************************************************************/ 

unsigned char set_ctrl1_one_costar(unsigned char data,unsigned short costar_nb,chip *map)
{
    
    return (unsigned char) send_one_costar(CO_CTRL_1,CO_CTRL_SZ,(unsigned long) data,costar_nb,map);
    
}


/***************************************************************************************

	set_ctrl2_one_costar(data,costar_nb,map)
	
	unsigned char 	data
	unsigned short 	costar_nb
	chip 		*map
	
	return : 	unsigned short old_data
	
	Description : 	this function set the control2 register of the costar "costar_nb". 
			
				      return the old data.	

******************************************************************************************/ 

unsigned char set_ctrl2_one_costar(unsigned char data,unsigned short costar_nb,chip *map)
{
    
    return (unsigned char) send_one_costar(CO_CTRL_2,CO_CTRL_SZ,(unsigned long) data,costar_nb,map);
    
}

/***************************************************************************************

	start_conversion_all_costar(map)
	
	chip 	*map
	
	return : nothing
	
	Description : 	this function start the adc conversion on all the costars.	

******************************************************************************************/ 


void start_conversion_all_costar(chip *map)
{
    set_ctrl1_all_costar(CONTROL_REG1|0x80,map);
}


/***************************************************************************************

	stop_conversion_all_costar(map)
	
	chip 	*map
	
	return : nothing
	
	
	Description : 	this function stop the adc conversion on all the costars.	

******************************************************************************************/ 


void stop_conversion_all_costar(chip *map)
{   
    set_ctrl1_all_costar(CONTROL_REG1,map);
}

/***************************************************************************************

	start_conversion_one_costar(costar_nb,map)
	
	unsigned short 	costar_nb
	chip 		*map
	
	return : nothing
	
	
	Description : 	this function start the adc conversion on the costars "costar_nb".	

******************************************************************************************/ 


void start_conversion_one_costar(unsigned short costar_nb,chip *map)
{
 /*    unsigned char data; */
    
    /*data = (unsigned char) get_one_costar(CO_CTRL_1,CO_CTRL_SZ,costar_nb,map);*/ /* get the old ctrl1 value */
         			
    /*data |= (1<<7);*/	/* set the conversion bit */
    
    send_one_costar(CO_CTRL_1,CO_CTRL_SZ,CONTROL_REG1|0x80,costar_nb,map); /* send the new ctrl1 value */
    
}


/***************************************************************************************

	stop_conversion_one_costar(costar_nb,map)
	
	unsigned short 	costar_nb
	chip 		*map
	
	return : nothing
		
	Description : 	this function stop the adc conversion on the costars "costar_nb".	

******************************************************************************************/ 


void stop_conversion_one_costar(unsigned short costar_nb,chip *map)
{
/*     unsigned char data; */
    
    /*data = (unsigned char) get_one_costar(CO_CTRL_1,CO_CTRL_SZ,costar_nb,map);*/ /* get the old ctrl1 value */
         			
    /*data &= 0x7f ;*/ 	/* set the conversion bit */
    
    send_one_costar(CO_CTRL_1,CO_CTRL_SZ,CONTROL_REG1,costar_nb,map); /* send the new ctrl1 value */
    
}




/***************************************************************************************

	set_dac_all_costar(data,old_data,map)
	
	float 		data[number of costar][number of dac channel (4)]
	float 		old_data[number of costar][number of dac channel (4)]
	chip  		map[number of chip]
	
	return : nothing 
	
	Description : 	this function set the dinout register of all the costars according to
			the  array "data", and fill the array "old_data" (map descibes the chain).
				

******************************************************************************************/ 

void  set_dac_all_costar(unsigned char data[][4],unsigned char old_data[][4],chip *map)
{
    unsigned short list_in[MAX_LENGTH_STREAM],list_out[MAX_LENGTH_STREAM];
    unsigned short nb_costar,i,j;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    
    for(i=0;i<nb_costar;i++)     			
        for(j=0;j<4;j++)
	    set_stream(list_out,i*CO_DAC_SZ+j*(CO_DAC_SZ/4),data[i][j],CO_DAC_SZ/4);
    
    send_all_costar(CO_DAC_0,CO_DAC_SZ,list_in,list_out,map);
    
    for(i=0;i<nb_costar;i++)
        for(j=0;j<4;j++)
	    old_data[i][j] = (unsigned char) get_stream(list_in,i*CO_DAC_SZ+j*(CO_DAC_SZ/4),CO_DAC_SZ/4);
}

/***************************************************************************************

	set_dac_one_costar(data,old_data,costar_nbmap)
	
	float 		data[number of dac channel (4)]
	unsigned short	costar_nb
	chip  		map[number of chip]
	
	return : 	nothing
	
	Description : 	this function set the dinout register of the costar "costar_nb"  according to
			the  array "data", and fill the array "old_data" (map describes the chain).
				

******************************************************************************************/ 

void  set_dac_one_costar(unsigned char *data,unsigned char *old_data,unsigned short costar_nb,chip *map)
{
    unsigned long list,list_in;
    unsigned short j;
    
    list=0;    			
    for(j=0;j<4;j++)
        list+=((unsigned long) data[j])<<(j*CO_DAC_SZ/4);
    
    list_in = send_one_costar(CO_DAC_0,CO_DAC_SZ,list,costar_nb,map);
    
    for(j=0;j<4;j++)
	old_data[j] = (((unsigned char) list_in)>>(j*CO_DAC_SZ/4))&255;
}


/***************************************************************************************

	get_adc0_all_costar(data,map)
	
	float 		data[number of costar][number of adc channel (4)]
	chip  		map[number of chip]
	
	return : nothing 
	
	Description : 	this function gets the adc0 register of all the costars and puts the
			floating point values (in Volts) in the array "data" 
			(map describes the chain).
				

******************************************************************************************/ 
void get_adc0_all_costar(unsigned char data[][4],chip *map)
{
    unsigned short list_in[MAX_LENGTH_STREAM],list_out[MAX_LENGTH_STREAM];
    unsigned short nb_costar,i,chan;
    unsigned long c;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    for (chan=0;chan<4;chan++)
    {
       for(i=0;i<nb_costar;i++)
          set_stream(list_out,i*CO_ADC_TEST_SZ,8+chan,CO_ADC_TEST_SZ);
       
       send_all_costar(CO_ADC_TEST_0,CO_ADC_TEST_SZ,list_in,list_out,map); /* select the channel */
       
       start_conversion_all_costar(map);
       for(c=0;c<100000;c++);/* wait for 10 clock pulses ? */
       stop_conversion_all_costar(map);
       
       get_all_costar(CO_ADC_0,CO_ADC_SZ,list_in,map);                     /* get the datas from that channel */
    
       for(i=0;i<nb_costar;i++)						/* stock the floating point value in the array */
          data[i][(chan==3)?0:chan+1] = (unsigned char) get_stream(list_in,i*CO_ADC_SZ+chan*(CO_ADC_SZ/4),CO_ADC_SZ/4);
     
     }
}


/***************************************************************************************

	get_adc1_all_costar(data,map)
	
	float 		data[number of costar][number of adc channel (4)]
	chip  		map[number of chip]
	
	return : nothing 
	
	Description : 	this function gets the adc1 register of all the costars and puts the
			floating point values in the array "data". the conversion into floating 
			point value is done according to the type of data (temperature,voltage
			supply, etc..)
			 
	note	    :	data[n][0] : external channel 	(V)
			data[n][1] : GND - VSS 		(V)
			data[n][2] : VDD - VSS 		(V)
			data[n][3] : temperature	(C)
				

******************************************************************************************/ 
void get_adc1_all_costar(unsigned char data[][4],chip *map)
{
    unsigned short list_in[MAX_LENGTH_STREAM],list_out[MAX_LENGTH_STREAM];
    unsigned short nb_costar,i,chan;
    unsigned long c;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    for (chan=0;chan<4;chan++)
    {
       for(i=0;i<nb_costar;i++)
          set_stream(list_out,i*CO_ADC_TEST_SZ,8+chan,CO_ADC_TEST_SZ);
       
       send_all_costar(CO_ADC_TEST_1,CO_ADC_TEST_SZ,list_in,list_out,map); /* select the channel */
       
       start_conversion_all_costar(map);
       
       for(c=0;c<100000;c++);/* wait for 10 clock pulses ? */
       
       stop_conversion_all_costar(map);
      
       get_all_costar(CO_ADC_1,CO_ADC_SZ,list_in,map);                     /* get the datas from that channel */
    
       for(i=0;i<nb_costar;i++)			/* stock the floating point value in the array */
          data[i][(chan==3)?0:chan+1] = (unsigned char) get_stream(list_in,i*CO_ADC_SZ+chan*(CO_ADC_SZ/4),CO_ADC_SZ/4);
	  
     }
}


/***************************************************************************************

	get_adc0_one_costar(data,costar_nb,map)
	
	float 		data[number of adc channel (4)]
	unsigned short  costar_nb
	chip  		map[number of chip]
	
	return : nothing 
	
	Description : 	this function gets the adc0 register of the costar "costar_nb" and 
			puts the floating point values (in Volts) in the array "data" 
			(map describes the chain).
				

******************************************************************************************/ 
void get_adc0_one_costar(unsigned char data[4],unsigned short costar_nb,chip *map)
{
    unsigned long list_in;
    unsigned short nb_costar,chan;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    list_in = get_one_costar(CO_ADC_0,CO_ADC_SZ,costar_nb,map);
    for (chan=0;chan<4;chan++){
      data[chan]=((unsigned char) (list_in>>(chan*CO_ADC_SZ/4))&255);
    }
}

void get_adc0_one_costar_debug(unsigned char data[4],unsigned short costar_nb,chip *map)
{
    unsigned long list_in,c;
    unsigned short nb_costar,chan;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    for (chan=0;chan<4;chan++)
    {
       send_one_costar(CO_ADC_TEST_0,CO_ADC_TEST_SZ,8+chan,costar_nb,map); /* select the channel */
       
       start_conversion_one_costar(costar_nb,map);
       for(c=0;c<100000;c++);/* wait for 10 clock pulses ? */
       stop_conversion_one_costar(costar_nb,map);
       
       list_in=get_one_costar(CO_ADC_0,CO_ADC_SZ,costar_nb,map);                     /* get the datas from that channel */
       data[chan]=((unsigned char) (list_in>>(chan*CO_ADC_SZ/4))&255);   
     }
}


/***************************************************************************************

	get_adc1_one_costar(data,costar_nb,map)
	
	float 		data[number of adc channel (4)]
	unsigned short  costar_nb
	chip  		map[number of chip]
	
	return : nothing 
	
	Description : 	this function gets the adc1 register of the costar "costar_nb" and puts the
			floating point values in the array "data". the conversion into floating 
			point value is done according to the type of data (temperature,voltage
			supply, etc..) 
	
	note	    :	data[0] : external channel 	(V)
			data[1] : GND - VSS 		(V)
			data[2] : VDD - VSS 		(V)
			data[3] : temperature		(C)
			

******************************************************************************************/ 
void get_adc1_one_costar(unsigned char data[4],unsigned short costar_nb,chip *map)
{
    unsigned long list_in;
    unsigned short nb_costar,chan;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    list_in = get_one_costar(CO_ADC_1,CO_ADC_SZ,costar_nb,map);
    for (chan=0;chan<4;chan++){
      data[chan]=((unsigned char) (list_in>>(chan*CO_ADC_SZ/4))&255);
     }
}

void get_adc1_one_costar_debug(unsigned char data[4],unsigned short costar_nb,chip *map)
{
    unsigned long list_in,c;
    unsigned short nb_costar,chan;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
    for (chan=0;chan<4;chan++)
    {
       send_one_costar(CO_ADC_TEST_1,CO_ADC_TEST_SZ,8+chan,costar_nb,map); /* select the channel */
       
       start_conversion_one_costar(costar_nb,map);
       for(c=0;c<100000;c++);/* wait for 10 clock pulses ? */
       stop_conversion_one_costar(costar_nb,map);
       
       list_in=get_one_costar(CO_ADC_1,CO_ADC_SZ,costar_nb,map);                     /* get the datas from that channel */
       data[chan]=((unsigned char) (list_in>>(chan*CO_ADC_SZ/4))&255);   
     }
}


/***************************************************************************************

	get_adc0_some_costar(data,list_chip,map)
	
	float 		data[number of costar][number of adc channel (4)]
	char		list_chip[number of chip]
	chip  		map[number of chip]
	
	return : nothing 
	
	Description : 	this function gets the adc0 register of some of the costars 
			(list_chip tells which chip have to be read) and puts the
			raw adc values in the array "data" (map describes the chain).
			Caution : This function only works with 2nd generation COSTARs !				

******************************************************************************************/ 
void get_adc0_some_costar(unsigned char data[][4],char *list_chip,chip *map)
{
    unsigned short list_in[MAX_LENGTH_STREAM];
    unsigned short nb_costar,i,sh,chan;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
       
    get_some_costar(CO_ADC_0,CO_ADC_SZ,list_in,list_chip,map);                     /* get the datas */
    
    for(i=0,sh=0;i<nb_costar;i++)						
    {
      if(list_chip[i]==0)
      {
         for (chan=0;chan<4;chan++)
	    data[i][chan]=0xff;
	 sh++;
      }
      else
        for (chan=0;chan<4;chan++)
	   data[i][chan] = (unsigned char) get_stream(list_in,(i-sh)*CO_ADC_SZ+chan*(CO_ADC_SZ/4),CO_ADC_SZ/4);
    }

}

/***************************************************************************************

	get_adc1_some_costar(data,list_chip,map)
	
	float 		data[number of costar][number of adc channel (4)]
	char		list_chip[number of chip]
	chip  		map[number of chip]
	
	return : nothing 
	
	Description : 	this function gets the adc0 register of some of the costars 
			(list_chip tells which chip have to be read) and puts the
			raw adc values in the array "data" (map describes the chain).
			Caution : This function only works with 2nd generation COSTARs !				

******************************************************************************************/ 
void get_adc1_some_costar(unsigned char data[][4],char *list_chip,chip *map)
{
    unsigned short list_in[MAX_LENGTH_STREAM];
    unsigned short nb_costar,i,sh,chan;
    
    if ((nb_costar = map_costar_nb(map))==0) return ;
       
    get_some_costar(CO_ADC_1,CO_ADC_SZ,list_in,list_chip,map);                     /* get the datas */
    
    for(i=0,sh=0;i<nb_costar;i++)						
    {
      if(list_chip[i]==0)
      {
         for (chan=0;chan<4;chan++)
	    data[i][chan]=0xff;
	 sh++;
      }
      else
        for (chan=0;chan<4;chan++)
	   data[i][chan] = (unsigned char) get_stream(list_in,(i-sh)*CO_ADC_SZ+chan*(CO_ADC_SZ/4),CO_ADC_SZ/4);
    }

}
